# -*- coding: utf-8 -*-
__license__ = """Copyright (c) 2008-2010, Toni Ruža, All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS IS'
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE."""

__author__ = u"Toni Ruža <gmr.gaf@gmail.com>"
__url__  = "http://bitbucket.org/raz/wxautocompletectrl"


import wx
import re
import fixRef, validateCount
#import win32api, win32gui

CS_DROPSHADOW = 0x00020000

class SuggestionsPopup(wx.MiniFrame):
    def __init__(self, parent):
        wx.MiniFrame.__init__(self, parent, style=wx.BORDER_RAISED|wx.STAY_ON_TOP|wx.FRAME_NO_TASKBAR)
        self._suggestions = self._listbox(self)
        #self._suggestions.SetSelectionBackground(wx.Colour(180, 40, 40))
        self._suggestions.SetItemCount(0)
        self._unformated_suggestions = None
        
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self._suggestions, 1, wx.EXPAND)
        self.SetSizer(sizer)
        self.Layout()
    
        #Add dropshadow
        #GCL_STYLE= -26
        #cstyle= win32gui.GetClassLong(self.GetHandle(), GCL_STYLE)
        #if cstyle & CS_DROPSHADOW == 0:
        #    win32api.SetClassLong(self.GetHandle(), GCL_STYLE, cstyle | CS_DROPSHADOW)


    class _listbox(wx.HtmlListBox):
        items = None
        vsLstLen = 0
        
        def OnGetItem(self, n):
            return self.items[n]
        
        def OnDrawSeparator(self, dc, rect, index):
            if index != self.vsLstLen:
                return
            oldpen = dc.GetPen()
            dc.SetPen(wx.Pen(wx.Colour(150,150,250)))
            dc.DrawLine(rect.x, rect.y,
                        rect.x + rect.width -2,
                        rect.y)
            rect.Deflate(0, 2)
            dc.SetPen(oldpen)

        def OnDrawBackground(self, dc, rect, item):
            if item > self.vsLstLen and item % 2 == 0:
                bgCol = wx.Colour(240,240,255)
                dc.SetBrush(wx.Brush(bgCol))
                dc.SetPen(wx.Pen(bgCol))
                dc.DrawRectangleRect(rect) 
            elif item % 2 == 0:
                bgCol = wx.Colour(255,240,240)
                dc.SetBrush(wx.Brush(bgCol))
                dc.SetPen(wx.Pen(bgCol))
                dc.DrawRectangleRect(rect) 
        
            
    def SetSuggestions(self, suggestions, unformated_suggestions, vsLstLen, wdLstLen):
        #print wdLstLen, 'wdLstLen'
        self._suggestions.SetSize((200, wdLstLen*20))
        self.SetClientSize(self._suggestions.GetSize())
        self._suggestions.vsLstLen = vsLstLen
        self._suggestions.items = suggestions
        self._suggestions.SetItemCount(len(suggestions))
        self._suggestions.SetSelection(0)
        self._suggestions.Refresh()
        self._unformated_suggestions = unformated_suggestions

    def CursorUp(self):
        selection = self._suggestions.GetSelection()
        if selection > 0:
            self._suggestions.SetSelection(selection - 1)

    def CursorDown(self):
        selection = self._suggestions.GetSelection()
        last = self._suggestions.GetItemCount() - 1
        if selection < last:
            self._suggestions.SetSelection(selection + 1)

    def CursorHome(self):
        if self.IsShown():
            self._suggestions.SetSelection(0)

    def CursorEnd(self):
        if self.IsShown():
            self._suggestions.SetSelection(self._suggestions.GetItemCount() - 1)

    def GetSelectedSuggestion(self):
        return self._unformated_suggestions[self._suggestions.GetSelection()]

    def GetSuggestion(self, n):
        return self._unformated_suggestions[n]


class AutoTextCtrl(wx.TextCtrl):
    def __init__(self, parent, size, height=300, vsCompleter=None, wdCompleter=None, multiline=False, frequency=250):
        
        self.mainApp = wx.GetApp().GetTopWindow()
        self.vsCount = validateCount.vsCount
        self.chCount = validateCount.chCount
        
        style = wx.TE_PROCESS_ENTER
        if multiline:
            style = style | wx.TE_MULTILINE
        
        wx.TextCtrl.__init__(self, parent, value='Enter Word or Ref.', size=size, style=style)
        self.height = height
        self.frequency = frequency
        
        if vsCompleter:
            self.vsCompleter = vsCompleter
        if wdCompleter:
            self.SetCompleter(wdCompleter)
            
        self.queued_popup = False
        self.skip_event = False

    def validate(self, ref):
        def check(mo):
            bk = mo.group(1)
            ch = mo.group(2)
            ch = re.sub(r'^0+(\d+)', r'\1', ch)
            vs = mo.group(3)
            
            if vs:
                vs = re.sub(r'^0+(\d+)', r'\1', vs)
                vs = re.sub(r'^0+', r'', vs)
            if not vs:
                vs = ''
            if re.search(r'^0+', ch): 
                ch = 1
            if not int(ch) <= self.chCount[bk]:
                #print mo.group(0), 'Bad chap.'
                return '%s %s:%s' % (bk, self.chCount[bk], vs)
            
            if vs and not int(vs) <= self.vsCount['%s %s:' % (bk, ch)]:
                #print mo.group(0), 'Bad verse', self.vsCount['%s %s:' % (bk, ch)], 'max'
                return '%s %s:%s' % (bk, ch, self.vsCount['%s %s:' % (bk, ch)])
            
            return mo.group(0)
        
        vRef = re.sub(r'(\w\w\w) (\d+):(\d+)?', check, ref)
        vRef = re.sub(r'(\w\w\w) (\d+):(\d+)', check, vRef)
        return vRef
    

    def SetCompleter(self, completer):
        """
        Initializes the autocompletion. The 'completer' has to be a function
        with one argument (the current value of the control, ie. the query)
        and it has to return two lists: formated (html) and unformated
        suggestions.
        """
        self.completer = completer

        frame = self.Parent
        while not isinstance(frame, wx.Frame):
            frame = frame.Parent

        self.popup = SuggestionsPopup(frame)

        frame.Bind(wx.EVT_MOVE, self.OnMove)
        self.Bind(wx.EVT_TEXT, self.OnTextUpdate)
        self.Bind(wx.EVT_SIZE, self.OnSizeChange)
        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        #self.Bind(wx.EVT_SET_FOCUS, self.OnFocus)
        self.popup._suggestions.Bind(wx.EVT_LEFT_UP, self.OnSuggestionClicked)
        self.popup._suggestions.Bind(wx.EVT_KEY_DOWN, self.OnSuggestionKeyDown)

    def AdjustPopupPosition(self):
        self.popup.Position = self.ClientToScreen((0, self.Size.height)).Get()

    def OnFocus(self, event):
        wx.CallAfter(self.SelectAll)
        self.skip_event = False

    def OnMove(self, event):
        self.AdjustPopupPosition()
        event.Skip()

    def OnTextUpdate(self, event):
        event.Skip()
        
        if not self.IsModified():
            return
        
        if re.search(r'^0+', self.Value):
            return
                            
        self.preForm = ''
        self.preUnform = ''
        
        hit1 = re.search(r'^ ?(\d+[.:]\d+)', self.Value)
        hit2 = re.search(r'^ ?(\d+[.:])(?! ?\D)', self.Value)
        hit3 = re.search(r'^ ?(\d+)(?![.:])(?! ?\D)', self.Value)
        
        fullBook = fixRef.fullTitle(self.mainApp.currentChap[:3])
        
        if hit1:
            #print 'hit1'
            ref = hit1.group(1).replace('.', ':')
            valRef = self.validate(u"%s %s" % (self.mainApp.currentChap[:3], ref))
            valRef = re.search(r'(\d+:\d+)', valRef).group(1)
            self.preForm = u'<font color="brown"><b>%s</b></font> %s' % (fullBook, valRef)
            self.preUnform = u"%s %s" % (self.mainApp.currentChap[:3], valRef)
        
        elif hit2:
            #print 'hit2'
            ref = hit2.group(1).replace('.', ':')
            valRef = self.validate(u"%s %s" % (self.mainApp.currentChap[:3], ref))
            valCh = re.search(r'(\d+):', valRef).group(1)

            self.preForm = u'<font color="brown"><b>%s</b></font> %s' % (fullBook, valCh)
            self.preUnform = u"%s %s" % (self.mainApp.currentChap[:3], valCh)

        elif hit3:
            #print 'hit3'
            vs = hit3.group(1)
            valRef = self.validate(u"%s%s" % (self.mainApp.currentChap, hit3.group(1)))
            valVs = re.search(r':(\d+)', valRef).group(1)
            
            self.preForm = u'<font color="brown"><b>%s</b></font>%s%s' % (fullBook, self.mainApp.currentChap[3:], valVs)
            self.preUnform = u"%s %s" % (self.mainApp.currentChap, valVs)

        if self.skip_event:
            self.skip_event = False
            return
        
        if not self.queued_popup:
            wx.CallLater(self.frequency, self.AutoComplete)
            self.queued_popup = True
        


    def AutoComplete(self):
        self.queued_popup = False
        if self.Value != "":
            value = self.Value.lower()
            
            refValue = re.sub(r'([123])([skctpj])', r'\1 \2', value, re.I)
            
            try:
                if re.match('\d', refValue) and len(refValue) > 1:
                    cap = refValue[2].upper()
                    refValue = refValue[:2] + cap + refValue[3:]
                else:
                    refValue =refValue.capitalize()
            except:
                refValue =refValue.capitalize()
                
            vsLstLen = 0
            
            num = None
            dig = re.search(r'\w ?(\d+[.:]?(?:\d+)?)', refValue)
            if dig:
                num = dig.group(1)
                num = re.sub(r'^0+', '1', num)
                num = re.sub(r'^0+(\d+)', r'\1', num)
                num = num.replace(".", ":")
                if not ':' in num:
                    num = num + ':'
                refValue = re.sub(r'(\w) ?\d+[.:]?(?:\d+)?', r'\1',  refValue)
            
            vsFormated, vsUnformated = self.vsCompleter(refValue)
            
            if num:
                numList = []
                for ref in vsUnformated:
                    valRef = self.validate(u"%s %s" % (fixRef.refMatch(ref), num))
                    valNum = re.match(r'\w\w\w (.+)', valRef).group(1)
                    numList.append(valNum)
                                
                vsFormated = ['%s %s' % (v, numList[id]) for id, v in enumerate(vsFormated)]
                vsUnformated = ['%s %s' % (v, numList[id]) for id, v in enumerate(vsUnformated)]
                
            formated, unformated = self.completer(value)
            if len(vsFormated) > 0:
                vsLstLen = len(vsFormated)
                formated = vsFormated[:len(vsFormated)] + formated
                unformated = vsUnformated[:len(vsUnformated)] + unformated
                        
            if self.preForm:
                formated.insert(0, self.preForm)
                unformated.insert(0, self.preUnform)
                vsLstLen +=1
            
            if len(unformated) > 18:
                formated = formated[:18]
                unformated = unformated[:18]
                
            if len(formated) > 0:
                self.popup.SetSuggestions(formated, unformated, vsLstLen, len(unformated))
                self.AdjustPopupPosition()
                self.Unbind(wx.EVT_KILL_FOCUS)
                self.popup.Show()
                self.SetFocus()
                wx.CallAfter(self.Bind, wx.EVT_KILL_FOCUS, self.OnKillFocus)
                wx.CallAfter(self.mainApp.Raise)
                
            else:
                self.popup.Hide()
        else:
            self.popup.Hide()

    def OnSizeChange(self, event):
        self.popup.Size = (self.Size[0], self.height)
        event.Skip()

    def OnKeyDown(self, event):
        key = event.GetKeyCode()

        if key == wx.WXK_UP:
            self.popup.CursorUp()
            return

        elif key == wx.WXK_DOWN:
            self.popup.CursorDown()
            return

        elif key in (wx.WXK_RETURN, wx.WXK_NUMPAD_ENTER) and self.popup.Shown:
            self.skip_event = True
            self.SetValue(self.popup.GetSelectedSuggestion())
            self.SetInsertionPointEnd()
            self.popup.Hide()
            self.mainApp.OnSearch(None)
            return

        elif key == wx.WXK_HOME:
            self.popup.CursorHome()

        elif key == wx.WXK_END:
            self.popup.CursorEnd()

        elif event.ControlDown() and unichr(key).lower() == "a":
            self.SelectAll()

        elif key == wx.WXK_ESCAPE:
            self.popup.Hide()
            return

        event.Skip()

    def OnSuggestionClicked(self, event):
        self.skip_event = True
        n = self.popup._suggestions.HitTest(event.Position)
        self.Value = self.popup.GetSuggestion(n)
        self.SetInsertionPointEnd()
        self.popup.Hide()
        self.mainApp.OnSearch(None)
        event.Skip()

    def OnSuggestionKeyDown(self, event):
        key = event.GetKeyCode()
        if key in (wx.WXK_RETURN, wx.WXK_NUMPAD_ENTER):
            self.skip_event = True
            self.SetValue(self.popup.GetSelectedSuggestion())
            self.SetInsertionPointEnd()
            self.popup.Hide()
            self.mainApp.OnSearch(None)
        event.Skip()

    def OnKillFocus(self, event):
        if not self.popup.IsActive():
            self.popup.Hide()
        event.Skip()

