import wx, re, os, wx.html, time
import contextMenu, editor, baseClass
#import wx.lib.buttons as buttons
import flatNotebook as fnb
import sqlite3 as sqlite



class DevotionWin(wx.MiniFrame):
    def __init__(self, parent):
        wx.MiniFrame.__init__(self, parent, size=(640, 480), style = wx.DEFAULT_FRAME_STYLE|wx.CLIP_CHILDREN)
        self.mainApp = parent
        
        self.panel = wx.Panel(self, -1)
        calPanel = wx.Panel(self, -1)
        
        self.NBdev = fnb.FlatNotebook(self.panel, -1, panel=0, style=fnb.FNB_FANCY_TABS|fnb.FNB_NODRAG|fnb.FNB_DROPDOWN_TABS_LIST|fnb.FNB_NO_NAV_BUTTONS|fnb.FNB_NO_X_BUTTON)
        
        close = wx.Button(calPanel, -1, ' Close') #, size=(40, -1))
        self.dpc = wx.DatePickerCtrl(calPanel, size=(120,-1), style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        
        self.SetTitle(' Bible Analyzer Daily Devotional')
        #calPanel.SetBackgroundColour('White')
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer2 = wx.BoxSizer(wx.VERTICAL)
        calanderSizer = wx.BoxSizer(wx.HORIZONTAL)
        
        sizer2.Add(self.NBdev, 1, wx.EXPAND|wx.ALL, 7)
        calanderSizer.Add(self.dpc, 0, wx.BOTTOM|wx.LEFT, 7)
        calanderSizer.Add((-1, -1), 1, 0, 0)
        calanderSizer.Add(close, 0, wx.ALIGN_RIGHT|wx.BOTTOM|wx.RIGHT, 7)
        self.panel.SetSizer(sizer2)
        calPanel.SetSizer(calanderSizer)
        sizer.Add(self.panel, 1, wx.EXPAND, 0)
        sizer.Add(calPanel, 0, wx.EXPAND, 0)
        self.SetSizer(sizer)
        self.Layout()
        
        self.Bind(wx.EVT_DATE_CHANGED, self.OnNewDate, self.dpc)
        self.Bind(wx.EVT_BUTTON, self.OnCloseWindow, close)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)
        

        ## Prayer List Editor

        self.prayerEdit = editor.Editor(self, 3)

        self.prayerEdit.SetToolTip(wx.ToolTip('Prayer List Editor'))        

        self.NBdev.AddPage(self.prayerEdit, 'Prayer List', True)

        self.prayerEdit = self.prayerEdit.editor

        self.prayerEdit.DiscardEdits()

        



        self.devAbbrList = []
        self.devDescList = []
        self.devFileList = []
        self.devDisplay = []
        
        
        for set in self.mainApp.devData:
            self.devAbbrList.append(set[0])
            self.devDescList.append(set[1])
            fn = set[2]
            self.devFileList.append(fn)
   
        for title in self.devAbbrList:
            self.NBdev.AddPage(self.CreatePage(self.devDescList[self.devAbbrList.index(title)]), title, True)
            
        current = time.localtime(time.time())
        month = time.strftime("%m", current)
        day = time.strftime("%d", current)
        if month[0] == str(0):
            month = month[1]
        if day[0] == str(0):
            day = day[1]
        for devFN in self.devFileList:
            self.OnView(devFN, month, day, self.devFileList.index(devFN))
            
        self.NBdev.SetSelection(0)
        self.NBdev.SetFocus()

        #self.panel.Bind(wx.EVT_PAINT, self.OnPaint)
        #self.panel.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)

        # Make/Load Prayer List File
        if os.access(self.mainApp.userDir + '/prayer.xml', 1):
            self.prayerEdit.LoadFile(self.mainApp.userDir + '/prayer.xml', 2)
        
    
    def OnPaint(self, event):
        dc = wx.BufferedPaintDC(self.panel)
        clientRect = self.panel.GetClientRect()
        startColor = wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE)
        endColor = wx.Colour(255, 255, 255)
        ArtManager.Get().PaintStraightGradientBox(dc, clientRect, startColor, endColor, True)
    
    def OnEraseBackground(self, event):
        pass
        
        
        
    def CreatePage(self, title, answer = ''):
        p = wx.Panel(self.NBdev)
        box = wx.StaticBox(p, -1, title)
        display = baseClass.HtmlWindow(p, -1, style=wx.BORDER_THEME)
        self.devDisplay.append(display)
        
        sizer = wx.StaticBoxSizer(box, wx.HORIZONTAL)
        sizer.Add(display, 1, wx.EXPAND, 0)
        display.Bind(wx.EVT_MOTION, self.mainApp.OnMotion)
        display.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        display.Bind(wx.EVT_CHAR, self.OnChar)
        display.Bind(wx.html.EVT_HTML_CELL_CLICKED, self.OnClick)
        display.Bind(wx.EVT_LEAVE_WINDOW, self.mainApp.leaveWindow)
        display.SetPage(answer)
        p.SetSizer(sizer)
        return p

    
    def OnView(self, fName, month, day, idx):
        if day[0] == str(0):
            day = day[1]
        selStr = 'SELECT devotion FROM devotion WHERE month LIKE \'%s\' and day LIKE \'%s\'' % (month, day)
        con = sqlite.connect(fName)
        con.text_factory = str
        
        cur = con.cursor()
        cur.execute(selStr)
        row = cur.fetchone()
        
        try:
            devTxt = unicode(row[0], "utf-8")
        except UnicodeError:
            devTxt = unicode(row[0], "latin1")
            print 'Devotion, latin1 encoding'
        except:
            return


        if devTxt:
            self.devDisplay[idx].SetPage('<font face="%s">%s</font>' % (self.mainApp.font, devTxt))



    def OnNewDate(self, event):
        new = event.GetDate()
        month = new.Format('%m')
        day = new.Format('%d')
        if month[0] == str(0):
            month = month[1]
        for devFN in self.devFileList:
            self.OnView(devFN, month, day, self.devFileList.index(devFN))


    def OnClick(self, event):
        event.Skip()
        self.SetFocus()
        cell = event.GetCell()
        select = cell.ConvertToText(None)
        
        mouseButton = event.GetMouseEvent().GetButton()
        if mouseButton == 1:
            if '_' in select:
                self.GetParent().ShowRefContext(select)
            else:
                return
        if mouseButton == 3:
            tab = self.NBdev.GetSelection()
            self.devDisplay[tab -1].SetFocus()
            conMenu = contextMenu.ContextMenu(self) 
            conMenu.MakeShortMenu(select)

    
    def OnKeyDown(self, event):
        event.Skip()
        keycode = event.GetKeyCode()
    
    
    def OnChar(self, event):
        keycode = event.GetKeyCode()
        if keycode == 3:
            self.OnCopy(event)
        elif keycode == 1:
            self.OnSelectAll(event)


    def OnSelectAll(self, event):
        focus = self.FindFocus()
        focus.SelectAll()
    
    def OnCopy(self, event):
        try:
            page = self.FindFocus()
            self.do = wx.TextDataObject()
            self.do.SetText(page.SelectionToText())
        except:
            wx.MessageBox("Nothing selected to copy!", "Notice!")
            return
        if wx.TheClipboard.Open():
            wx.TheClipboard.SetData(self.do)
            wx.TheClipboard.Close()
        else:
            wx.MessageBox("Unable to open the clipboard", "Error")

    
    def OnCloseWindow(self, event):
        self.prayerEdit.SaveFile(self.mainApp.userDir + '/prayer.xml', 2)
        self.Hide()
        #self.Destroy()
        #self.mainApp.Refresh()
        
