import wx, wx.html, cStringIO, re, os, sys
import toolImg, fixRef
import wx.richtext as rt
import agw.aui as Aui
import PyRTFParser


books = [
    'Gen', 'Exo', 'Lev', 'Num', 'Deu', 'Jos', 'Jdg', 'Rth', '1Sa', '2Sa', '1Ki', '2Ki', '1Ch', '2Ch',
    'Ezr', 'Neh', 'Est', 'Job', 'Psa', 'Pro', 'Ecc', 'Son', 'Isa', 'Jer', 'Lam', 'Eze', 'Dan', 'Hos',
    'Joe', 'Amo', 'Oba', 'Jon', 'Mic', 'Nah', 'Hab', 'Zep', 'Hag', 'Zec', 'Mal', 'Mat', 'Mar', 'Luk',
    'Joh', 'Act', 'Rom', '1Co', '2Co', 'Gal', 'Eph', 'Phi', 'Col', '1Th', '2Th', '1Ti', '2Ti', 'Tit',
    'Phm', 'Heb', 'Jam', '1Pe', '2Pe', '1Jo', '2Jo', '3Jo', 'Jud', 'Rev']



class Editor(wx.Panel):
    def __init__(self, parent, type):
        wx.Panel.__init__(self, parent)
        
        self.mainApp = wx.GetApp().GetTopWindow()
        self.rtPrint = rt.RichTextPrinting()
        
        saveImg = toolImg.getEditSaveBitmap()
        copyImg = toolImg.getCopyBitmap()
        pasteImg = toolImg.getEditPasteBitmap()
        undoImg = toolImg.getUndoBitmap()
        boldImg = toolImg.getEditBoldBitmap()
        italicImg = toolImg.getEditItalicBitmap()
        underlineImg = toolImg.getEditUnderlineBitmap()
        leftImg = toolImg.getEditAlignleftBitmap()
        centerImg = toolImg.getEditCentreBitmap()
        rightImg = toolImg.getEditAlignrightBitmap()
        fontImg = toolImg.getEditFontBitmap()
        
        printImg = toolImg.getPrinterBitmap()
        
        linkImg = toolImg.getWebLinkBitmap()
        hiLiteImg = toolImg.getHiLiteBitmap()
        
        linkAllImg = toolImg.getLinkAllBitmap()
        openImg = toolImg.getOpenBitmap()
        newImg = toolImg.getNewBitmap()
        
        #openImg = wx.Image('folder-open.png', wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        
        self.editor = rt.RichTextCtrl(self, style=wx.BORDER_NONE|wx.WANTS_CHARS)
        
        tbPanel = wx.Panel(self)
        self._mgr = Aui.AuiManager(tbPanel)

        prepend_items, append_items = [], []
        item = Aui.AuiToolBarItem()
        item.SetKind(wx.ITEM_SEPARATOR)
        append_items.append(item)
        
        
        self.miniTB = Aui.AuiToolBar(tbPanel, -1, wx.DefaultPosition, wx.DefaultSize, type = 1, agwStyle = Aui.AUI_TB_OVERFLOW|Aui.AUI_TB_PLAIN_BACKGROUND)

        if type == 1:
            self.miniTB.AddSimpleTool(917, "New", newImg, "New Document")
            self.miniTB.AddSimpleTool(915, "Open", openImg, "Open File")
            self.miniTB.AddSimpleTool(913, "Save", saveImg, "Save File")
            #self.miniTB.AddSpacer(2)

        self.miniTB.AddSimpleTool(914, "Copy", copyImg, "Copy Selected")
        self.miniTB.AddSimpleTool(901, "Paste", pasteImg, "Paste")
        self.miniTB.AddSimpleTool(902, "Print", printImg, "Print Window")
        #self.miniTB.AddSpacer(2)
        #self.miniTB.AddSimpleTool(903, "Undo", undoImg, "Undo")
        self.miniTB.AddSpacer(2)
        self.miniTB.AddSimpleTool(904, "Bold", boldImg, 'Bold')
        self.miniTB.AddSimpleTool(905, "Italic", italicImg, 'Italic')
        self.miniTB.AddSimpleTool(906, "Underline", underlineImg, 'Underline')
        self.miniTB.AddSpacer(2)
        self.miniTB.AddSimpleTool(907, "Left", leftImg, 'Align left')
        self.miniTB.AddSimpleTool(908, "Center", centerImg, 'Align center')        
        self.miniTB.AddSimpleTool(909, "Right", rightImg, 'Align right')
        self.miniTB.AddSpacer(2)
        self.miniTB.AddSimpleTool(910, "Font", fontImg, "Font Properties")
        
        if type != 3:
            self.miniTB.AddSimpleTool(911, "Highlite", hiLiteImg, "Highlight Selected Text")
            self.miniTB.AddSimpleTool(912, "Link", linkImg, "Make Selection a Link (Ctrl to Clear)")
            self.miniTB.AddSimpleTool(916, "Link All", linkAllImg, "Link all References on Page (Ctrl to Clear)")
        
        self.miniTB.SetCustomOverflowItems(prepend_items, append_items)        
        self.miniTB.Realize()
        
        info = Aui.AuiPaneInfo().Name("Editor TB").CaptionVisible(False).Top().DockFixed()
        self._mgr.AddPane(self.miniTB, info)
        
        self._mgr.GetArtProvider().SetMetric(Aui.AUI_DOCKART_PANE_BORDER_SIZE, 0)
        
        self._mgr.Update()
        self.miniTB.Realize()
        
        mainEditorSizer = wx.BoxSizer(wx.VERTICAL)
        mainEditorSizer.Add(tbPanel, 0, wx.EXPAND, 0)
        mainEditorSizer.Add(self.editor, 1, wx.ALL|wx.EXPAND, 1)
        
        self.SetSizer(mainEditorSizer)

        
        
        if type == 1:
            self.Bind(wx.EVT_TOOL, self.OnFileSave, id=913)
            self.Bind(wx.EVT_TOOL, self.OnFileOpen, id=915)
            self.Bind(wx.EVT_TOOL, self.OnFileClose, id=917)
            self.editor.Bind(wx.EVT_KEY_DOWN, self.OnKey)

        self.Bind(wx.EVT_TOOL, self.OnEditorCopy, id=914)
        self.Bind(wx.EVT_TOOL, self.OnEditorPaste, id=901)
        
        #self.Bind(wx.EVT_TOOL, self.OnUndo, id=903)
        
        self.Bind(wx.EVT_TOOL, self.OnBold, id=904)
        self.Bind(wx.EVT_TOOL, self.OnItalic, id=905)
        self.Bind(wx.EVT_TOOL, self.OnUnderline, id=906)
        self.Bind(wx.EVT_TOOL, self.OnAlignLeft, id=907)
        self.Bind(wx.EVT_TOOL, self.OnAlignRight, id=909)
        self.Bind(wx.EVT_TOOL, self.OnAlignCenter, id=908)
        self.Bind(wx.EVT_TOOL, self.OnEditorPrint, id=902)
        self.Bind(wx.EVT_TOOL, self.OnFont, id=910)
        
        self.editor.Bind(wx.EVT_MENU, self.OnCutCopy, id=wx.ID_CUT)
        self.editor.Bind(wx.EVT_MENU, self.OnCutCopy, id=wx.ID_COPY)
        self.editor.Bind(wx.EVT_MENU, self.OnPaste, id=wx.ID_PASTE)        
        
        if type != 3:
            self.Bind(wx.EVT_TOOL, self.OnHiLite, id=911)
            self.Bind(wx.EVT_TOOL, self.OnMakePageLinks, id=916)    
            self.Bind(wx.EVT_TOOL, self.OnURL, id=912)
        
        self.editor.Bind(wx.EVT_TEXT_URL, self.OnURLClick)
        
        self.editor.Bind(wx.EVT_MOTION, self.OnOverLink)
        
        
        self.path = ''
        self.currentFilePath = None
        
        self.BGColorList = {
        'white':wx.Colour(255, 255, 255),
        'gray':wx.Colour(220, 220, 220), 
        'yellow':wx.Colour(255, 255, 190), 
        'blue':wx.Colour(191, 231, 255),
        'green':wx.Colour(180, 238, 180),
        'red':wx.Colour(255, 200, 200),
        }
        
        self.BGColor = self.BGColorList['yellow']
        
    
    
    def OnHiLite(self, event):
        rng = self.editor.GetSelection()
        
        txt = self.editor.GetStringSelection()
        if not txt:
            return
        
        self.editor.Delete(rng)
        self.editor.SetInsertionPoint(rng[0])
        
        style = rt.TextAttrEx()
        self.editor.GetStyle(rng[0], style)        
        style.SetBackgroundColour(self.BGColor)
        
        self.editor.BeginStyle(style)
        self.editor.WriteText(txt)
        self.editor.EndStyle();
        self.editor.MarkDirty()
        
        if self.editor == self.mainApp.editor:
            self.mainApp.resNB.SetPageImage(0, 0)
        
    
    def OnGrayBG(self, event):
        self.BGColor = self.BGColorList['gray']
    
    def OnYellowBG(self, event):
        self.BGColor = self.BGColorList['yellow']
    
    def OnBlueBG(self, event):
        self.BGColor = self.BGColorList['blue']
        
    def OnWhiteBG(self, event):
        self.BGColor = self.BGColorList['white']

    def OnGreenBG(self, event):
        self.BGColor = self.BGColorList['green']

    def OnRedBG(self, event):
        self.BGColor = self.BGColorList['red']


    def OnOverLink(self, event):
        event.Skip()
        pos = event.GetPosition()
        hx, hy = self.editor.HitTest(pos)
        style = rt.TextAttrEx()
        self.editor.GetStyle(hy, style)
        if style.GetURL() and hx == 0:
            url = style.GetURL()
            #print url
            if re.search(r'\d:\d', url):
                x, y = wx.GetMousePosition()
                self.mainApp.GetPopup(x, y, url, 1)
        else:
            try:
                self.mainApp.toolwin.Destroy()
                self.mainApp.popRef = ''
            except:
                pass


    def OnURL(self, event):
        rng = self.editor.GetSelection() 
        
        if wx.GetKeyState(wx.WXK_CONTROL):
            self.OnRemoveLink(rng)
            return
        
        txt = self.editor.GetStringSelection()
        if not txt:
            return
        
        dlg = wx.TextEntryDialog(self, 'Web URL ("http://www.BibleAnalyzer.com") or\nVerse ("Joh 3:16"):',
        'Enter URL or Verse:', '')

        fix = re.sub(r'\b\w+?[ _]\d+[.:]\d+(-\d+)?', self.fxRef, txt)
        dlg.SetValue(fix.replace('_', ''))

        if dlg.ShowModal() == wx.ID_OK and dlg.GetValue() != '':
            url = dlg.GetValue()
        else:
            return
        
        dlg.Destroy()
        
        tempStyle = rt.RichTextAttr()
        self.editor.GetStyle(self.editor.GetInsertionPoint(), tempStyle)
        defaultFont = tempStyle.GetFont()
        fSize = tempStyle.GetFont().GetPointSize()
        fFace = tempStyle.GetFont().GetFaceName()

        urlStyle = rt.RichTextAttr()
        if re.search(r'\d:\d', url):
            urlStyle.SetTextColour(wx.Colour(51, 102, 102))
        else:
            urlStyle.SetTextColour(wx.BLUE)
        
        urlStyle.SetFontUnderlined(True)
        urlStyle.SetURL(url)
        urlStyle.SetFontSize(fSize)
        urlStyle.SetFontFaceName(fFace)
        
        if rng[1] == self.editor.GetLastPosition():
            self.editor.SetInsertionPoint(rng[1])
            self.editor.WriteText(' ')

        self.editor.SetStyle(rng, urlStyle)

        attr = rt.RichTextAttr()
        self.editor.SetDefaultStyle(attr)

        self.editor.MarkDirty()
        
        self.mainApp.resNB.SetPageImage(0, 0)
        
        
    
    def OnURLClick(self, event):
        lnk = event.GetString()
        print lnk
        if 'http://' in lnk:
            wx.LaunchDefaultBrowser(lnk)
        elif re.search(r'\d:\d', lnk):
            self.mainApp.ShowRefContext(lnk)


    def OnRemoveLink(self, rng):
        attr = rt.RichTextAttr()
        #attr = self.editor.GetBasicStyle()
        attr.SetFlags(rt.TEXT_ATTR_URL|rt.TEXT_ATTR_FONT_UNDERLINE|rt.TEXT_ATTR_TEXT_COLOUR)
        self.editor.SetStyle(rng, attr)
        self.editor.SetStyleEx(rng, attr, flags = rt.RICHTEXT_SETSTYLE_REMOVE)
        self.editor.MarkDirty()
        self.mainApp.resNB.SetPageImage(0, 0)


    def OnRemoveLinks(self, event):
        buffer = self.editor.GetBuffer()
        xmlHandler = rt.RichTextXMLHandler()
        stream = cStringIO.StringIO()
        if xmlHandler.SaveStream(buffer, stream):
            contents = stream.getvalue()
            #contents = re.sub(r'<text textcolor="#336666" fontunderlined="1" url="[^>]+>([^<]+)', r'<text>\1', contents) 
            contents = re.sub(r'textcolor="#336666"|fontunderlined="1"', r'', contents)
            contents = re.sub(r'url="[^h][^>]+', r'', contents)
            output = cStringIO.StringIO(contents)
            xmlHandler.LoadStream(buffer, output)
            self.editor.Refresh()
            self.editor.MarkDirty()
            self.mainApp.resNB.SetPageImage(0, 0)
    
    
    def OnMakePageLinks(self, event):
        ctrlDown = False 
        if wx.GetKeyState(wx.WXK_CONTROL):
            ctrlDown = True 

        if self.editor == self.mainApp.editor and self.editor.IsModified():
            self.preSaveCheck()
        
        self.OnRemoveLinks(None)
        if ctrlDown == True: 
            return
        
        buffer = self.editor.GetBuffer()
        xmlHandler = rt.RichTextXMLHandler()
        stream = cStringIO.StringIO()
        if xmlHandler.SaveStream(buffer, stream):
            contents = stream.getvalue()
            
            fixed = re.sub(r'\b\w+?\.?[ _]\d+[.:]\d+(-\d+)?', self.fxRefs, contents) 
            fixed = re.sub(r'<text></text>', '', fixed)
            
            #print fixed

            output = cStringIO.StringIO(fixed)
            xmlHandler.LoadStream(buffer, output)
                
            self.editor.Refresh()
            self.editor.MarkDirty()
            self.mainApp.resNB.SetPageImage(0, 0)

    
    def fxRef(self, match):
        
        fn = self.mainApp.bibFileList[self.mainApp.bibNB.GetSelection()]
        fixed = fixRef.parseRef(match.group(0), fn, 1)
        if fixed and fixed[:3] in books:
            return fixed
        else:
            return match.group(0)
    
    def fxRefs(self, match):
        fn = self.mainApp.bibFileList[self.mainApp.bibNB.GetSelection()]
        fixed = fixRef.parseRef(match.group(0), fn, 1)
        print fixed
        if fixed and fixed[:3] in books:
            return '</text><text textcolor="#336666" fontunderlined="1" url="%s">%s</text><text>' % (fixed, match.group(0))
        else:
            return match.group(0)
        
    def preSaveCheck(self):
        dlg = wx.MessageDialog(self, '  Save ScripturePad Changes?  \n',
                   ' Save ScripturePad?  ', wx.YES_NO)
        if dlg.ShowModal() == wx.ID_YES:
            self.OnFileSave(None)


    def saveCheck(self):
        dlg = wx.MessageDialog(self, '  Save ScripturePad Changes?  \n',
                   ' Save ScripturePad?  ', wx.YES_NO)
        if dlg.ShowModal() == wx.ID_YES:
            self.OnFileSave(None)



    def OnKey(self, event):
        event.Skip()
        self.mainApp.resNB.SetPageImage(0, 0)

    
    def OnBasicStyle(self):
        fontData = wx.Font(10, wx.ROMAN, wx.NORMAL, wx.NORMAL, 0, "Georgia")
        fontData.SetFaceName(self.mainApp.SPFace)
        fontData.SetPointSize(self.mainApp.SPSize)
        defaultStyle = rt.TextAttrEx()
        defaultStyle.SetFont(fontData)
        defaultStyle.SetTextColour(wx.Color(0, 0, 0))
        self.editor.SetBasicStyle(defaultStyle)
        self.editor.SetDefaultStyle(defaultStyle)

    
    def OnFileOpen(self, evt):
        if self.editor.IsModified():
            self.saveCheck()
        
        wildcard, types = rt.RichTextBuffer.GetExtWildcard(save=False)
        wildcard += '|RTF files (*.rtf)|*.rtf|PLL files (*.pll)|*.pll'
        dlg = wx.FileDialog(self, "Choose a filename",
                            wildcard=wildcard,
                            style=wx.OPEN)
        dlg.SetFilterIndex(1)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            if path:
                self.editor.Clear()
                
                if path.endswith('rtf'):
                    handler = PyRTFParser.PyRichTextRTFHandler()
                    handler.LoadFile(self.editor, path)
                elif path.endswith('pll'):
                    self.editor.LoadFile(path, 1)
                else:
                    fileType = types[dlg.GetFilterIndex()]
                    self.editor.LoadFile(path, fileType)
                
                self.currentFilePath = path
                
                #self.mainApp.resNB.SetPageImage(0, -1)
                self.mainApp.lastSaved = self.currentFilePath
                self.mainApp.UpdateSPadCaption(os.path.basename(self.currentFilePath))
                self.mainApp.editor.SetFocus()

        dlg.Destroy()
    
    
    def OnFileClose(self, event):
        if self.editor.IsModified():
            self.saveCheck()
        
        self.OnBasicStyle()
        self.editor.Clear()
        
        self.editor.SetFilename('')
        self.mainApp.resNB.SetPageImage(0, -1)
        self.mainApp.UpdateSPadCaption()
        self.currentFilePath = None

    
    def OnFileSave(self, evt):
        if not self.currentFilePath:
            self.OnFileSaveAs(evt)
            return
        if self.currentFilePath.endswith('pll'):
            self.editor.SaveFile(type = 1)
        elif self.currentFilePath.endswith('rtf'):
            print 'rtf'
            handler = PyRTFParser.PyRichTextRTFHandler()
            handler.SaveFile(self.editor.GetBuffer(), self.currentFilePath)
        else:
            print 'xml'
            self.editor.SaveFile()
        
        self.mainApp.resNB.SetPageImage(0, -1)
        self.mainApp.lastSaved = self.currentFilePath
        #self.mainApp.UpdateSPadCaption(self.editor.GetFilename())
    
    
    def OnFileSaveAs(self, evt):
        wildcard, types = rt.RichTextBuffer.GetExtWildcard(save=True)
        wildcard += '|RTF files (*.rtf)|*.rtf|PLL files (*.pll)|*.pll'
        
        dlg = wx.FileDialog(self, "Choose a filename",
                            wildcard=wildcard,
                            style=wx.SAVE)
        dlg.SetFilterIndex(2)
        if dlg.ShowModal() == wx.ID_OK:
            self.path = dlg.GetPath()
            if self.path:
                if self.path[-3:] == 'rtf':
                    handler = PyRTFParser.PyRichTextRTFHandler()
                    handler.SaveFile(self.editor.GetBuffer(), self.path)
                elif self.path[-3:] == 'pll':
                    self.editor.SaveFile(self.path, 1)
                else:
                    fileType = types[dlg.GetFilterIndex()]
                    ext = rt.RichTextBuffer.FindHandlerByType(fileType).GetExtension()
                    if not self.path.endswith(ext):
                        self.path += '.' + ext
                    self.editor.SaveFile(self.path, fileType)
        
                self.currentFilePath = self.path
                self.mainApp.lastSaved = self.path
                self.mainApp.resNB.SetPageImage(0, -1)
                self.mainApp.UpdateSPadCaption(os.path.basename(self.currentFilePath))
        
        dlg.Destroy()

    
    
    def OnUndo(self, evt):
        self.editor.Undo()

    def OnRedo(self, evt):
        self.editor.Redo()

    def OnBold(self, evt):
        self.editor.ApplyBoldToSelection()
        
    def OnItalic(self, evt): 
        self.editor.ApplyItalicToSelection()
        
    def OnUnderline(self, evt):
        self.editor.ApplyUnderlineToSelection()
        
    def OnAlignLeft(self, evt):
        self.editor.ApplyAlignmentToSelection(rt.TEXT_ALIGNMENT_LEFT)
        
    def OnAlignRight(self, evt):
        self.editor.ApplyAlignmentToSelection(rt.TEXT_ALIGNMENT_RIGHT)
        
    def OnAlignCenter(self, evt):
        self.editor.ApplyAlignmentToSelection(rt.TEXT_ALIGNMENT_CENTRE)


    def OnBullet(self, evt):
            attr = rt.TextAttrEx()
            #attr.SetFlags(attr.GetFlags() |rt.TEXT_ATTR_BULLET_STYLE | rt.TEXT_ATTR_LEFT_INDENT | rt.TEXT_ATTR_PARAGRAPH )
            ip = self.editor.GetInsertionPoint()
            if self.editor.GetStyle(ip, attr):
                r = rt.RichTextRange(ip, ip)
                if self.editor.HasSelection():
                    r = self.editor.GetSelectionRange()

                attr.SetBulletStyle(rt.TEXT_ATTR_BULLET_STYLE_STANDARD) 
                attr.SetLeftIndent(50,50) 
                attr.SetParagraphSpacingBefore(attr.GetParagraphSpacingBefore() + 20)
                self.editor.SetStyle(r, attr)



    def OnNoBullet(self, evt):
            attr = rt.TextAttrEx()
            ip = self.editor.GetInsertionPoint()
            
            if self.editor.GetStyle(ip, attr):
                r = rt.RichTextRange(ip, ip)
                if self.editor.HasSelection():
                    r = self.editor.GetSelectionRange()

                attr.SetBulletStyle(rt.TEXT_ATTR_BULLET_STYLE_NONE)
                attr.SetLeftIndent(0,0)
                attr.SetParagraphSpacingBefore(attr.GetParagraphSpacingBefore() - 20)
                self.editor.SetStyle(r, attr)




    def OnIndentMore(self, evt):
        attr = rt.TextAttrEx()
        attr.SetFlags(rt.TEXT_ATTR_LEFT_INDENT)
        ip = self.editor.GetInsertionPoint()
        if self.editor.GetStyle(ip, attr):
            r = rt.RichTextRange(ip, ip)
            if self.editor.HasSelection():
                r = self.editor.GetSelectionRange()

            attr.SetLeftIndent(attr.GetLeftIndent() + 100)
            attr.SetFlags(rt.TEXT_ATTR_LEFT_INDENT)
            self.editor.SetStyle(r, attr)
             
    def OnIndentLess(self, evt):
        attr = rt.TextAttrEx()
        attr.SetFlags(rt.TEXT_ATTR_LEFT_INDENT)
        ip = self.editor.GetInsertionPoint()
        if self.editor.GetStyle(ip, attr):
            r = rt.RichTextRange(ip, ip)
            if self.editor.HasSelection():
                r = self.editor.GetSelectionRange()

        if attr.GetLeftIndent() >= 100:
            attr.SetLeftIndent(attr.GetLeftIndent() - 100)
            attr.SetFlags(rt.TEXT_ATTR_LEFT_INDENT)
            self.editor.SetStyle(r, attr)

    def OnParagraphSpacingMore(self, evt):
        attr = rt.RichTextAttr()
        attr.SetFlags(rt.TEXT_ATTR_PARA_SPACING_AFTER)
        ip = self.editor.GetInsertionPoint()
        if self.editor.GetStyle(ip, attr):
            r = rt.RichTextRange(ip, ip)
            if self.editor.HasSelection():
                r = self.editor.GetSelectionRange()

            attr.SetParagraphSpacingAfter(attr.GetParagraphSpacingAfter() + 20)
            attr.SetFlags(rt.TEXT_ATTR_PARA_SPACING_AFTER)
            self.editor.SetStyle(r, attr)

        
    def OnParagraphSpacingLess(self, evt):
        attr = rt.RichTextAttr()
        attr.SetFlags(rt.TEXT_ATTR_PARA_SPACING_AFTER)
        ip = self.editor.GetInsertionPoint()
        if self.editor.GetStyle(ip, attr):
            r = rt.RichTextRange(ip, ip)
            if self.editor.HasSelection():
                r = self.editor.GetSelectionRange()

            if attr.GetParagraphSpacingAfter() >= 20:
                attr.SetParagraphSpacingAfter(attr.GetParagraphSpacingAfter() - 20);
                attr.SetFlags(rt.TEXT_ATTR_PARA_SPACING_AFTER)
                self.editor.SetStyle(r, attr)

        
    def OnLineSpacingSingle(self, evt):
        attr = rt.RichTextAttr()
        attr.SetFlags(rt.TEXT_ATTR_LINE_SPACING)
        ip = self.editor.GetInsertionPoint()
        if self.editor.GetStyle(ip, attr):
            r = rt.RichTextRange(ip, ip)
            if self.editor.HasSelection():
                r = self.editor.GetSelectionRange()

            attr.SetFlags(rt.TEXT_ATTR_LINE_SPACING)
            attr.SetLineSpacing(10)
            self.editor.SetStyle(r, attr)
 
                
    def OnLineSpacingHalf(self, evt):
        attr = rt.RichTextAttr()
        attr.SetFlags(rt.TEXT_ATTR_LINE_SPACING)
        ip = self.editor.GetInsertionPoint()
        if self.editor.GetStyle(ip, attr):
            r = rt.RichTextRange(ip, ip)
            if self.editor.HasSelection():
                r = self.editor.GetSelectionRange()

            attr.SetFlags(rt.TEXT_ATTR_LINE_SPACING)
            attr.SetLineSpacing(15)
            self.editor.SetStyle(r, attr)

        
    def OnLineSpacingDouble(self, evt):
        attr = rt.RichTextAttr()
        attr.SetFlags(rt.TEXT_ATTR_LINE_SPACING)
        ip = self.editor.GetInsertionPoint()
        if self.editor.GetStyle(ip, attr):
            r = rt.RichTextRange(ip, ip)
            if self.editor.HasSelection():
                r = self.editor.GetSelectionRange()

            attr.SetFlags(rt.TEXT_ATTR_LINE_SPACING)
            attr.SetLineSpacing(20)
            self.editor.SetStyle(r, attr)


    def OnFont(self, evt):
        if not self.editor.HasSelection():
            return

        r = self.editor.GetSelectionRange()
        fontData = wx.FontData()
        fontData.EnableEffects(True)
        attr = rt.TextAttrEx()
        attr.SetFlags(rt.TEXT_ATTR_FONT)
        if self.editor.GetStyle(self.editor.GetInsertionPoint(), attr):
            fontData.SetInitialFont(attr.GetFont())
            fontData.SetColour(attr.GetTextColour())


        dlg = wx.FontDialog(self, fontData)
        if dlg.ShowModal() == wx.ID_OK:
            data = dlg.GetFontData()
            font = data.GetChosenFont()
            colour = data.GetColour()
            if font:
                attr.SetFlags(rt.TEXT_ATTR_FONT)
                attr.SetFont(font)
                attr.SetTextColour(colour)
                self.editor.SetStyle(r, attr)
        dlg.Destroy()


    def OnColour(self, evt):
        colourData = wx.ColourData()
        attr = rt.TextAttrEx()
        attr.SetFlags(rt.TEXT_ATTR_TEXT_COLOUR)
        if self.editor.GetStyle(self.editor.GetInsertionPoint(), attr):
            colourData.SetColour(attr.GetTextColour())

        dlg = wx.ColourDialog(self, colourData)
        if dlg.ShowModal() == wx.ID_OK:
            colourData = dlg.GetColourData()
            colour = colourData.GetColour()
            if colour:
                if not self.editor.HasSelection():
                    self.editor.BeginTextColour(colour)
                else:
                    r = self.editor.GetSelectionRange()
                    attr.SetFlags(rt.TEXT_ATTR_TEXT_COLOUR)
                    attr.SetTextColour(colour)
                    self.editor.SetStyle(r, attr)
        dlg.Destroy()

    
    def OnEditorPrint(self, event):
        self.rtPrint.PrintBuffer(self.editor.GetBuffer())
    
    
    def OnEditorPrintPre(self, event):
        try:
            self.rtPrint.PreviewBuffer(self.editor.GetBuffer())
        except:
            pass

    def ForwardEvent(self, evt):
        print 'forward'
        self.editor.ProcessEvent(evt)

    
    def OnEditorCut(self, event):
        #self.editor.Cut()
        self.OnCutCopy(None)

    def OnEditorDelete(self, event):
        self.editor.DeleteSelection()

    def OnEditorSelectAll(self, event):
        self.editor.SelectAll()

    def OnEditorCopy(self, event):
        #self.editor.Copy()
        self.OnCutCopy(None)
    
    def OnEditorPaste(self, event):
        #self.editor.Paste()
        self.OnPaste(None)
        #self.OnMakePageLinks(None)


    ## Format to RTF for clipboard
    def GetFormattedSelection(self, format):
        """ Return a string with the formatted contents of just the RichText control's current selection.
            format can either be 'XML' or 'RTF'. """
        # If a valid format is NOT passed ...
        if not format in ['XML', 'RTF']:
            # ... return a blank string
            return ''
        
        # NOTE:  The wx.RichTextCtrl doesn't provide an easy way to get a formatted selection that I can figure out.
        #        This is a hack, but it works.

        # Freeze the control so things will work faster
        self.editor.Freeze()
        # We'll need to undo everything from this point on.  Let's do it as ONE operation.
        self.editor.BeginBatchUndo('RTCBufferSelection')
        # Get the start and end of the current selection
        sel = self.editor.GetSelection()       
        # Delete everything AFTER the end of the current selection
        self.editor.Delete((sel[1], self.editor.GetLastPosition()))
        # Delete everything BEFORE the start of the current selection
        if sel[0] > 0:
            self.editor.Delete((0, sel[0]))
        # This leaves us with JUST the selection in the control!

        # If XML format is requested ...
        if format == 'XML':
            # Create a Stream
            stream = cStringIO.StringIO()
            # Get an XML Handler
            handler = richtext.RichTextXMLHandler()
            # Save the contents of the control to the stream
            handler.SaveStream(self.editor.GetBuffer(), stream)
            # Convert the stream to a usable string
            tmpBuffer = stream.getvalue()
        # If RTF format is requested ....
        elif format == 'RTF':
            # Get an RTF Handler
            handler = PyRTFParser.PyRichTextRTFHandler()
            # Get the string representation by leaving off the filename parameter
            tmpBuffer = handler.SaveFile(self.editor.GetBuffer())

        # End Undo batching
        self.editor.EndBatchUndo()
        # Undo the changes.  This restores ALL the text
        self.editor.Undo()
        # Now thaw the control so that updates will be displayed again
        self.editor.Thaw()
            
        # Return the buffer's XML string
        return tmpBuffer

    def OnCutCopy(self, event):
        """ Handle Cut and Copy events, over-riding the RichTextCtrl versions.
            This implementation supports Rich Text Formatted text, and at least on Windows can
            share formatted text with other programs. """
        # Create a Composite Data Object for the Clipboard
        compositeDataObject = wx.DataObjectComposite()

        # Get the current selection in RTF format
        rtfSelection = self.GetFormattedSelection('RTF')
        # Create a Custom Data Format for RTF
        if 'wxMac' in wx.PlatformInfo:
            rtfFormat = wx.CustomDataFormat('public.rtf')
        else:
            # Create a Custom Data Format for RTF
            rtfFormat = wx.CustomDataFormat('Rich Text Format')
        # Create a Custom Data Object for the RTF format
        rtfDataObject = wx.CustomDataObject(rtfFormat)
        # Save the RTF version of the control selection to the RTF Custom Data Object
        rtfDataObject.SetData(rtfSelection)
        # Add the RTF Custom Data Object to the Composite Data Object
        compositeDataObject.Add(rtfDataObject)
        
        # Get the current selection in Plain Text
        txtSelection = self.editor.GetStringSelection()
        # Create a Text Data Object
        txtDataObject = wx.TextDataObject()
        # Save the Plain Text version of the control selection to the Text Data Object
        txtDataObject.SetText(txtSelection)
        # Add the Plain Text Data Object to the Composite Data object
        compositeDataObject.Add(txtDataObject)

        # Open the Clipboard
        if wx.TheClipboard.Open():
            # Place the Composite Data Object (with RTF and Plain Text) on the Clipboard
            wx.TheClipboard.SetData(compositeDataObject)
            # Close the Clipboard
            wx.TheClipboard.Close()

        # If we are CUTting (rather than COPYing) ...
        if event and event.GetId() == wx.ID_CUT:
            # ... delete the selection from the Rich Text Ctrl.
            self.editor.DeleteSelection()

    def OnPaste(self, event):
        """ Handle Paste events, over-riding the RichTextCtrl version.
            This implementation supports Rich Text Formatted text, and at least on Windows can
            share formatted text with other programs. """
        # Open the Clipboard
        if wx.TheClipboard.Open():
            if 'wxMac' in wx.PlatformInfo:
                rtfFormat = wx.CustomDataFormat('public.rtf')
            else:
                # Create a Custom Data Format for RTF
                rtfFormat = wx.CustomDataFormat('Rich Text Format')
            # See if the RTF Format is supported by the current clipboard data object
            if wx.TheClipboard.IsSupported(rtfFormat):
                # Specify that the data object accepts data in RTF format
                customDataObject = wx.CustomDataObject(rtfFormat)
                # Try to get data from the Clipboard
                success = wx.TheClipboard.GetData(customDataObject)
                # If the data in the clipboard is in an appropriate format ...
                if success:
                    # ... get the data from the clipboard
                    formattedText = customDataObject.GetData()
                    # Prepare the control for data
                    self.editor.Freeze()
                    self.editor.BeginSuppressUndo()
                    # Start exception handling
                    try:
                        # Use the custom RTF Handler
                        handler = PyRTFParser.PyRichTextRTFHandler()
                        # Load the RTF data into the Rich Text Ctrl via the RTF Handler.
                        # Note that for RTF, the wxRichTextCtrl CONTROL is passed with the RTF string.
                        handler.LoadString(self.editor, formattedText)
                    # exception handling
                    except:
                        print "Custom RTF Handler Load failed"
                        pass

                    # Signal the end of changing the control
                    self.editor.EndSuppressUndo()
                    self.editor.Thaw()
            # If there's not RTF data, see if there's Plain Text data
            elif wx.TheClipboard.IsSupported(wx.DataFormat(wx.DF_TEXT)):
                # Create a Text Data Object
                textDataObject = wx.TextDataObject()
                # Get the Data from the Clipboard
                wx.TheClipboard.GetData(textDataObject)
                # Write the plain text into the Rich Text Ctrl
                self.editor.WriteText(textDataObject.GetText())
            # Close the Clipboard
            wx.TheClipboard.Close()
