 # -*- coding: utf-8 -*-

import re, string, wx, chapIdx
import sqlite3 as sqlite



refmatches = {
    "Ge":"Gen", "Ex":"Exo", "Le":"Lev", "Nu":"Num", "De":"Deu",  "Jos":"Jos",
    "Judg":"Jdg",  "Ru":"Rth",  "1 ?Sa":"1Sa", "2 ?Sa":"2Sa",  "1 ?Ki":"1Ki",  "2 ?Ki":"2Ki",
    "1 ?Ch":"1Ch",  "2 ?Ch":"2Ch", "Ezr":"Ezr", "Ne":"Neh",  "Es":"Est",  "Job":"Job",
    "Ps":"Psa",  "Pr":"Pro", "Ec":"Ecc", "So":"Son", "Is":"Isa",  "Je":"Jer",
    "La":"Lam", "Eze":"Eze", "Da":"Dan", "Ho":"Hos", "Joe":"Joe",  "Am":"Amo",
    "Ob":"Oba",  "Jon":"Jon", "Mi":"Mic", "Na":"Nah",  "Hab":"Hab",  "Zep":"Zep",
    "Hag":"Hag",  "Zec":"Zec", "Mal":"Mal", "Jgs":"Jdg", "Rth":"Rth",
    "Mat":"Mat",  "Mar":"Mar", "Lu":"Luk", "Joh": "Joh",  "Ac":"Act",  "Ro":"Rom",
    "1 ?Co":"1Co",  "2 ?Co":"2Co",  "Ga":"Gal",  "Ep":"Eph",  "Phi":"Phi", "Php":"Phi", "Col":"Col",
    "1 ?Th":"1Th",  "2 ?Th":"2Th",  "1 ?Ti": "1Ti", "2 ?Ti": "2Ti",  "Tit": "Tit", "He": "Heb",
    "Ja": "Jam", "1 ?Pe":"1Pe", "2 ?Pe":"2Pe", "1 ?Jo":"1Jo", "2 ?Jo": "2Jo",  "3 ?Jo":"3Jo",
    "Jud": "Jud","Re": "Rev", "Mt":"Mat", "Mk":"Mar", "Lk":"Luk", "Jn":"Joh",  "Jas":"Jam",
    "1 ?Kgs":"1Ki",  "2 ?Kgs":"2Ki", "1 ?Jn":"1Jo",  "2 ?Jn":"2Jo", "3 ?Jn":"3Jo",
    "Phm":"Phm", "Phile":"Phm", "Jdg":"Jdg", "1S":"1Sa", "2S":"2Sa", "1K":"1Ki", "2K":"2Ki",
    "1P":"1Pe", "2P":"2Pe", "1J":"1Jo", "2J":"2Jo", "3J":"3Jo", "Jde":"Jud", "Tts":"Tit"
    }

refOrder = [
     "Ge", "Ex", "Le", "Nu", "De", "Jos", "Judg", "Jdg", "Jgs", "Ru", "1 ?Sa", "2 ?Sa", "1 ?Ki", "2 ?Ki",
     "1 ?Ch", "2 ?Ch", "Ezr", "Ne", "Es", "Job", "Ps", "Pr", "Ec", "So", "Is", "Je",
     "La", "Eze", "Da", "Ho", "Joe", "Am", "Ob", "Jon", "Mi", "Na", "Hab", "Zep", "Rth",
     "Hag", "Zec", "Mal", "Mat", "Mar", "Lu", "Ac", "Ro", 
     "1 ?Co", "2 ?Co", "Ga", "Ep", "Col", "1 ?Th", "2 ?Th", "1 ?Ti", "2 ?Ti", "Tit", "He",
     "Ja", "1 ?Pe", "2 ?Pe", "1 ?Jo", "2 ?Jo", "3 ?Jo", "Jud","Re", "Mt", "Mk", "Lk", "Jas",
     "1 ?Kgs", "2 ?Kgs", "1 ?Jn", "2 ?Jn", "3 ?Jn", "Phm", "Phile",  "1S", "2S", "1K", "2K",
     "1P", "2P", "1J", "2J", "3J", "Jde", "Tts", "Jn", "Joh", "Phi", "Php" 
    ]

chapIdx = chapIdx.chapIdx

multiRngLen = 0

# Finds correct reference for search
def refMatch(fullRef):
    fullRef = fullRef.replace(' ', '')
    
    # Make ref with number capitalized
    if re.match('\d', fullRef) and len(fullRef) > 1:
        cap = string.upper(fullRef[1])
        fullRef = fullRef[0] + cap + fullRef[2:]
    else:
        fullRef = fullRef.capitalize()

    # Find match
    if '_' in fullRef:
        bkIdx = fullRef.find('_')
        bk = fullRef[:bkIdx]
        for book in refOrder:
            hit = re.match(book + '\w*', bk)
            if hit:
                return refmatches[book] + fullRef[bkIdx:]
    else:
        for book in refOrder:
            hit = re.match(book + '\w*', fullRef)
            if hit:
                return refmatches[book]
    
            


# Matches ref with a full book name
def fullTitle(abbr):
    if re.match('\d', abbr):
        cap = string.upper(abbr[1])
        abbr = abbr[0] + cap + abbr[2:]
    else:
        abbr = abbr.capitalize()

    books = {
        'Gen':'Genesis', 'Exo':'Exodus', 'Lev':'Leviticus', 'Num':'Numbers', 'Deu':'Deuteronomy', 'Jos':'Joshua', 'Jdg':'Judges',
        'Rth':'Ruth', '1Sa':'1 Samuel', '2Sa':'2 Samuel', '1Ki':'1 Kings', '2Ki':'2 Kings', '1Ch':'1 Chronicles',
        '2Ch':'2 Chronicles', 'Ezr':'Ezra', 'Neh':'Nehemiah', 'Est':'Esther', 'Job':'Job', 'Psa':'Psalms', 'Pro':'Proverbs',
        'Ecc':'Ecclesiastes', 'Son':'Song Solomon', 'Isa':'Isaiah', 'Jer':'Jeremiah', 'Lam':'Lamentations', 'Eze':'Ezekiel',
        'Dan':'Daniel', 'Hos':'Hosea', 'Joe':'Joel', 'Amo':'Amos', 'Oba':'Obadiah', 'Jon':'Jonah', 'Mic':'Micah', 'Nah':'Nahum',
        'Hab':'Habbakkuk', 'Zep':'Zephaniah', 'Hag':'Haggai', 'Zec':'Zechariah', 'Mal':'Malachi', 
        'Mat':'Matthew', 'Mar':'Mark',
        'Luk':'Luke', 'Joh':'John', 'Act':'Acts', 'Rom':'Romans', '1Co':'1 Corinthians', '2Co':'2 Corinthians', 'Gal':'Galatians',
        'Eph':'Ephesians', 'Phi':'Philippians', 'Col':'Colossians', '1Th':'1 Thessalonians', '2Th':'2 Thessalonians',
        '1Ti':'1 Timothy', '2Ti':'2 Timothy', 'Tit':'Titus', 'Phm':'Philemon', 'Heb':'Hebrews', 'Jam':'James', '1Pe':'1 Peter',
        '2Pe':'2 Peter', '1Jo':'1 John', '2Jo':'2 John', '3Jo':'3 John', 'Jud':'Jude', 'Rev':'Revelation',}

    try:
        return books[abbr]
    except:
        return abbr


# Matches 1611 ref with a full book name
def fullTitle1611(abbr):
    if re.match('\d', abbr):
        cap = string.upper(abbr[1])
        abbr = abbr[0] + cap + abbr[2:]
    else:
        abbr = abbr.capitalize()

    books = {
        'Gen':u'Geneſis.', 'Exo':'Exodus.', 'Lev':'Leuiticus.', 'Num':'Numbers.', 'Deu':'Deuteronomie.', 'Jos':u'Ioſhua.', 'Jdg':'Iudges.',
        'Rth':'Ruth.', '1Sa':'I Samuel.', '2Sa':'II Samuel.', '1Ki':'I Kings.', '2Ki':'II Kings.', '1Ch':'I Chron.',
        '2Ch':'II Chron.', 'Ezr':'Ezra.', 'Neh':'Nehemiah.', 'Est':u'Eſther.', 'Job':'Job.', 'Psa':u'Pſalms.', 'Pro':'Prouerbes.',
        'Ecc':u'Eccleſiaſtes.', 'Son':'Solomons Song.', 'Isa':u'Iſaiah.', 'Jer':'Ieremiah.', 'Lam':'Lamentations.', 'Eze':'Ezekiel.',
        'Dan':'Daniel.', 'Hos':u'Hoſea.', 'Joe':'Ioel.', 'Amo':'Amos.', 'Oba':'Obadiah.', 'Jon':'Ionah.', 'Mic':'Micah.', 'Nah':'Nahum.',
        'Hab':'Habbakkuk.', 'Zep':'Zephaniah.', 'Hag':'Haggai.', 'Zec':'Zechariah.', 'Mal':'Malachi.', 
        
        '1Es':u'I. Eſdras. [Apocrypha]', '2Es':u'II. Eſdras. [Apocrypha]', 'Tob':'Tobit. [Apocrypha]', 
        'Jth':'Ivdeth. [Apocrypha]', 'RoE':u'Rest of Eſther. [Apocrypha]', 'Wis':u'Wiſdom of Solomon. [Apocrypha]', 
        'Ecs':u'Eccleſiasticus. [Apocrypha]', 'Bar':'Barvch. [Apocrypha]', 'SoC':'Song of the Three Holy Children. [Apocrypha]', 
        'Sus':u'Suſanna. [Apocrypha]', 'Bel':'Bel and the Dragon. [Apocrypha]', 'Man':u'Prayer of Manaƅ߿es. [Apocrypha]', 
        '1Ma':'I. Maccabees. [Apocrypha]', '2Ma':'II. Maccabees. [Apocrypha]',
        
        'Mat':'S.Matthew.', 'Mar':'S.Marke',
        'Luk':'S.Luke.', 'Joh':'S.Iohn', 'Act':'The Actes.', 'Rom':'Romanes.', '1Co':'I Corinthians.', '2Co':'II Corinthians', 'Gal':'Galatians.',
        'Eph':u'Epheſians.', 'Phi':'Philippians.', 'Col':u'Coloſſians.', '1Th':u'I Theſſalonians', '2Th':u'II Theſſalonians',
        '1Ti':'I Timothie.', '2Ti':'II Timothie.', 'Tit':'Titus.', 'Phm':'Philemon.', 'Heb':'Hebrewes.', 'Jam':'Iames.', '1Pe':'I Peter.',
        '2Pe':'II Peter.', '1Jo':'I Iohn.', '2Jo':'II Iohn.', '3Jo':'III Iohn.', 'Jud':'Iude.', 'Rev':'Reuelation.'}

    try:
        return books[abbr]
    except:
        return abbr



# The following: get start and end indexs from range.


def flagIdx(ref):
    idx = ref.find(':')
    vs = 0
    if ref[-1].isdigit():
        vs = int(ref[idx +1:])
    chap = ref[:idx +1]
    return chapIdx[chap] + vs
    
    #for grp in chapIdx.chapIdx:
    #    if grp[0].startswith(chap):
    #        #print chap, grp, 'ref grp'
    #        return grp[1] + vs 


# For popups
def getIdx(ref, fn):
    con = sqlite.connect(fn)
    cur = con.cursor()
    
    cur.execute('SELECT id FROM bible WHERE ref LIKE ?', (ref+'%',))
    rng = cur.fetchall()
    #print ref, rng[0][0], rng[-1][0]
    
    if rng:
        return rng[0][0], rng[-1][0]
    else:
        return None
    



lastBk = None
lastChap = None

lastFn = None
currRefList = None


# For ranges
def findIdx(ref, fn):
    
    global lastFn
    global currRefList
    
    if not ':' in ref and ref[-1].isdigit():
        ref = ref + ':'
            
        
    if fn != lastFn:
        con = sqlite.connect(fn)
        cur = con.cursor()
        
        selStr = "SELECT id, ref FROM bible"    
        cur.execute(selStr)
        currRefList = cur.fetchall()
        lastFn = fn
       
    hit = []
    for r in currRefList:
        if r[1].startswith(ref):
            if re.search(r'%s\b' % ref, r[1]):
                hit.append(r)
    
    if hit:
        start = hit[0][0]
        end = hit[-1][0]
    else:
        return 'rngError', None
    
    print ref, 'findIdx', start, end
       
    if start > end:
        return
    
    #print ref, start, end
    return start, end

#findIdx('Joh 1:1', 'D:\\Scripts\\Bible Analyzer 4.0\\Modules\\Bible\\av1769.bib')


# Make refs uniform
def fix(rf):
    #print rf, 'rf'
    if ' ' in rf:
        bkIdx = rf.find(' ')
        bk = rf[:bkIdx]
        bk = refMatch(bk)
        try:
            return bk + rf[bkIdx:]
        except:
            return rf
    else:
        if rf.isdigit():
            return rf

        fixRf = refMatch(rf)
        if fixRf:
            return fixRf
        else:
            return 'noRef'



def parseRef(ref, fn = None, fromEntry = 0):
    ref = ref.strip()
    
    #print ref, fn, 'fn'
    
    ref = ref.replace('_', ' ')
    
    idxList = []
    global lastChap
    
    ref = prelimFix(ref) 
    
    if fromEntry == 1:
        return fix(ref)
    
    if ':' in ref:
        idx = ref.find(':')
        chap = ref[:idx]
        lastChap = chap  

    if ref.isdigit():
        ref = '%s:%s' %(lastChap, ref)
    
    book = ref[:3]
    
    if '-' in ref:
        refList = ref.split('-')
        
        for cyc, r in enumerate(refList):
            r = r.strip(' ')
            
            # For refs like Mar 7:32-8:9
            if ':' in r and not ' ' in r:
                r = '%s %s' %(lastChap[:3], r)
                        
            if r.isdigit():
                if ' ' in ref and not ':' in ref:
                    r = '%s %s' %(book, r)
                else:
                    r = '%s:%s' %(lastChap, r)
            
            start, end = findIdx(r, fn)
            
            if cyc == 0:
                idxList.append(start)
                fixed = r
            else:
                idxList.append(end)
                fixed = fixed + '-' + r
                
        
    # Single book, chapter, or verse
    else:
        if ref[-1] == ':':
            ref = ref[:-1]
        ref = ref.strip(' ')                    

        if ref.isdigit():
            ref = '%s:%s' % (lastChap[:3], ref)
        
        fixed = ref
        
        start, end = findIdx(ref, fn)
        
        if start == 'rngError' or end == 'rngError':
            return 'rngError', None

        idxList.append(start)
        idxList.append(end)
        
    fixed = re.sub(r': -', ' -', fixed)
    fixed = re.sub(r':$', '', fixed)
    
    return idxList, fixed


def prelimFix(ref):
    ref = ref.strip(' ')
    ref = ref.replace('III ', '3 ')
    ref = ref.replace('II ', '2 ')
    ref = ref.replace('I ', '1 ') 
    
    ref = ref.replace(';', ',')
    
    ref = re.sub(r',(\w)', r', \1', ref, re.I|re.MULTILINE)
    
    #print ref, 'prelim'
    
    # Fix 1 Co to 1Co
    ref = re.sub(r'([123]) ([sSkKcCtTpPjJ])', r'\1\2', ref, re.I|re.MULTILINE)
    
    # Adjust "-" spacing
    #ref = re.sub(r'(\d) ?- ?(\d)', r'\1-\2', ref, re.I|re.MULTILINE)
    #ref = re.sub(r'(\d) ?- ?([a-zA-Z])', r'\1 - \2', ref, re.I|re.MULTILINE)

    # When . or space between chapter and verse (ge2.1
    fix = re.compile(r'(\d)[.: ]+(\d)', re.I|re.MULTILINE)
    ref = fix.sub(r'\1:\2', ref)

    # Split when no space after book (ge2 to ge 2)
    ref = re.sub(r'([a-z])\.?(\d)', r'\1 \2', ref, re.I|re.MULTILINE)

    ref = ref.replace('.', '')    
    
    return ref
    



def parseRng(rng, fn = None):
    #print rng, 'rng1'
    
    mainApp = wx.GetApp().GetTopWindow()
    if not fn:
        fn = mainApp.bibFileList[mainApp.avIndex]
    
    global multiRngLen
    
    rng = prelimFix(rng)
    
    splList = re.split(r'([,-])', rng)
    #print splList
    
    tmp = []
    for s in splList:
        if s == ',' or s == '-':
            tmp.append(s)
            continue
        s = fix(s.strip())
        if s == 'noRef':
            mainApp.OnRefError()
            return None, None
        tmp.append(s)
    
    rng = ''.join(tmp)
    rng = rng.replace(',', ', ')
    
    #print rng, 'rng2'
         
    bookPat = re.compile(r"( ?\w+\.? )(\d+:\d+(?:-\d+)?[,;.)\]]) (\d+:\S+)", re.MULTILINE)
    chapPat = re.compile(r"( ?\w+\.? )(\d+:)(\S+) (\d+(?:-\d+)?[,;.)\]])", re.MULTILINE)

    while bookPat.search(rng) or chapPat.search(rng):
        #print rng, 'rng'
        rng = bookPat.sub(r"\1\2 \1\3", rng)
        rng = chapPat.sub(r"\1\2\3 \1\2\4", rng)

    simpleFix = re.sub(r', (\w+ \d+:)', r'; \1', rng, re.I|re.MULTILINE)
    simpleFix = re.sub(r', (\d+:)', r'; \1', simpleFix, re.I|re.MULTILINE)
    
    #print rng, 'rng3'

    idxList = []
    if ',' in rng:
        fixedRef = []
        multiRng = rng.split(',')
        for r in multiRng:
            try:
                idxGroup, fixed = parseRef(r.strip(), fn)
                #print r
                
                idxList.append(idxGroup)
                fixedRef.append(fixed)
                
                if '-' in fixed:
                    idx = fixed.find(' -')
                    fixed = fixed[:idx]
                if len(fixed) > multiRngLen:
                    multiRngLen = len(fixed)
            except:
                mainApp.OnRefError()
                return
        
        begIdx = 0
        endIdx = idxList[1]
        fixedRef = ', '.join(fixedRef)  

    else:
        
        idxGroup, fixedRef = parseRef(rng, fn)
        
        if not idxGroup:
            mainApp.OnRefError()
            return
        if idxGroup[0] == 'rngError':
            mainApp.OnRngError()
            return None, None
        
        idxList.append(idxGroup)
        
    #print idxList, simpleFix
    return idxList, simpleFix
    

#print parseRng('Mat 5.21-Mar 10.4')
