import os, os.path, re, zipfile, wx, wx.html, StringIO, toolImg, baseClass, advSearch, fixRef, popup, time
import wx.media
#import flatMenu as FM
import sqlite3 as sqlite
import tools.convertImg


import dragscroller


tmpDir = os.environ["TMP"]
#print tmpDir


class ImageViewer(wx.Frame):
    def __init__(self, parent):
        wx.Frame.__init__(self, parent, style = wx.DEFAULT_FRAME_STYLE|wx.CLIP_CHILDREN)
        
        self.mainApp = wx.GetApp().GetTopWindow()
        self.advSearch = advSearch.primSearch(self)
        
        self.imgCheck()

        #Group File Details For Choice Display
        self.imgGroupTitle = []

        #Individual Image Titles, nested
        self.imgTitles = []

        #List of Images file names, nested
        self.imgFileList = []  
        self.imgFNList = []
             
        #self.GetImages()
        self.GetDBImages()
        
        self.Panel = wx.Panel(self, -1)

        self.viewerTree = wx.TreeCtrl(self.Panel, -1, style=wx.TR_HAS_BUTTONS|wx.TR_HIDE_ROOT|wx.TR_LINES_AT_ROOT|wx.BORDER_THEME)
        self.fitSideButton = wx.ToggleButton(self.Panel, -1, "Fit Side")
        self.fitAllButton = wx.ToggleButton(self.Panel, -1, "Fit All")
        self.Display = baseClass.HtmlWindow(self.Panel, -1, style=wx.BORDER_THEME)       
        
        self.SetTitle("Bible Analyzer Image Viewer")
        
        icon = toolImg.getScrollIcon()
        self.SetIcon(icon)
        
        self.Menu = self.CreateMenu()
        self.SetMenuBar(self.Menu)

        self.SetMinSize((800, 600))
        self.viewerTree.SetMinSize((200, -1))
        self.fitSideButton.SetMinSize((50, -1))
        self.fitAllButton.SetMinSize((50, -1))


        mainSizer = wx.BoxSizer(wx.VERTICAL)
        self.fullSizer = wx.BoxSizer(wx.HORIZONTAL)
        treeSizer = wx.BoxSizer(wx.VERTICAL)
        buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
        
        buttonSizer.Add(self.fitSideButton, 0, wx.LEFT, 0)
        buttonSizer.Add(self.fitAllButton, 0, wx.LEFT, 5)
        
        treeSizer.Add(self.viewerTree, 1, wx.TOP|wx.ALIGN_CENTER_HORIZONTAL, 0)
        treeSizer.Add(buttonSizer, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL, 10)
        
        self.fullSizer.Add(treeSizer, 0, wx.TOP|wx.EXPAND, 0)
        self.fullSizer.Add((3, 0))
        self.fullSizer.Add(self.Display, 1, wx.EXPAND, 0)
        
        mainSizer.Add(self.fullSizer, 1, wx.EXPAND, 0)
        self.Panel.SetSizer(mainSizer)
        mainSizer.Fit(self)
        
        #self.Layout()
        self.Center()


        self.root = self.viewerTree.AddRoot('Top')
        
        self.Display.Bind(wx.EVT_MOTION, self.OnMotion)
        #self.Display.Bind(wx.html.EVT_HTML_CELL_HOVER, self.OnMotion)
        #self.Display.Bind(wx.html.EVT_HTML_CELL_CLICKED, self.OnLeftDown)
        
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnSelChanged)
        self.Bind(wx.EVT_TOGGLEBUTTON, self.OnFitAll, self.fitAllButton)
        self.Bind(wx.EVT_TOGGLEBUTTON, self.OnFitSide, self.fitSideButton)
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        
        self.Bind(wx.EVT_MENU, self.OnExport, id=1001)
        self.Bind(wx.EVT_MENU, self.OnPrint, id=1002)
        self.Bind(wx.EVT_MENU, self.OnPrintPre, id=1003)
        self.Bind(wx.EVT_MENU, self.OnPrintSetup, id=1004)
        self.Bind(wx.EVT_MENU, self.OnHide, id=1005)
        
        self.Bind(wx.EVT_MENU, self.OnClose, id=1010)
        
        self.Display.Bind(wx.EVT_LEAVE_WINDOW, self.leaveWindow)
        self.Display.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Display.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        
        self.Display.Bind(wx.EVT_CHAR, self.OnCancel)
        
        self.scroller = dragscroller.DragScroller(self.Display)
        
        self.ePrint = wx.html.HtmlEasyPrinting()
        self.ePrint.SetParentWindow(self)
        
        data = self.ePrint.GetPageSetupData()
        data.SetMarginTopLeft((10,10))
        data.SetMarginBottomRight((10,10))
        data.SetPaperId(1)

        
        self.BuildTree()
        
        intro = '''<center><font face="Verdana" size="+2" color="#660000">Bible Analyzer Image Viewer</font><br><br><font face="Verdana">
        Select an image to view from the titles on the left.<br><br>You can add you own <i>.jpg</i> and <i>.gif</i> images by placing them 
        in the<br>%s folder.<br> They will be found under <i>User Images</i> upon the next start.</font></center>''' % self.mainApp.myDocsDir.replace("\\", "/")
        
        self.Display.SetPage(intro)
        
        self.Toggle = 0
        self.inMem = 0
        self.inMemPrint = 0
        self.mapData = None
        self.txtData = None
        self.lastWord = None
        self.toolwin = None
        self.mc = None
        self.audioTitle = None
        self.mp3Idx = 0
        self.mp3Path = None
        self.regions = None
        
        self.memFs = wx.MemoryFSHandler()
        wx.FileSystem_AddHandler(self.memFs)
    


    def OnLeftDown(self, event):
        event.Skip()
        self.leaveWindow(None)
        
        if wx.GetKeyState(wx.WXK_CONTROL):
            self.scroller.Start(event.GetPosition())
        else:
            if self.mapData:
                #if not self.regions:
                #self.SetRegions()

                disp = self.Display.CalcScrolledPosition(self.Display.GetScreenPosition())
                pos = wx.GetMousePosition()
                adjPos0 = pos[0] - disp[0]
                adjPos1 = pos[1] - disp[1]
                
                for rg in self.regions:
                    if rg.Contains(adjPos0, adjPos1) == wx.InRegion:
                        
                        word = self.words[self.regions.index(rg)]
                        try:
                            if '[[Note]]' in word:
                                note = word.replace('[[Note]] ', '')
                                self.mainApp.OnDictMini(note, 'Image Note', 0)
                                break
                            
                            elif '[[Audio]]' in word:
                                audioTitle = self.newWord.replace('[[Audio]] ', '')
                                self.playAudio(audioTitle)
                                break
                            
                            elif '[[TTS]]' in word:
                                ttsParse = re.search(r'\[\[popup\]\](.+)\[\[read\]\](.+)', word)
                                ttsRead = ttsParse.group(2)
                                self.mainApp.OnTTSReadImage(ttsRead)
                                break
                            
                            elif re.search(r' \d+:', word):
                                self.mainApp.ShowRefContext(word)
                                break
                            
                            else:
                                self.OnWordSearch(word, self.rng[self.regions.index(rg)])
                                break
                        except:
                            print 'Image click error'
                            

    def OnLeftUp(self, event):
        event.Skip()
        self.scroller.Stop()
    
    
    def leaveWindow(self, event):
        try:
            self.toolwin.Destroy()
        except:
            pass

        try:
            self.mainApp.toolwin.Destroy()
        except:
            pass

    
    def OnWordSearch(self, word, rng, fromMotion = 0):
        # 0-word, 1-fName, 2-abbr, 3-full, 4-opt, 5-case, 6-fface, 7-showRed, 8-redSearch, 9-idxList, 10-exclude,
        # 11-rangeStr, 12-refOnly, 13-resWdList, 14-doRoot, 15-fixedRef
        
        rngIdx, rngLabel = fixRef.parseRng(rng)
        
        #print rngIdx, rngLabel
        
        fPath = self.mainApp.bibFileList[self.mainApp.avIndex]
        
        popup = 0
        if fromMotion:
            popup = 1
        
        if word and rngIdx:
            result, verseList, title, count, findStr, totalHits, exclude, rngRefList = self.advSearch.StdSearch\
            (word, fPath, 'AV', 1, 0, 0, self.mainApp.font, 1, 0, rngIdx, None, rngLabel, 0, 0, 0, rngLabel, 0, 0, popup)
        else:
            return
        
        if fromMotion:
            return result, verseList, title, count, findStr, totalHits, exclude, rngRefList
        
        # Add data to search query list
        temp = [word, fPath, 'AV', 1, 0, 0, 1, 0, count, findStr, totalHits, exclude, rngLabel, rngRefList, rngIdx]
        self.mainApp.searchQList.append(temp)
        
        #print self.mainApp.searchQList
        
        if self.mainApp.fontSize > 0:
            result = '<font size="+%s">%s</font>' % (self.mainApp.fontSize, result)
            
        try:
            title = unicode(title, "utf-8")
        except:
            title = ''
        
        #self.mainApp.Raise()
        #self.mainApp.Freeze()
        self.mainApp.AddResult(word, title, result)
        self.mainApp.Showing = result
        #self.mainApp.Thaw()
        
        if self.mainApp.getMaximized():
            self.mainApp.mainMgr.RestoreMaximizedPane()
            self.mainApp.mainMgr.MaximizePane(self.mainApp.getResPane())
            self.mainApp.mainMgr.Update()


    def SetRegions(self):
        self.regions = []
        self.words = []
        self.rng = []
        for set in self.mapData:
            c1 = int(set[0])
            c2 = int(set[1])
            c3 = int(set[2])
            c4 = int(set[3])
            self.regions.append(wx.Region(c1,c2,c3,c4))
            self.words.append(set[4])
            self.rng.append(set[5])

    
    def OnMotion(self, event):
        event.Skip()
        if not self.mapData or wx.GetKeyState(wx.WXK_CONTROL):
            return
        
        pos = event.GetPosition()
        win = event.GetEventObject()
        sPos = win.CalcUnscrolledPosition(pos)
        
        self.SetRegions()
        
        pos = wx.GetMousePosition()
        
        imgRegion = wx.Region(10, 10, self.imgSize[0], self.imgSize[1])
        
        if imgRegion.Contains(sPos[0], sPos[1]) == wx.InRegion:
        
            hit = 0
            for rg in self.regions:
                
                if rg.Contains(sPos[0], sPos[1]) == wx.InRegion:
                    
                    self.Display.SetCursor(wx.StockCursor(wx.CURSOR_HAND))
                    self.newWord = self.words[self.regions.index(rg)]
                    
                    #print self.newWord
                    
                    if self.newWord == self.lastWord and self.toolwin:
                        return
                    
                    if self.toolwin:
                        self.toolwin.Destroy()
                    try:
                        self.mainApp.toolwin.Destroy()
                    except:
                        pass
                    
                    self.lastWord = self.newWord
                    
                    time.sleep(.1)
                    hit = 1
                    
                    if '[[Note]]' in self.newWord:
                        #print self.newWord, 'Note'
                        note = self.newWord.replace('[[Note]] ', '')
                        
                        noteTxt = '<b>Note:</b>'

                        if '[[title]]' in note:
                            ttsParse = re.search(r'\[\[title\]\](.+)\[\[body\]\](.+)', note)
                            title = ttsParse.group(1)
                            noteTxt = '<b>Note:</b> %s' % title
                            note = ttsParse.group(2)

                        if len(note) > 750:
                            note = note[:750] + '...<br>...<i>Click to continue.</i>'
                    
                        result = '''<body background="memory:bg.png"><font size="%s">
                        <font color="#660000">%s</font><br><br>%s<br><br><center><i>Click to view entire text in MultiWindow</i></center>
                        ''' % (self.mainApp.popFontSize +2, noteTxt, note)
                        
                        self.showPopup(pos[0], pos[1], result)
                        return
                    
                    elif '[[Audio]]' in self.newWord:
                        audioTitle = self.newWord.replace('[[Audio]] ', '')
                        
                        result = '''<body background="memory:bg.png"><font size="%s">
                        <font color="#660000"><b>Audio <i>%s</i> available</b>:</font> <i>Click</i> to play; <i>Esc</i> to cancel.
                        ''' % (self.mainApp.popFontSize +2, audioTitle.capitalize())
                        
                        self.showPopup(pos[0], pos[1], result)
                        return
                    
                    elif '[[TTS]]' in self.newWord:
                        if not self.mainApp.tts:
                            return
                        ttsParse = re.search(r'\[\[popup\]\](.+)\[\[read\]\](.+)', self.newWord)
                        ttsPopup = ttsParse.group(1)
                        ttsRead = ttsParse.group(2)
                        
                        result = '''<body background="memory:bg.png"><font size="%s">
                        <font color="#660000"><b>Text-To-Speech Available:</font><br><br>%s</b><br><br><center><i>Click</i> to read; <i>Esc</i> to cancel.</center>
                        ''' % (self.mainApp.popFontSize +2, ttsPopup)
                        
                        self.showPopup(pos[0], pos[1], result)
                        return
                    
                    elif re.search(r' \d+:', self.newWord):
                        line = self.mainApp.GetPopupVerse(self.newWord)
                        if line:
                            self.showPopup(pos[0], pos[1], line)
                        return
                    
                    result, verseList, title, count, findStr, totalHits, exclude, rngRefList = \
                    self.OnWordSearch(self.newWord, self.rng[self.regions.index(rg)], 1)
                    
                    result = re.sub(r'<body.*|Verses.*|</?u>', '', result)
                    result = re.sub(r'_', ' ', result)
                    result = re.sub(r'<b>\d+</b>.*<hr>', '', result)
                    result = re.sub(r'<br><br>', '<br>', result)
                    result = result.strip()
                    resList = result.split('<br>')
                    
                    res = '<br>'.join(resList[:3])
                    if count > 3:
                        res = res + '<br>&nbsp;&nbsp;&nbsp;&nbsp;<font color="#336666"><i>...cont.</i> (click to view all)</font></font>'
                        #print '1', res
                    elif count < 3:
                        res = res[:-13] + '</font>'
                        #print '2', res


                    result = '''<body background="memory:bg.png"><font size="%s">
                    <center><font color="#660000"><b>%s</b> - %s (%s)</font><br>
                    <font size="-1"> %s times in %s verses </font></center>
                    <table  cellspacing="1" ><tbody><tr><td></td></tr></tbody></table>%s</font>
                    ''' % (self.mainApp.popFontSize +3, self.newWord.capitalize(), 'AV', self.rng[self.regions.index(rg)], totalHits, count, res)

                    self.showPopup(pos[0], pos[1], result)
                    break
                else:
                    self.Display.SetCursor(wx.NullCursor)
            
            if not hit:
                if self.toolwin:
                    self.toolwin.Destroy()
        else:
            self.mainApp.OnMotion(event)


    def showPopup(self, x, y, line):
        self.toolwin = popup.HtmlPopup(None, line, self.mainApp.popColorIdx, self.mainApp.popFontSize, 0)

        scnWidth = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_X)
        scnHeight = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_Y)
        w, h = self.toolwin.GetSizeTuple()
        if x +10 + w  > scnWidth:
            x = x - w -15
        if y +25 + h > scnHeight:
            y = y - h -25
        
        self.toolwin.SetPosition((x +5, y +15))
        self.toolwin.Show()


    def playAudio(self, audioTitle):
        if not self.mc:
            try:
                if 'XP' in wx.GetOsDescription():
                    self.ImgMC = wx.media.MediaCtrl(self, style=wx.SIMPLE_BORDER)
                else:    
                    self.ImgMC = wx.media.MediaCtrl(self, style=wx.SIMPLE_BORDER, szBackend = wx.media.MEDIABACKEND_WMP10 )
            except:
                return
                
            self.ImgMC.Bind(wx.media.EVT_MEDIA_LOADED, self.OnPlay)
            #self.ImgMC.Bind(wx.media.EVT_MEDIA_FINISHED, self.OnFinished)

        
        idx = self.viewerTree.GetPyData(self.parent)
        
        con = sqlite.connect(self.imgFNList[idx])
        cur = con.cursor()
        cur.execute('SELECT audioData FROM audio where title=?', (audioTitle,))
        row = cur.fetchone()
        
        if self.mp3Path:
            os.remove(self.mp3Path)
        
        self.mp3Idx += 1
        self.mp3Path = '%s\\%s.mp3' % (tmpDir, self.mp3Idx)
       
        tmpout = open(self.mp3Path, 'wb')
        tmpout.write(row[0])
        tmpout.close()
            
        self.audioTitle = audioTitle    
        self.ImgMC.Load(self.mp3Path)
        


    def OnPlay(self, event):
        self.ImgMC.Play()

    def OnFinished(self, event):
        del self.ImgMC

    def OnCancel(self, event):
        keycode = event.GetKeyCode()
        if keycode == 27:
            if self.mc:
                self.mc.Stop()
            if self.mainApp.tts and self.mainApp.tts.tts.IsSpeaking():
                self.mainApp.tts.Stop()



    ## Image functions

    def imgCheck(self):
        imgPath = self.mainApp.allUsersDir + '/Modules/Image/'
        cnt = 1
        hit = False
        for fname in os.listdir(imgPath):
            if fname.endswith('img'):
                base, ext = os.path.splitext(fname)
                if not os.access(imgPath + base + '.imd', 1):
                    hit = True
                    dlg = wx.ProgressDialog('Converting Image Files.', 'This is just to make the dialog bigger', maximum = 12, \
                    style = wx.PD_AUTO_HIDE|wx.PD_APP_MODAL|wx.PD_REMAINING_TIME)
                    dlg.Update(cnt, 'Converting %s' % base)
                    tools.convertImg.convertImg(imgPath, base + '.img')
                    cnt += 1
                    dlg.Destroy()
        if hit:
            dlg = wx.MessageDialog(self.mainApp, 'The Image modules have been converted to an updated format.\nThey will appear in the Image Viewer when Bible Analyzer is restarted.',
            'Image File Conversion', wx.OK | wx.ICON_INFORMATION	)
            dlg.ShowModal()
            dlg.Destroy()
    

    def GetDBImages(self):
        for fname in self.mainApp.imdList:
            con = sqlite.connect(fname)
            cur = con.cursor()
            temp = []
            
            # Get main title
            cur.execute('SELECT * FROM title')
            row = cur.fetchone()
            self.imgGroupTitle.append(row[0])
            
            # Get image titles
            cur.execute('SELECT title FROM images')
            rows = cur.fetchall()
            temp = [(row[0]) for row in rows]
            self.imgTitles.append(temp)
            
            self.imgFNList.append(fname)
        
        self.GetUserImages()
        
    
    def GetUserImages(self):
        # User added images
        user = []
        userFn = []
        self.imgGroupTitle.append('User Images ')
        for fname in self.mainApp.userImgList:
            base = os.path.basename(fname)
            user.append(base[:-4].capitalize())
            userFn.append(fname)
        self.imgTitles.append(user)
        self.imgFileList.append(userFn)
        


    def BuildTree(self):
        idx = 0
        for group in self.imgGroupTitle:
            child = self.viewerTree.AppendItem(self.root, group[:-1])
            self.viewerTree.SetPyData(child, idx)
            self.viewerTree.SetItemBold(child, True)
            imgIdx = 0
            for img in self.imgTitles[idx]:
                subChild = self.viewerTree.AppendItem(child, img)
                self.viewerTree.SetPyData(subChild, imgIdx)
                imgIdx += 1
            
            idx += 1
    
        
    
    
    def OnHide(self, event):
        self.fullSizer.Hide(0, True)
        if self.Toggle == 0:
            self.fullSizer.Hide(0, True)
            self.Toggle = 1
        else:
            self.fullSizer.Show(0, True)
            self.Toggle = 0
        self.fullSizer.Layout()

    
    def OnExport(self, event):
        try:
            self.img
        except:
            dlg = wx.MessageDialog(self, 'You must select an image first.',
                                         'Select Image', wx.OK | wx.ICON_INFORMATION	)
            dlg.ShowModal()
            dlg.Destroy()
            return
        
        dlg = wx.FileDialog(self, message="Export JPG Image", defaultDir=os.getcwd(), defaultFile = "",
                            wildcard = '.jpg', style=wx.SAVE)
            
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.origImg.SaveFile(path, wx.BITMAP_TYPE_JPEG)
        
        dlg.Destroy()
    
        
    def OnPrint(self, event):
        img = self.fixPrint()
        page = '<body><img src="memory:print.jpg"></body>'
        self.ePrint.PrintText(page)
    
    
    def OnPrintPre(self, event):
        img = self.fixPrint()
        page = '<body><img src="memory:print.jpg"></body>'
        self.ePrint.PreviewText(page)
    
    
    def fixPrint(self):
        if not self.origImg:
            return
        
        img = self.origImg
        
        imgWidth = img.GetWidth()
        imgHeight = img.GetHeight()
        
        #print imgWidth, imgHeight, 'img1'
        
        data = self.ePrint.GetPrintData()
        orient = data.GetOrientation()
        
        if orient == 1:
            paperWid = 710
            paperHgt = 910
        else:
            paperWid = 910
            paperHgt = 710
        
        if imgWidth > paperWid or imgHeight > paperHgt:
            if self.ratio > 1:
                imgWidth = paperWid
                imgHeight = imgWidth / self.ratio
                if imgHeight > paperHgt:
                    imgHeight = paperHgt
                    imgWidth = imgHeight * self.ratio               
            else:
                imgHeight = paperHgt
                imgWidth = imgHeight * self.ratio
                if imgWidth > paperWid:
                    imgWidth = paperWid
                    imgHeight = imgWidth / self.ratio
                    
            
            img.Rescale(imgWidth, imgHeight)

        if self.inMemPrint:
            self.memFs.RemoveFile('print.jpg')
            self.inMemPrint = 0
        
        self.memFs.AddFile('print.jpg', img, wx.BITMAP_TYPE_JPEG)
        self.inMemPrint = 1
        #return img
        
    
    
    def OnSelChanged(self, event):
        if self.inMem:
            self.memFs.RemoveFile('img.jpg')
            self.inMem = 0
            
        self.mapData = None
        self.txtData = None

        item = event.GetItem()
        self.parent = self.viewerTree.GetItemParent(item)
        
        if self.parent == self.root:
               return
            
        imgIdx = self.viewerTree.GetPyData(item)
         
        if self.viewerTree.GetItemText(self.parent) == 'User Images':
            fn = self.imgFileList[-1][imgIdx]
            self.img = wx.Image(fn)
            
            self.origImg = wx.Image(fn)
            self.userImg = 1
        
        else:
            idx = self.viewerTree.GetPyData(self.parent)
            
            con = sqlite.connect(self.imgFNList[idx])
            cur = con.cursor()
            cur.execute('SELECT * FROM images where id=?', (imgIdx +1,))
            row = cur.fetchone()
            
            if len(row) > 3:
                try:
                    self.mapData = eval(row[3])
                except:
                    self.mapData = None

            if len(row) > 4:
                self.txtData = row[4]

            self.imageData = row[2]
            
            self.img = wx.ImageFromStream(StringIO.StringIO(self.imageData))
            self.origImg = wx.ImageFromStream(StringIO.StringIO(self.imageData))
            
            self.userImg = 0
                
        self.ratio = float(self.img.GetWidth())/float(self.img.GetHeight())
        
        self.winsize = self.Display.GetClientSize()
        self.width = self.winsize[0]
        self.height = self.winsize[1] 
        
        self.imgSize = (self.img.GetWidth(), self.img.GetHeight())
                
        if self.fitSideButton.GetValue() == True:
            self.OnFitSide(None)
        
        elif self.fitAllButton.GetValue() == True:
            self.OnFitAll(None)
        
        else:
            self.showImg(self.img)
        

    def showImg(self, img):
        wx.BeginBusyCursor()
        self.memFs.AddFile('img.jpg', img, wx.BITMAP_TYPE_JPEG)
        self.inMem = 1
        self.Display.LoadPage('memory:img.jpg')
        if self.txtData:
            self.Display.AppendToPage(self.txtData)
        #self.memFs.RemoveFile('img.jpg')
        wx.EndBusyCursor()


    def resizeWarning(self):
        dlg = wx.MessageDialog(self, 'This is a linked image; resizing it will destroy the link coordinates.\n\
To maintain proper coordinates, the image must be viewed at full size.',
    'Linked Image Resize Warning', wx.OK | wx.ICON_WARNING)
        dlg.ShowModal()
        dlg.Destroy()



    def OnFitAll(self, event):
        
        if self.inMem:
            self.memFs.RemoveFile('img.jpg')
            self.inMem = 0
        
        try:
            if self.fitAllButton.GetValue() == True:
                if self.mapData:
                    self.resizeWarning()

                self.fitSideButton.SetValue(False)
                
                size = self.Display.GetSize()
                if self.userImg == 0:
                    img = wx.ImageFromStream(StringIO.StringIO(self.imageData))
                else:
                    img = self.img
                    
                imgWidth = self.img.GetWidth()
                imgHeight = self.img.GetHeight()
                
                print imgWidth, imgHeight
                
                if self.ratio > 1:
                    imgWidth = self.width -30
                    imgHeight = imgWidth / self.ratio
                else:
                    imgHeight = self.height -30
                    imgWidth = imgHeight * self.ratio
                    
                print imgWidth, imgHeight
                
                img.Rescale(imgWidth, imgHeight)
                self.showImg(img)
                
            else:
                self.showImg(self.origImg)
        except:
            return
    
    def OnFitSide(self, event):
        
        if self.inMem:
            self.memFs.RemoveFile('img.jpg')
            self.inMem = 0

        try:
            if self.fitSideButton.GetValue() == True:
                if self.mapData:
                    self.resizeWarning()
                
                self.fitAllButton.SetValue(False)
                
                imgWidth = self.img.GetWidth()
                imgHeight = self.img.GetHeight()
                
                if self.userImg == 0:
                    img = wx.ImageFromStream(StringIO.StringIO(self.imageData))
                else:
                    img = self.img
                
                if self.ratio > 1:
                    imgHeight = self.height
                    imgWidth = imgHeight * self.ratio
                else:
                    imgWidth = self.width 
                    imgHeight = imgWidth / self.ratio

                img.Rescale(imgWidth -35, imgHeight -40)
                self.showImg(img)                
                
            else:
                self.showImg(self.origImg)        
        except: 
            return
    
    def OnPrintSetup(self, event):
        
        self.ePrint.PageSetup()
        
    
    
    
    def CreateMenu(self):
        viewMenu  = wx.Menu()
        fileMenu  = wx.Menu()
        printMenu  = wx.Menu()
        menu = wx.MenuBar()
        
        self.togHide = wx.MenuItem(viewMenu, 1005, "Hide/Show Image Tree\tCtrl+H", " Hide/Show Image Tree", wx.ITEM_NORMAL)
        viewMenu.AppendItem(self.togHide)

        self.close = wx.MenuItem(viewMenu, 1010, "Exit\tAlt+X", " Close Viewer", wx.ITEM_NORMAL)
        viewMenu.AppendItem(self.close)

        self.export = wx.MenuItem(fileMenu, 1001, "Export Current Image", " Export Current Image", wx.ITEM_NORMAL)
        fileMenu.AppendItem(self.export)

        self.printSetup = wx.MenuItem(printMenu, 1004, "Printer Setup", " Printer Setup", wx.ITEM_NORMAL)
        printMenu.AppendItem(self.printSetup)

        self.printImg = wx.MenuItem(printMenu, 1002, "Print Current Image", " Print Image", wx.ITEM_NORMAL)
        printMenu.AppendItem(self.printImg)
        
        self.printImgPre = wx.MenuItem(printMenu, 1003, "Print Preview Current Image", " Print preview for Image", wx.ITEM_NORMAL, None)
        printMenu.AppendItem(self.printImgPre)

        menu.Append(viewMenu, "&View")
        menu.Append(fileMenu, "&Export")
        menu.Append(printMenu, "&Print")
        self.addToolText = False
        return menu


    def OnClose(self, event):
        self.Hide()
        self.mainApp.Raise()



class MyApp(wx.App):
    def OnInit(self):
        #wx.InitAllImageHandlers()
        Viewer = ImageViewer(None, -1, "")
        #self.SetTopWindow(Viewer)
        Viewer.Show()
        return 1


if __name__ == "__main__":
    app = MyApp(0)
    app.MainLoop()
