# -*- coding: UTF-8 -*-

import os, re, wx, controlMini, toolImg, baseClass, tools.bookConvert
import mainPopup, utility
import wx.lib.buttons as buttons
import sqlite3 as sqlite




class TextReader(wx.Frame):
    def __init__(self, parent):
        wx.Frame.__init__(self, parent, style = wx.DEFAULT_FRAME_STYLE|wx.FRAME_FLOAT_ON_PARENT|wx.CLIP_CHILDREN)
        
        self.mainApp = wx.GetApp().GetTopWindow()
        self.bkData = self.mainApp.bkData
        
        #print self.bkData
        
        prevImg = toolImg.getPrevArrowBitmap()
        
        self.buttonPanel = wx.Panel(self, -1)
        self.Display = baseClass.HtmlWindow(self.buttonPanel, -1, style=wx.BORDER_THEME)
        self.back = buttons.GenBitmapButton(self.buttonPanel, -1, prevImg, size=(20, 22), style=wx.BORDER_NONE)
        self.close = buttons.GenButton(self.buttonPanel, -1, "Close")

        self.mPop = mainPopup.MainPopup(self, self.Display)


        self.Menu = self.CreateMenu()
        self.SetMenuBar(self.Menu)
        
        self.Display.SetName('textReader')
        
        self.ePrint = wx.html.HtmlEasyPrinting()
        self.ePrint.SetParentWindow(self)
        
        #self.display.Bind(wx.EVT_MOTION, self.mainApp.OnMotion)       
        #self.display.Bind(wx.EVT_LEAVE_WINDOW, self.mainApp.leaveWindow)
        
        self.Display.Bind(wx.EVT_MOTION, self.mPop.OnMotion)
        self.Display.Bind(wx.EVT_LEFT_DOWN, self.mPop.OnLeftDown)
        
        #self.display.Bind(wx.html.EVT_HTML_CELL_CLICKED, self.mainApp.OnDictClicked)
        
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        self.Bind(wx.EVT_BUTTON, self.OnClose, self.close)
        self.Bind(wx.EVT_BUTTON, self.OnBack, self.back)
        

        self.SetTitle("Library Viewer, Bible Analyzer")
        icon = toolImg.getScrollIcon()
        self.SetIcon(icon)
        self.SetSize((640, 480))
        
        #self.back.SetMinSize((50, 22))
        self.close.SetMinSize((50, 22))
        self.back.SetToolTipString('Go Back from Image')

        mainSizer = wx.BoxSizer(wx.VERTICAL)
        buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
        #mainSizer.Add(self.Menu, 0, wx.EXPAND)
        
        mainSizer.Add(self.Display, 1, wx.EXPAND, 0)
        buttonSizer.Add((20, 20), 1, 0, 0)
        buttonSizer.Add(self.back, 0, wx.ALL, 5)
        buttonSizer.Add((20, 20), 0, 0, 0)
        buttonSizer.Add(self.close, 0, wx.ALL, 5)
        #self.buttonPanel.SetSizer(buttonSizer)
        
        mainSizer.Add(buttonSizer, 0, wx.EXPAND, 0)
        self.buttonPanel.SetSizer(mainSizer)
        mainSizer.Fit(self)
        self.Layout()
        self.Centre()
        
        self.imgMapData = None
        
        
                

    def OnBack(self, event):
        if len(self.mainApp.bkWinDisplay.ToText()) == 0: # Image
            self.mainApp.bkWinDisplay.SetPage(self.mainApp.prePage)
        else:
            self.mainApp.cpMini.BookViewer.OnContents()

        
        
    def OnSelectAll(self, event):
        self.Display.SelectAll()

    def OnPrint(self, event):
        wx.html.HtmlEasyPrinting.PrintText(self.ePrint, self.mainApp.prePage)
        
    def OnPrintPre(self, event):
        wx.html.HtmlEasyPrinting.PreviewText(self.ePrint, self.mainApp.prePage)



    def OnClose(self, event):
        self.Hide()


    def CreateMenu(self):
        fileMenu  = wx.Menu()
        editMenu  = wx.Menu()
        printMenu  = wx.Menu()
        menu = wx.MenuBar()


        self.Back = wx.MenuItem(fileMenu, 1015, "Back\tCtrl+B", " Back from Image", wx.ITEM_NORMAL)
        fileMenu.AppendItem(self.Back)
        self.Bind(wx.EVT_MENU, self.OnBack, self.Back)

        self.preBook = wx.MenuItem(fileMenu, -1, "Previous Book\tAlt+F7", " View Previous Book", wx.ITEM_NORMAL)
        fileMenu.AppendItem(self.preBook)
        self.Bind(wx.EVT_MENU, self.mainApp.bookPanel.OnPrevBook, self.preBook)
        
        self.nextBook = wx.MenuItem(fileMenu, -1, "Next Book\tAlt+F8", " View Next Book", wx.ITEM_NORMAL)
        fileMenu.AppendItem(self.nextBook)
        self.Bind(wx.EVT_MENU, self.mainApp.bookPanel.OnNextBook, self.nextBook)

        fileMenu.AppendSeparator()
        
        self.preChap = wx.MenuItem(fileMenu, -1, "Previous Chapter\tAlt+F5", " View Previous Chapter", wx.ITEM_NORMAL)
        fileMenu.AppendItem(self.preChap)
        self.Bind(wx.EVT_MENU, self.mainApp.bookPanel.OnPrevChap, self.preChap)
        
        self.nextChap = wx.MenuItem(fileMenu, -1, "Next Chapter\tAlt+F6", " View Next Chapter", wx.ITEM_NORMAL)
        fileMenu.AppendItem(self.nextChap)
        self.Bind(wx.EVT_MENU, self.mainApp.bookPanel.OnNextChap, self.nextChap)


        fileMenu.AppendSeparator()
        
        self.selectAll = wx.MenuItem(editMenu, -1, "Select All\tCtrl+A", " Select All", wx.ITEM_NORMAL)
        editMenu.AppendItem(self.selectAll)
        self.Bind(wx.EVT_MENU, self.OnSelectAll, self.selectAll)
        
        self.copy = wx.MenuItem(editMenu, 1009, "Copy Selected\tCtrl+C", " Copy Selected", wx.ITEM_NORMAL)
        editMenu.AppendItem(self.copy)
        self.Bind(wx.EVT_MENU, self.mainApp.OnCopy, self.copy)

        close = wx.MenuItem(editMenu, 1010, "Exit\tAlt+X", " Close Viewer", wx.ITEM_NORMAL)
        editMenu.AppendItem(close)

        self.printPage = wx.MenuItem(fileMenu, -1, "Print Current Page", " Print last opened Library", wx.ITEM_NORMAL)
        printMenu.AppendItem(self.printPage)
        self.Bind(wx.EVT_MENU, self.OnPrint, self.printPage)
        
        self.printPre = wx.MenuItem(fileMenu, -1, "Print Preview Current Page", " Print preview for Bible", wx.ITEM_NORMAL, None)
        printMenu.AppendItem(self.printPre)
        self.Bind(wx.EVT_MENU, self.OnPrintPre, self.printPre)


        menu.Append(editMenu, "&Edit")
        menu.Append(fileMenu, "&View")
        menu.Append(printMenu, "&Print")
        self.addToolText = False
        return menu




class BookViewer(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        
        self.mainApp = wx.GetApp().GetTopWindow()
        self.bkData = self.mainApp.bkData
        self.checkState = utility.CheckWinState()
        
        self.titles = {}
        
        self.Tree = wx.TreeCtrl(self, -1, style=wx.TR_HAS_BUTTONS|wx.TR_LINES_AT_ROOT|wx.TR_HIDE_ROOT|wx.BORDER_THEME) 

        self.libSizer = wx.BoxSizer(wx.VERTICAL)
        title = controlMini.FMTitlePanel(self, "Library: Books & Articles")
        self.libSizer.Add(title, 0, wx.EXPAND|wx.ALL, 5)
        self.libSizer.Add(self.Tree, 1, wx.EXPAND|wx.LEFT|wx.RIGHT, 5)
        self.SetSizer(self.libSizer)
        self.Layout()

        self.root = self.Tree.AddRoot('Top')
        self.BuildTree()

        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnSelChanged)
        self.oldParent = None
        self.oldItem = None
        
        #print self.bkData[0]
    

    def BuildTree(self):
        idx = 0
        groups = {}
        
        for group in self.bkData:
            #print group[4]
            
            if group[4]:                                                # If group title
                #print group[4]
                if not group[4] in groups:                              # Append group name
                    groupTitle = self.Tree.AppendItem(self.root, group[4])
                    self.Tree.SetPyData(groupTitle, idx)
                    self.Tree.SetItemBold(groupTitle, True)
                    self.Tree.SetItemTextColour(groupTitle, wx.Colour(0, 0, 150))
                    self.titles[group[4]] = groupTitle

                    groups[group[4]] = groupTitle
                else:                                                   # If already there
                    groupTitle = groups[group[4]]
                    
                bkTitle = self.Tree.AppendItem(groupTitle, group[1].replace('[q]', '"'))
                self.Tree.SetItemTextColour(bkTitle, wx.Colour(0, 0, 150))
                title = '%s/%s' % (group[4].strip(' \n'), group[1].strip(' \n'))
                self.titles[title] = bkTitle
            
            else:
                bkTitle = self.Tree.AppendItem(self.root, group[1].replace('[q]', '"'))
                self.Tree.SetItemTextColour(bkTitle, wx.Colour(55, 55, 55))
                self.titles[group[1].strip(' \n')] = bkTitle 
                #print bkTitle


            self.Tree.SetPyData(bkTitle, idx)
            self.Tree.SetItemBold(bkTitle, True)
            
            chapIdx = 1
            for chap in group[3]:

                subChild = self.Tree.AppendItem(bkTitle, chap.replace('[q]', '"'))
                self.Tree.SetPyData(subChild, chapIdx)
                self.Tree.SetItemTextColour(subChild, wx.Colour(25, 25, 70))
                title = '%s/%s' % (group[1].strip(' \n'), chap)
                self.titles[title] = subChild 
                chapIdx += 1           
                
            idx += 1
        
        self.Tree.SortChildren(self.Tree.GetRootItem())
        #print 'sort tree'
        self.Tree.SetFocus()
            
        
        #t = self.titles.keys()
        #for i in t:
        #    if 'Weakness' in i:
        #        print i.encode('utf-8'), 'titles'

    
    def OnSelChanged(self, event):
        #print event.GetEventObject(), 'sel event ob'
        #print id(event)
        #event.Skip()
        
        item = event.GetItem()
        
        if item == self.oldItem:
            print 'Same node'
            #return
        
        self.oldItem = item
        
        self.mainApp.imgMapData = None
        
        itemTxt = self.Tree.GetItemText(item).strip(' \n')
        parent = self.Tree.GetItemParent(item)
        
        #print itemTxt.encode('utf-8'), 'item'
        
        if parent == self.root:                                     # For contents list, a group or book title (root is parent)
            lnkList = []
            chapIdx = self.Tree.GetPyData(item)

            if not self.bkData or chapIdx > len(self.bkData):       # A shorter Session list of titles
                return
            
            first, cookie = self.Tree.GetFirstChild(item)
            
            if self.Tree.ItemHasChildren(first):                    # A group title
                iTxt = self.Tree.GetItemText(first).strip(' \n')
                lnk = '<a href="book://%s/%s"><nu>%s</nu></a>' % (itemTxt.replace('"', '[q]'), iTxt.replace('"', '[q]'), iTxt.replace('[q]', '"'))
                lnkList.append(lnk)
                
                next = first
                while self.Tree.GetNextSibling(next):
                    next = self.Tree.GetNextSibling(next)
                    iTxt = self.Tree.GetItemText(next).strip(' \n')
                    lnk = '<a href="book://%s/%s"><nu>%s</nu></a>' % (itemTxt.replace('"', '[q]'), iTxt.replace('"', '[q]'), iTxt.replace('[q]', '"'))
                    lnkList.append(lnk)
                    
            else:                                                   # A book title
                for t in self.bkData[chapIdx][3]:
                    if len(t) == 0:
                        continue
                    ind = ''
                    if t[0] == ' ':
                        cnt = len(re.search(r'^ +', t).group(0))
                        ind = '&nbsp;' *cnt

                    tLnk = t.replace('"', '[q]')
                    lnk = '%s<a href="book://%s/%s"><nu>%s</nu></a>' % (ind, itemTxt.replace('"', '[q]'), tLnk, t.replace('[q]', '"'))
                    lnkList.append(lnk)

            htm = '<body link="#336666"><div align="center">\
            <table><tr><td><font size="-1"><a href="book://goCont"><nu>Book&nbsp;/&nbsp;Article&nbsp;Titles</nu></a></font></td><tr></table>\
            <table width="90%%" align="center" bgcolor="#CDCFDC" cellpadding="5">\
            <tr><td bgcolor="#F9F9F9">\
            <font size="+1" color="#707070" size="+1"><i>%s</i></font></td></tr></tbody></table></div>\
            <hr width="90%%"><br>\
            <div align="center"><table width="90%%" bgcolor="#CDCFDC" cellpadding="15">\
            <tr><td bgcolor="#F9F9F9">%s</td></tr></table></div>' % (itemTxt.replace('[q]', '"'), '<br>'.join(lnkList))

            self.displayBk(htm)
            return
        
        elif self.Tree.GetItemParent(parent) and self.Tree.ItemHasChildren(item):      # A volume title under a group (root is grandparent and item has children)
            lnkList = []
            chapIdx = self.Tree.GetPyData(item)
            for t in self.bkData[chapIdx][3]:
                
                if len(t) == 0:
                    continue
                ind = ''
                if t[0] == ' ':
                    cnt = len(re.search(r'^ +', t).group(0))
                    ind = '&nbsp;' *cnt
                
                lnk = '%s<a href="book://%s/%s"><nu>%s</nu></a>' % (ind, itemTxt.replace('"', '[q]'), t.replace('"', '[q]'), t.replace('[q]', '"'))
                lnkList.append(lnk)
                
            parentTxt = self.Tree.GetItemText(parent).strip(' \n')
            title = '%s / %s' % (parentTxt.replace('[q]', '"'), itemTxt.replace('[q]', '"'))
                
            htm = '<body link="#336666"><div align="center">\
            <table><tr><td><font size="-1"><a href="book://goCont"><nu>%s</nu></a></font></td></tr></table>\
            <table width="90%%" align="center" bgcolor="#CDCFDC" cellpadding="5">\
            <tr><td bgcolor="#F8F8F8">\
            <font size="+1" color="#707070" size="+1"><i>%s</i></font></td><tr></table></div>\
            <hr width="90%%"><br>\
            <div align="center"><table width="90%%" bgcolor="#CDCFDC" cellpadding="15">\
            <tr><td bgcolor="#F8F8F8">%s</td></tr></table></div>'\
            % (parentTxt.replace('[q]', '"'), title.replace('[q]', '"'), '<br>'.join(lnkList))
            
            self.displayBk(htm)
            return

        self.mainApp.bookTitle = self.Tree.GetItemText(parent).strip(' \n')
        
        bookIdx = self.Tree.GetPyData(parent)
        chapIdx = self.Tree.GetPyData(item)
        
        #print self.bkData[bookIdx]
        #print len(self.bkData[bookIdx]), 'bookIdx len'
        
        if self.bkData[bookIdx][5]:
            self.mainApp.imgMapData = self.bkData[bookIdx][5][chapIdx -1]
            #print self.bkData[bookIdx][3][chapIdx]
        
        #print self.mainApp.imgMapData[:5], 'map data'
    
        con = sqlite.connect(self.bkData[bookIdx][2])
        con.text_factory = str
        cur = con.cursor()    
       
        cur.execute('SELECT text FROM book WHERE id=?', (chapIdx,))
        
        try:
            txt = cur.fetchone()[0]
        except:
            print 'book tree selection error'
            return
            
        if isinstance(txt, str):    
            try:
                txt = unicode(txt, "utf-8")
            except:
                txt = unicode(txt, "latin1")
                print 'latin1 encoding'
        
        #print self.bkData[bookIdx]
        
        vol = ''
        if self.bkData[bookIdx][4]:
            vol = '%s / ' % self.bkData[bookIdx][4]
        
        title = '%s%s / %s' % (vol.replace('[q]', '"'), self.bkData[bookIdx][1].strip(' \n').replace('[q]', '"'), itemTxt.replace('[q]', '"'))
        
        txt = '<body link="#336666"><div align="center">\
        <table><tr><td><font size="-1"><a href="book://goPrev"><nu>Previous</nu></a> - <a href="book://goCont"><nu>Contents</nu></a> - <a href="book://goNext"><nu>Next</nu></a></font></td></tr></table>\
        <table width="90%%" align="center" bgcolor="#CDCFDC" cellpadding="5"><tr><td bgcolor="#F8F8F8">\
        <font color="#707070" size="+1"><i>%s</i></font>\
        </td><tr></table></div>\
        <br><br><br>%s<br><br><br>\
        <center><font size="-1"><a href="book://goPrev"><nu>Previous</nu></a> - <a href="book://goCont"><nu>Contents</nu></a> - \
        <a href="book://goNext"><nu>Next</nu></a></font></center>'\
        % (title.replace('[q]', '"'), txt.replace('[q]', '"'))
        
        txt = u'<font face="%s">%s</font>' % (self.mainApp.font, txt)
        
        txt = u'%s</font>' % txt
        #txt = re.sub(r'(<u>.*?</u>)', r'<font color="#336666">\1</font>', txt)
        #txt = re.sub(r'<u>(\w\w\w)_([^<]+)</u>', r'<a href="bible://\1 \2"><nu>\1 \2</nu></a>', txt)
        txt = re.sub(r'<u>(\w\w\w)_([^<]+)</u>', r'<a href="bible://\1 \2">\1 \2</a>', txt)
        
        #txt = re.sub(r'<font size=', r'<font face="%s" size=' % self.mainApp.font, txt)
        #print txt
        
        
        # Fix image links in htm
        txt = re.sub(r'="modules/', r'="%s' % self.mainApp.allUsersDir +'/modules/', txt)
        #print txt.encode('utf-8')
        self.displayBk(txt)


    def displayBk(self, txt):
        grkFont = '<font face="Galaxie Unicode Greek" color="brown">'
        hebFont = '<font face="Galaxie Unicode Hebrew" color="brown">'
        
        txt = re.sub(r'<gu>(.*?)</gu>', r'%s\1</font>' % grkFont, txt)
        txt = re.sub(r'<hu>(.*?)</hu>', r'%s\1</font>' % hebFont, txt)
        
        if self.mainApp.fontSize > 0:
            txt = '<font size="+%s">%s</font>' % (self.mainApp.fontSize, txt)
        
        #print txt
        
        useBkWinTxt = '<center><font color="navy"><b>Books are set in <i>Preferences</i> to display in a separate window.</b></font><br><br>\
        To view book contents again in this window (default),<br>\
        uncheck the <i>Show Books in Separate Window</i> option in Preferences.</center>'
        
        if self.mainApp.showBkWin == 0:
            #self.mainApp.Freeze()
            self.mainApp.bookDisplay.SetPage(txt)
            self.mainApp.resultCurrent[0] = txt
            self.mainApp.prePage = txt
            self.checkState.CheckState()
            #self.mainApp.Thaw()
            
            #dispBox = self.mainApp.bookDisplay.GetGrandParent().GetSizer().GetStaticBox()
            #dispBox.SetLabel(bkData[bookIdx][1] + ' -- ' + bkData[bookIdx][3][chapIdx -1])
        else:
            self.mainApp.OnBkFrame()
            display = self.mainApp.bkWinDisplay
            display.SetPage(txt)
            
            self.mainApp.bookDisplay.SetPage(useBkWinTxt)

            self.mainApp.bkWin.Show()
            self.mainApp.prePage = txt
    
        self.mainApp.resNB.SetSelection(1)
        #if self.mainApp.getMaximized():
            #print 'is res max'
            #self.mainApp.mainMgr.RestoreMaximizedPane()
            #self.mainApp.mainMgr.MaximizePane(self.mainApp.getResPane())
            #self.mainApp.mainMgr.Update()

    
    def OnContents(self):
        current = self.Tree.GetSelection()
        try:
            parent = self.Tree.GetItemParent(current)
            if parent != self.root:
                self.Tree.SelectItem(parent)
            else:
                self.ShowRootItems()
        except:
            self.ShowRootItems()
    

    def ShowRootItems(self):
        lnkList = []
        first, cookie = self.Tree.GetFirstChild(self.root)
        
        try:
            iTxt = self.Tree.GetItemText(first).strip(' \n')
            if self.Tree.GetItemTextColour(first) != wx.Colour(55, 55, 55):
                iTxt = '<b>%s</b>' % iTxt
        except:
            print 'No tree item'
            return
            
        lnk = '<a href="book://%s"><nu>%s</nu></a>' % (iTxt.replace('"', '[q]'), iTxt)
        lnkList.append(lnk)
        
        next = first
        while self.Tree.GetNextSibling(next):
            next = self.Tree.GetNextSibling(next)
            iTxt = self.Tree.GetItemText(next).strip(' \n')
            if self.Tree.GetItemTextColour(next) != wx.Colour(55, 55, 55):
                iTxt = '<b>%s</b>' % iTxt
            
            lnk = '<a href="book://%s"><nu>%s</nu></a>' % (iTxt.replace('"', '[q]'), iTxt)
            lnkList.append(lnk)
        
        htm = '<body link="#336666"><div align="center">\
        <table width="90%%" align="center" bgcolor="#CDCFDC" cellpadding="5"><tr><td bgcolor="#F8F8F8">\
        <font size="+1" color="#707070"><i>%s</i></font><br>\
        <font size="-1" color="gray" size="+1"><i>Titles Can Also Be Viewed in the Master Control Panel (F3)\
        <br>(Titles are Links - Click to Navigate) </i></font></td><tr></table></div>\
        <hr width="90%%"><br>\
        <div align="center"><table width="90%%" bgcolor="#CDCFDC" cellpadding="15">\
        <tr><td bgcolor="#F8F8F8">%s</td><tr></table></div>' % ('Book / Article Titles', '<br>'.join(lnkList))
        
        self.displayBk(htm)
        return
        
    
    def OnNextChap(self, event):
        current = self.Tree.GetSelection()
        if self.Tree.ItemHasChildren(current):
            current = self.Tree.GetFirstChild(current)
            next = current[0]
        else:
            next = self.Tree.GetNextSibling(current)
            if next.IsOk():
                self.Tree.SelectItem(next)
            else:
                book = self.Tree.GetItemParent(current)
                if book.IsOk():
                    nextBook = self.Tree.GetNextSibling(book)
                    if nextBook.IsOk():                    
                        first = self.Tree.GetFirstChild(nextBook)
                        self.Tree.SelectItem(first[0])


    def OnPrevChap(self, event):
        current = self.Tree.GetSelection()
        prev = self.Tree.GetPrevSibling(current)
        if prev.IsOk():
            self.Tree.SelectItem(prev)
        else:
            book = self.Tree.GetItemParent(current)
            if book.IsOk():
                prevBook = self.Tree.GetPrevSibling(book)
                if prevBook.IsOk():                    
                    last = self.Tree.GetLastChild(prevBook)
                    self.Tree.SelectItem(last)


    def OnPrevBook(self, event):
        current = self.Tree.GetSelection()
        if self.Tree.ItemHasChildren(current):
            current = self.Tree.GetFirstChild(current)
            current = current[0]
        book = self.Tree.GetItemParent(current)
        nextBook = self.Tree.GetPrevSibling(book)
        if nextBook.IsOk():
            first = self.Tree.GetFirstChild(nextBook)
            self.Tree.SelectItem(first[0])
    
    def OnNextBook(self, event):
        current = self.Tree.GetSelection()
        if self.Tree.ItemHasChildren(current):
            current = self.Tree.GetFirstChild(current)
            current = current[0]
        book = self.Tree.GetItemParent(current)
        nextBook = self.Tree.GetNextSibling(book)
        if nextBook.IsOk():
            first = self.Tree.GetFirstChild(nextBook)
            self.Tree.SelectItem(first[0])

