import wx, os
import mainImg
import wx
import genButton as buttons
import dialogs
import parGen
import fixRef

import flatMenu as FM

import agw.aui as Aui



class SegmentButton(wx.PyPanel):
    def __init__(self, parent, id, size, segments, labels, events, tips):
        wx.PyPanel.__init__(self, parent, id)
        self.mainApp = wx.GetApp().GetTopWindow()
        
        assert len(labels) == segments
        
        segSize = (size[0] /2, size[1] /2) # 4 segments
        if segments == 3:
            segSize = (size[0], size[1] /3.33)
        if segments == 2:
            segSize = (size[0], size[1] /2)
        
        segList = []
        
        for seg, num in enumerate(range(segments)):
            b = buttons.GenButton(self, -1, labels[num], size = (segSize[0], segSize[1]))
            b.SetUseFocusIndicator(False)
            #b.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
            b.SetForegroundColour(wx.Colour(128, 0, 0))
            b.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
            b.SetBezelWidth(1)
            b.SetToolTipString(tips[num])
            self.Bind(wx.EVT_BUTTON, events[num], b)
            segList.append(b)
        
        gsRows = (2, 2) # 4 segments
        if segments == 3:
            gsRows = (3, 1)
        elif segments == 2:
            gsRows = (2, 1)
        
        gs = wx.GridSizer(gsRows[0], gsRows[1])
        for b in segList:
            gs.Add(b, 0, wx.EXPAND)

        self.SetSizer(gs)





class MainMenu(FM.FlatMenuBar):
    def __init__(self, parent):
        FM.FlatMenuBar.__init__(self, parent, -1, 16, 2, options = FM.FM_OPT_IS_LCD)
        
        self.mainApp = parent
        self.curNBInfo = None
        self.oldNBPageInfo = None
        
        fileMenu  = FM.FlatMenu()
        self.viewMenu  = FM.FlatMenu()
        smView = FM.FlatMenu()
        #self.layMenu = FM.FlatMenu()
        ctrlMenu = FM.FlatMenu()
        toolsMenu = FM.FlatMenu()
        editorMenu = FM.FlatMenu()
        helpMenu = FM.FlatMenu()
        self.bmMenu = FM.FlatMenu()
        
        paraSM = FM.FlatMenu()
        hiLiteSM = FM.FlatMenu()

            

        # File Menu
        sAll = FM.FlatMenuItem(fileMenu, -1, "&Select All on current page\tCtrl+A", " Select all text in panel", wx.ITEM_NORMAL)
        fileMenu.AppendItem(sAll)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnSelectAll, sAll)
        
        copy = FM.FlatMenuItem(fileMenu, -1, "&Copy Selected on current page\tCtrl+C", " Copy selected text", wx.ITEM_NORMAL)
        fileMenu.AppendItem(copy)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnCopy, copy)

        fileMenu.AppendSeparator()

        copy = FM.FlatMenuItem(fileMenu, -1, "Export/Copy verse(s)\tCtrl+E", " Export/Copy verse(s) to File, S-Pad, or Clipboard", wx.ITEM_NORMAL)
        fileMenu.AppendItem(copy)
        self.mainApp.Bind(wx.EVT_MENU, self.ExportBib, copy)

        fileMenu.AppendSeparator()

        copy = FM.FlatMenuItem(fileMenu, -1, "&Print current page\tCtrl+P", " Print current page", wx.ITEM_NORMAL)
        fileMenu.AppendItem(copy)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnPrint, copy)

        copy = FM.FlatMenuItem(fileMenu, -1, "Print Preview current page\tAlt+Ctrl+P", " Print Preview current page", wx.ITEM_NORMAL)
        fileMenu.AppendItem(copy)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnPrintPre, copy)

        fileMenu.AppendSeparator()
                
        exit = FM.FlatMenuItem(fileMenu, -1, "Exit\tAlt+X", "Exit Bible Analyzer", wx.ITEM_NORMAL)
        fileMenu.AppendItem(exit)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnClose, exit)
        
        
        #View Menu        
        entryTB = FM.FlatMenuItem(smView, -1, "Hide/Show Entry Toolbar", "", wx.ITEM_CHECK, None)
        smView.AppendItem(entryTB)
        self.entryTbId = entryTB.GetId()
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHideEntryTB, entryTB)
        
        messageTB = FM.FlatMenuItem(smView, -1, "Hide/Show Message Toolbar", "", wx.ITEM_CHECK, None)
        smView.AppendItem(messageTB)
        self.messageTbId = messageTB.GetId()
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHideMessageTB, messageTB)

        helpTB = FM.FlatMenuItem(smView, -1, "Hide/Show Help Toolbar", "", wx.ITEM_CHECK, None)
        smView.AppendItem(helpTB)
        self.helpTbId = helpTB.GetId()
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHideHelpTB, helpTB)

        viewerTB = FM.FlatMenuItem(smView, -1, "Hide/Show Viewer Toolbar", "", wx.ITEM_CHECK, None)
        smView.AppendItem(viewerTB)
        self.viewerTbId = viewerTB.GetId()
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHideViewerTB, viewerTB)

        buttonTB = FM.FlatMenuItem(smView, -1, "Hide/Show Button Toolbar", "", wx.ITEM_CHECK, None)
        smView.AppendItem(buttonTB)
        self.buttonTbId = buttonTB.GetId()
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHideButtonTB, buttonTB)

        self.viewMenu.AppendMenu(-1, "Toolbars", smView, wx.ITEM_NORMAL)
        self.viewMenu.AppendSeparator()
        
        
        self.layoutMenu(None)
        
        self.viewMenu.AppendMenu(-1, "Select/Create Layouts", self.layMenu, wx.ITEM_NORMAL)
        self.viewMenu.AppendSeparator()
        
        bibExp = FM.FlatMenuItem(self.viewMenu, -1, "View/Expand/Contract Bible Window\tAlt+B", " View/Expand/Contract Bible Window", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(bibExp)
        self.mainApp.Bind(wx.EVT_MENU, self.bibMax, bibExp)
        
        libExp = FM.FlatMenuItem(self.viewMenu, -1, "View/Expand/Contract Dictionary Window\tAlt+D", " View/Expand/Contract Dictionary Window", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(libExp)
        self.mainApp.Bind(wx.EVT_MENU, self.dctMax, libExp)
        
        resExp = FM.FlatMenuItem(self.viewMenu, -1, "View/Expand/Contract Results Window\tAlt+S", " View/Expand/Contract Results Window", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(resExp)
        self.mainApp.Bind(wx.EVT_MENU, self.resMax, resExp)
        
        cmtExp = FM.FlatMenuItem(self.viewMenu, -1, "View/Expand/Contract Commentary Window\tAlt+C", " View/Expand/Contract Commentary Window", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(cmtExp)
        self.mainApp.Bind(wx.EVT_MENU, self.cmtMax, cmtExp)
        
        self.viewMenu.AppendSeparator()

        hideTree = FM.FlatMenuItem(self.viewMenu, -1, "Hide/Show Tree Panel\tAlt+T", " Hide/Show Tree Panel", wx.ITEM_CHECK, None)
        self.viewMenu.AppendItem(hideTree)
        self.treeShowId = hideTree.GetId()
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHideTree, hideTree)

        colTree = FM.FlatMenuItem(self.viewMenu, -1, "Collapse Bible Tree Books\tAlt+L", " Collapse Bible Tree Books", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(colTree)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.treeCollapse, colTree)
        
        self.viewMenu.AppendSeparator()
                
        nVerse = FM.FlatMenuItem(self.viewMenu, -1, "Next Bible Verse\tF6", " Next Bible Verse", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(nVerse)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnNextVerse, nVerse)

        pVerse = FM.FlatMenuItem(self.viewMenu, -1, "Previous Bible Verse\tF5", " Previous Bible Verse", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(pVerse)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnPreVerse, pVerse)
        
        nChap = FM.FlatMenuItem(self.viewMenu, -1, "Next Bible Chapter\tF8", " Next Bible Chapter", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(nChap)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnNextChap, nChap)

        pChap = FM.FlatMenuItem(self.viewMenu, -1, "Previous Bible Chapter\tF7", " Previous Bible Chapter", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(pChap)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnPreChap, pChap)

        self.viewMenu.AppendSeparator()

        nBookChap = FM.FlatMenuItem(self.viewMenu, -1, "Next Book Chapter\tAlt+F6", " Next Book Chapter", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(nBookChap)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnNextBookChap, nBookChap)

        pBookChap = FM.FlatMenuItem(self.viewMenu, -1, "Previous Book Chapter\tAlt+F5", " Previous Book Chapter", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(pBookChap)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnPreBookChap, pBookChap)
        
        bibRef = FM.FlatMenuItem(self.viewMenu, -1, "Toggle Bible Reference Panel\tCtrl+R", " Previous Book Chapter", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(bibRef)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnRefPanel, bibRef)
        
        self.viewMenu.AppendSeparator()

        fontPlus = FM.FlatMenuItem(self.viewMenu, -1, "Increase Windows Font Size\tCtrl++", " Previous Book Chapter", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(fontPlus)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnFontPlus, fontPlus)

        fontMinus = FM.FlatMenuItem(self.viewMenu, -1, "decrease Windows Font Size\tCtrl+-", " Previous Book Chapter", wx.ITEM_NORMAL, None)
        self.viewMenu.AppendItem(fontMinus)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnFontMinus, fontMinus)



        #Control Menu
        master = FM.FlatMenuItem(ctrlMenu, 1005, "Master Control Panel\tF2", " Toggle Master Control Panel", wx.ITEM_NORMAL)
        ctrlMenu.AppendItem(master)
        
        readView = FM.FlatMenuItem(ctrlMenu, 1003, "Book/Article Titles\tF3", " Open Book/Article Titles List", wx.ITEM_NORMAL, None)
        ctrlMenu.AppendItem(readView)
        
        imgView = FM.FlatMenuItem(ctrlMenu, 1004, "Image Viewer\tF4", " Open Image Viewer", wx.ITEM_NORMAL, None)
        ctrlMenu.AppendItem(imgView)
        
        devotions = FM.FlatMenuItem(ctrlMenu, 1002, "Devotions Viewer\tF9", " Open Daily Devotions Viewer", wx.ITEM_NORMAL, None)
        ctrlMenu.AppendItem(devotions)

        multiWin = FM.FlatMenuItem(ctrlMenu, 1010, "MultiWindow\tF10", " Toggle MultiWindow Viewer", wx.ITEM_NORMAL, None)
        ctrlMenu.AppendItem(multiWin)

        optDlg = FM.FlatMenuItem(ctrlMenu, 1006, "Preferences Dialog\tF11", " Open Preferences Dialog", wx.ITEM_NORMAL, None)
        ctrlMenu.AppendItem(optDlg)

        miniWin = FM.FlatMenuItem(ctrlMenu, 1008, "Audio/Quiz Panel\tF12", " Open Audio/Quiz Panel", wx.ITEM_NORMAL, None)
        ctrlMenu.AppendItem(miniWin)

        parGen = FM.FlatMenuItem(ctrlMenu, 1011, "Harmony/Parallel Generator\tAlt+G", " Harmony/Parallel Generator", wx.ITEM_NORMAL, None)
        ctrlMenu.AppendItem(parGen)

        ctrlMenu.AppendSeparator()

        QSearchFocus = FM.FlatMenuItem(ctrlMenu, -1, "Quick Find Entry (focus)\tAlt+Q", " Go to Quick Search/Ref Entry ", wx.ITEM_NORMAL, None)
        ctrlMenu.AppendItem(QSearchFocus)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.SetQSearchFocus, QSearchFocus)

        notesFocus = FM.FlatMenuItem(ctrlMenu, -1, "Show User Notes\tAlt+E", " Show User Notes Tab ", wx.ITEM_NORMAL, None)
        ctrlMenu.AppendItem(notesFocus)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnNotesFocus, notesFocus)


        ctrlMenu.AppendSeparator()

        ttsSpeak = FM.FlatMenuItem(ctrlMenu, -1, "Speak/Stop Current Text\tCtrl+F10", " Speak\Stop Current Text (Toggle)", wx.ITEM_NORMAL, None)
        ctrlMenu.AppendItem(ttsSpeak)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnTTSSpeak, ttsSpeak)

        pause = FM.FlatMenuItem(ctrlMenu, -1, "Pause/Play Audio\tAlt+/", " Pause Audio Player", wx.ITEM_NORMAL, None)
        ctrlMenu.AppendItem(pause)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnPause, pause)


        #Editor Menu
        new = FM.FlatMenuItem(editorMenu, -1, "New\tCtrl+N", " New document in ScripturePad", wx.ITEM_NORMAL)
        editorMenu.AppendItem(new)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnFileClose, new)
        
        open = FM.FlatMenuItem(editorMenu, -1, "Open\tCtrl+O", " Open file in ScripturePad", wx.ITEM_NORMAL)
        editorMenu.AppendItem(open)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnFileOpen, open)
        
        save = FM.FlatMenuItem(editorMenu, -1, "Save\tCtrl+S", " Save ScripturePad File", wx.ITEM_NORMAL)
        editorMenu.AppendItem(save)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnFileSave, save)        
        
        saveas = FM.FlatMenuItem(editorMenu, -1, "Save As", " Save ScripturePad File As", wx.ITEM_NORMAL)
        editorMenu.AppendItem(saveas)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnFileSaveAs, saveas)

        close = FM.FlatMenuItem(editorMenu, -1, "Close\tCtrl+W", " Close current ScripturePad File", wx.ITEM_NORMAL)
        editorMenu.AppendItem(close)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnFileClose, close)


        editorMenu.AppendSeparator()
        
        cut = FM.FlatMenuItem(editorMenu, -1, "Cut Selection\tCtrl+X", " Cut Selection", wx.ITEM_NORMAL)
        editorMenu.AppendItem(cut)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnEditorCut, cut)
        
        copy = FM.FlatMenuItem(editorMenu, -1, "Copy Selection", " Copy Selection", wx.ITEM_NORMAL)
        editorMenu.AppendItem(copy)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnEditorCopy, copy)
        
        paste = FM.FlatMenuItem(editorMenu, -1, "Paste from Clipboard\tCtrl+V", " Paste from Clipboard", wx.ITEM_NORMAL)
        editorMenu.AppendItem(paste)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnEditorPaste, paste)
        
        select = FM.FlatMenuItem(editorMenu, -1, "Select All", " Select All", wx.ITEM_NORMAL)
        editorMenu.AppendItem(select)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnEditorSelectAll, select)

        editorMenu.AppendSeparator()

        undo = FM.FlatMenuItem(editorMenu, -1, "Undo\tCtrl+Z", " Undo", wx.ITEM_NORMAL)
        editorMenu.AppendItem(undo)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnUndo, undo)

        redo = FM.FlatMenuItem(editorMenu, -1, "Redo", " Redo", wx.ITEM_NORMAL)
        editorMenu.AppendItem(redo)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnRedo, redo)

        editorMenu.AppendSeparator()

        prnt = FM.FlatMenuItem(editorMenu, -1, "Print\tCtrl+P", " Print Page", wx.ITEM_NORMAL)
        editorMenu.AppendItem(prnt)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnEditorPrint, prnt)

        printPre = FM.FlatMenuItem(editorMenu, -1, "Print Preview", " Print Preview", wx.ITEM_NORMAL)
        editorMenu.AppendItem(printPre)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnEditorPrintPre, printPre)

        editorMenu.AppendSeparator()

        bold = FM.FlatMenuItem(editorMenu, -1, "Bold\tCtrl+B", " Make selection bold", wx.ITEM_NORMAL)
        editorMenu.AppendItem(bold)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnBold, bold)
        
        italics = FM.FlatMenuItem(editorMenu, -1, "Italics\tCtrl+I", " Make selection italics", wx.ITEM_NORMAL)
        editorMenu.AppendItem(italics)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnItalic, italics)
        
        underline = FM.FlatMenuItem(editorMenu, -1, "Underline\tCtrl+U", " Underline selection", wx.ITEM_NORMAL)
        editorMenu.AppendItem(underline)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnUnderline, underline)
        
        editorMenu.AppendSeparator() 
        
        makeRefLinks = FM.FlatMenuItem(editorMenu, -1, "Make All Refs. Links\tCtrl+L", " Attempt to make all Refs. links in Page", wx.ITEM_NORMAL)
        editorMenu.AppendItem(makeRefLinks)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnMakePageLinks, makeRefLinks)

        makeRefLinks = FM.FlatMenuItem(editorMenu, -1, "Remove All Refs. Links\tCtrl+Alt+L", " Remove all Refs. links in Page", wx.ITEM_NORMAL)
        editorMenu.AppendItem(makeRefLinks)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnRemoveLinks, makeRefLinks)

        editorMenu.AppendSeparator() 

        font = FM.FlatMenuItem(editorMenu, -1, "Font Properties\tCtrl+F", " Font Properties", wx.ITEM_NORMAL)
        editorMenu.AppendItem(font)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnFont, font)

        fontColor = FM.FlatMenuItem(editorMenu, -1, "Font Color", " Font Color", wx.ITEM_NORMAL)
        editorMenu.AppendItem(fontColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnColour, fontColor)

        whiteBGColor = FM.FlatMenuItem(editorMenu, -1, "White", " White Background", wx.ITEM_NORMAL)
        hiLiteSM.AppendItem(whiteBGColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnWhiteBG, whiteBGColor)

        grayBGColor = FM.FlatMenuItem(editorMenu, -1, "Light Gray", " Light Gray Background", wx.ITEM_NORMAL)
        hiLiteSM.AppendItem(grayBGColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnGrayBG, grayBGColor)

        yellowBGColor = FM.FlatMenuItem(editorMenu, -1, "Light Yellow (Default)", " Light Yellow Background (Default)", wx.ITEM_NORMAL)
        hiLiteSM.AppendItem(yellowBGColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnYellowBG, yellowBGColor)
        
        blueBGColor = FM.FlatMenuItem(editorMenu, -1, "Light Blue", " Light Blue Background", wx.ITEM_NORMAL)
        hiLiteSM.AppendItem(blueBGColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnBlueBG, blueBGColor)

        greenBGColor = FM.FlatMenuItem(editorMenu, -1, "Light Green", " Light Green Background", wx.ITEM_NORMAL)
        hiLiteSM.AppendItem(greenBGColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnGreenBG, greenBGColor)

        redBGColor = FM.FlatMenuItem(editorMenu, -1, "Light Red", " Light Red Background", wx.ITEM_NORMAL)
        hiLiteSM.AppendItem(redBGColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnRedBG, redBGColor)
        
        editorMenu.AppendMenu(-1, "Font Background Color", hiLiteSM, wx.ITEM_NORMAL)
        
        editorMenu.AppendSeparator()
      
        aleft = FM.FlatMenuItem(editorMenu, -1, "Align Left", " Align left", wx.ITEM_NORMAL)
        editorMenu.AppendItem(aleft)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnAlignLeft, aleft)
        
        acenter = FM.FlatMenuItem(editorMenu, -1, "Align Center", " Align center", wx.ITEM_NORMAL)
        editorMenu.AppendItem(acenter)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnAlignCenter, acenter)

        aright = FM.FlatMenuItem(editorMenu, -1, "Align Right", " Align left", wx.ITEM_NORMAL)
        editorMenu.AppendItem(aright)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnAlignRight, aright)
        
        indentMore = FM.FlatMenuItem(editorMenu, -1, "Indent More", " Indent More", wx.ITEM_NORMAL)
        paraSM.AppendItem(indentMore)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnIndentMore, indentMore)

        indentLess = FM.FlatMenuItem(editorMenu, -1, "Indent Less", " Indent Less", wx.ITEM_NORMAL)
        paraSM.AppendItem(indentLess)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnIndentLess, indentLess)
        
        paraSM.AppendSeparator() 

        spacingMore = FM.FlatMenuItem(editorMenu, -1, "Increase Paragraph Spacing", " Increase Paragraph Spacing", wx.ITEM_NORMAL)
        paraSM.AppendItem(spacingMore)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnParagraphSpacingMore, spacingMore)

        spacingLess = FM.FlatMenuItem(editorMenu, -1, "Decrease Paragraph Spacing", " Decrease Paragraph Spacing", wx.ITEM_NORMAL)
        paraSM.AppendItem(spacingLess)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnParagraphSpacingLess, spacingLess)
        
        paraSM.AppendSeparator() 
        
        LineSpacingNorm = FM.FlatMenuItem(editorMenu, -1, "Normal Line Spacing", " Normal Line Spacing", wx.ITEM_NORMAL)
        paraSM.AppendItem(LineSpacingNorm)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnLineSpacingSingle, LineSpacingNorm)

        LineSpacingHalf = FM.FlatMenuItem(editorMenu, -1, "1.5 Line Spacing", " 1.5 Line Spacing", wx.ITEM_NORMAL)
        paraSM.AppendItem(LineSpacingHalf)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnLineSpacingHalf, LineSpacingHalf)

        LineSpacingDouble = FM.FlatMenuItem(editorMenu, -1, "Double Line Spacing", " Double Line Spacing", wx.ITEM_NORMAL)
        paraSM.AppendItem(LineSpacingDouble)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnLineSpacingDouble, LineSpacingDouble)

        editorMenu.AppendSeparator() 
        
        editorMenu.AppendMenu(-1, "Paragraph", paraSM, wx.ITEM_NORMAL)
        
        
        #Tools Menu
        showTools = FM.FlatMenuItem(toolsMenu, -1, "Module Creation Tools", " Show Module Creation/Conversion Tools", wx.ITEM_NORMAL, None)
        toolsMenu.AppendItem(showTools)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnTools, showTools)

        exportNotes = FM.FlatMenuItem(toolsMenu, -1, "Export User Notes to File", " Export User Notes to File", wx.ITEM_NORMAL, None)
        toolsMenu.AppendItem(exportNotes)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.ExportNotes, exportNotes)

        showConvert = FM.FlatMenuItem(toolsMenu, -1, "Convert 3.7 and Earlier Texts", " Convert 3.7 and Earlier Texts to .bk format", wx.ITEM_NORMAL, None)
        toolsMenu.AppendItem(showConvert)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnConvert, showConvert)

        showModFolder = FM.FlatMenuItem(toolsMenu, -1, "Display Primary Modules Folder", " Display Primary Modules Folder", wx.ITEM_NORMAL, None)
        toolsMenu.AppendItem(showModFolder)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnModFolder, showModFolder)

        showUserFolder = FM.FlatMenuItem(toolsMenu, -1, "Display User Modules Folder", " My Documents/User Modules Folder", wx.ITEM_NORMAL, None)
        toolsMenu.AppendItem(showUserFolder)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnUserFolder, showUserFolder)

        showConfigFolder = FM.FlatMenuItem(toolsMenu, -1, "Display User Data Folder", " My Documents/User Modules Folder", wx.ITEM_NORMAL, None)
        toolsMenu.AppendItem(showConfigFolder)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnConfigFolder, showConfigFolder)



        #Help Menu
        showHelp = FM.FlatMenuItem(helpMenu, -1, "Show Help\tF1", " Show Bible Analyzer User Manual", wx.ITEM_NORMAL, None)
        helpMenu.AppendItem(showHelp)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHelp, showHelp)
        
        BAsite = FM.FlatMenuItem(helpMenu, -1, "Check for Update", " Check for Bible Analyzer Update", wx.ITEM_NORMAL, None)
        helpMenu.AppendItem(BAsite)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnBAupdate, BAsite)

        BAforum = FM.FlatMenuItem(helpMenu, -1, "Bible Analyzer Forum", " Visit the Bible Analyzer Forum", wx.ITEM_NORMAL, None)
        helpMenu.AppendItem(BAforum)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnBAforum, BAforum)

        PWsite = FM.FlatMenuItem(helpMenu, -1, "Bible Analyzer Modules", " Bible Analyzer Modules Page", wx.ITEM_NORMAL, None)
        helpMenu.AppendItem(PWsite)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnModulesPage, PWsite)

        welcome = FM.FlatMenuItem(helpMenu, -1, "View Welcome Page", " View Welcome Page", wx.ITEM_NORMAL, None)
        helpMenu.AppendItem(welcome)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnWelcome, welcome)

        about = FM.FlatMenuItem(helpMenu, -1, "About Bible Analyzer", " About Bible Analyzer", wx.ITEM_NORMAL, None)
        helpMenu.AppendItem(about)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnAbout, about)
        
        ## Bookmarks
        self.addBookmarks()
        self.mainApp.Bind(wx.EVT_MENU_RANGE, self.doBookmark, id=10000, id2=10009)
        

        # Add menu to the menu bar
        self.Append(fileMenu, "&File")
        self.Append(self.viewMenu, "&View")
        self.Append(ctrlMenu, "Co&ntrol")
        self.Append(self.bmMenu, "Boo&kmarks")
        self.Append(editorMenu, "Scripture&Pad")
        self.Append(toolsMenu, "T&ools")        
        self.Append(helpMenu, "&Help")
                
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnMasterControlMini, id=1005)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnImageViewer, id=1004)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnLibViewer, id=1003)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnOptionsDlg, id=1006)
        
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnDevotions, id=1002)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnHelp, id=1007)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnMediaMini, id=1008)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnTTSSpeak, id=1009)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnToggleMini, id=1010)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnParGen, id=1011)
        
        self.parGen = None

    
    ## Bookmarks
    def addBookmarks(self):
        try:
            bookmarkList = self.mainApp.config["settings"]["bookmarklist"]
            self.bookmarkList = ['Go to: %s' % bm for bm in bookmarkList]
        except:
            self.bookmarkList = ['Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty']

        cnt = 10000
        for item in self.bookmarkList:
            item = '%s\tAlt+%s' % (item, cnt -10000)
            self.bmMenu.Append(cnt, item, "", wx.ITEM_NORMAL)
            cnt += 1
        
    
    def doBookmark(self, event):
        id = event.GetId()
        print id
        item = self.bmMenu.FindItem(id)
        txt = item.GetLabel()
        txt = txt.replace('Go to: ', '')
        print txt
        if txt == 'Empty' or wx.GetKeyState(wx.WXK_SHIFT):
            self.enterBookMark(id, item)
        else:
            self.mainApp.ShowRefContext(txt)

    
    def enterBookMark(self, id, item):
        dlg = wx.TextEntryDialog(
                self, 'Enter Reference for Bookmark %s' % (id -10000),
                'Enter Bookmark', 'Python')

        dlg.SetValue("%s" % self.mainApp.currentRef)

        if dlg.ShowModal() == wx.ID_OK:
            ref = dlg.GetValue()

            if ref[-1] == ':': 
                ref = ref + '1'
            ref = fixRef.prelimFix(ref)
            if not ':' in ref and ref[-1].isdigit():
                ref = ref + ':1'
            ref = fixRef.fix(ref)
            item.SetText('Go to: %s\tAlt+%s' % (ref, id -10000))

        dlg.Destroy()


        

    def helpToolbar(self):
        Pref = mainImg.getPrefBitmap()
        Help = mainImg.getHelpBitmap()
        Speech = mainImg.getSpeechBitmap()
        SpeechDis = mainImg.getSpeechDisBitmap()
        
        tb = Aui.AuiToolBar(self.mainApp, -1, wx.DefaultPosition, wx.DefaultSize, agwStyle = Aui.AUI_TB_PLAIN_BACKGROUND)
        tb.SetToolBitmapSize(wx.Size(32, 32))
        tb.AddTool(1009, "", Speech, SpeechDis, wx.ITEM_NORMAL, "Start\Stop Text to Speech", '', None)
        tb.AddSimpleTool(1006, "", Pref, "User Preferences")
        tb.AddSimpleTool(1007, "", Help, "User Manual")
        
        #if self.mainApp.loadTts == False:
        #    tb.EnableTool(1009, False)
        
        return tb

    def viewerToolbar(self):
        Calendar = mainImg.getCalendarBitmap()
        Read = mainImg.getBookBitmap()
        View = mainImg.getViewBitmap()
        Hymn = mainImg.getHymnBitmap()
        Multi = mainImg.getMultiBitmap()
        Para = mainImg.getParallelBitmap()
        
        tb = Aui.AuiToolBar(self.mainApp, -1, wx.DefaultPosition, wx.DefaultSize, agwStyle = Aui.AUI_TB_PLAIN_BACKGROUND)
        tb.SetToolBitmapSize(wx.Size(32, 32))
        tb.AddSimpleTool(1002, "", Calendar, "Daily Devotions")
        tb.AddSimpleTool(1008, "", Hymn, "Audio\Quiz Player")        
        tb.AddSimpleTool(1011, "", Para, "Harmony\Parallel Generator")
        tb.AddSimpleTool(1003, "", Read, "Book\Article Viewer")
        tb.AddSimpleTool(1004, "", View, "Image Viewer")
        tb.AddSimpleTool(1010, "", Multi, "MultiWindow\AV-Strong Index")
        
        return tb


    def entryToolbar(self):
        tb = Aui.AuiToolBar(self.mainApp, -1, wx.DefaultPosition, wx.DefaultSize, agwStyle = Aui.AUI_TB_PLAIN_BACKGROUND)
        
        self.QSentry = wx.ComboBox(tb, -1, 'Quick Search or Ref.', choices=[], style=wx.CB_DROPDOWN|wx.TE_PROCESS_ENTER)
        self.QSentry.SetToolTipString(' Enter Search Word(s) \n Or a Reference \n Alt+Q ')
                
        tb.AddControl(self.QSentry, 'Quick Entry')
        #tb.AddSpacer(10)
                
        return tb

    
    def imageToolbar(self):
        tb = Aui.AuiToolBar(self.mainApp, -1, wx.DefaultPosition, wx.DefaultSize, agwStyle = Aui.AUI_TB_PLAIN_BACKGROUND)
        
        #png = wx.Image('ba-scroll32.png', wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        png = mainImg.getScrollBitmap()
        mask = wx.Mask(png, wx.WHITE)
        png.SetMask(mask)
        sb = wx.StaticBitmap(tb, -1, png, (0, 0), (png.GetWidth(), png.GetHeight()))
        
        st = wx.StaticText(tb, -1, self.mainApp.toolTxt)
        st.SetForegroundColour(wx.Colour(120, 0, 0))
        st.SetFont(wx.Font(9, wx.DEFAULT, wx.ITALIC, wx.NORMAL, 0, "Verdana"))
        
        tb.AddSpacer(5)
        tb.AddControl(st, 'Quick Entry')
        tb.AddSpacer(10)
        tb.AddControl(sb, 'Quick Entry')
                
        return tb


        
    def buttonToolbar(self):
        self.buttonTB = Aui.AuiToolBar(self.mainApp, -1, wx.DefaultPosition, wx.DefaultSize, agwStyle = Aui.AUI_TB_PLAIN_BACKGROUND)
        Control = mainImg.getControlBitmap()
        #Control = wx.Image(("control.png")).ConvertToBitmap()
        
        size = (34, 30)
        segments = 4
        labels = ['B', 'R', 'C', 'D']
        tips = ['Bible Window', 'Results Window', 'Commentary Window', 'Dictionary Window']
        events = [self.bibMax, self.resMax, self.cmtMax, self.dctMax]
        
        segButton = SegmentButton(self.buttonTB, -1, size, segments, labels, events, tips)
        
        self.jumpBut = buttons.GenToggleButton(self.buttonTB, -1, 'J', size = (14, 18))
        self.jumpBut.SetUseFocusIndicator(False)
        self.jumpBut.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
        self.jumpBut.SetForegroundColour(wx.Colour(0, 0, 128))
        self.jumpBut.SetBezelWidth(1)
        self.jumpBut.Bind(wx.EVT_BUTTON, self.OnJumpState)
        self.jumpBut.SetToolTipString('Jump to corresponding window on click')

        layBut = buttons.GenButton(self.buttonTB, -1, 'L', size = (14, 18))
        layBut.SetUseFocusIndicator(False)
        layBut.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
        layBut.SetForegroundColour(wx.Colour(0, 0, 128))
        layBut.SetBezelWidth(1)
        layBut.Bind(wx.EVT_BUTTON, self.layoutMenu)
        layBut.SetToolTipString('Save and select layouts')

        self.buttonTB.AddControl(self.jumpBut, '')
        self.buttonTB.AddSpacer(2)
        self.buttonTB.AddControl(segButton, '')
        self.buttonTB.AddSpacer(2)
        self.buttonTB.AddControl(layBut, '')
        self.buttonTB.AddSpacer(5)
        self.buttonTB.AddSimpleTool(1005, "", Control, "Master Control Panel")

        return self.buttonTB

    
    def ExportBib(self, event):
        dlg = dialogs.ExportBib(None, -1, "Export/Copy Bible")
        dlg.Show()

    
    def OnJumpState(self, event):
        self.mainApp.jumpState = self.jumpBut.GetValue()


    def bibMax(self, event):
        name = 'Bible'
        pane = self.getPane(name)
        if pane:
            self.sortPane(pane, name)
            self.scrollBib()

    def dctMax(self, event):
        name = 'Dictionaries'
        pane = self.getPane(name)
        if pane:
            self.sortPane(pane, name)

    def resMax(self, event):
        name = 'Results'
        pane = self.getPane(name)
        if pane:
            self.sortPane(pane, name)

    def cmtMax(self, event):
        name = 'Commentaries'
        pane = self.getPane(name)
        if pane:
            self.sortPane(pane, name)
        
        
    def sortPane(self, pane, name):  
    
        if pane.IsNotebookPage() and self.mainApp.mainMgr.GetPane('__notebook_0').IsClosed():
            self.mainApp.mainMgr.GetPaneByName('__notebook_0').NotClosed().Restore().Show()
            self.mainApp.mainMgr.Update()
            print '1'
        
        elif pane.IsNotebookPage() and self.mainApp.mainMgr.GetPane('__notebook_0').IsFloating():
            self.reDock(self.mainApp.mainMgr.GetPane('__notebook_0'))
            print '2'
            
        elif pane.IsClosed():
            self.openPane(pane)
            print '3'
            
        elif pane.IsFloating():
            if name == 'Commentaries' or name == 'Dictionaries':
                if pane.dock_direction_get() == 6:
                    pane.dock_direction_set(3)
            elif name == 'Results':
                if pane.dock_direction_get() == 6:
                    pane.dock_direction_set(2)
            
            print '4', 
            self.reDock(pane)
            
        else:
            self.doMax(pane, name)
            if name == 'Bible':
                tab = self.mainApp.bibNB.GetSelection()
                self.mainApp.bibDisplay[tab].SetFocus()
                if '<a name="line"></a>' in self.mainApp.bibCurrent:
                    self.mainApp.bibDisplay[tab].ScrollToAnchor('line')
                    self.mainApp.bibDisplay[tab].ScrollLines(-2)
                    
            if name == 'Dictionaries':
                self.mainApp.dctDisplay[self.mainApp.dctNB.GetSelection()].SetFocus()
            if name == 'Commentaries':
                if self.mainApp.cmtNB.GetSelection() == 0:
                    self.mainApp.notes.SetFocus()
                else:
                    self.mainApp.cmtDisplay[self.mainApp.cmtNB.GetSelection() -1].SetFocus()
            if name == 'Results':
                if self.mainApp.resNB.GetSelection() == 0:
                    self.mainApp.editor.SetFocus()
                else:
                    self.mainApp.resDisplay[self.mainApp.resNB.GetSelection() -1].SetFocus()
            
            self.mainApp.OnSetBorder(None)
            print '5'

    
    def reDock(self, pane):
        self.mainApp.mainMgr.RestoreMaximizedPane()
        pane.Dock()
        
        self.mainApp.mainMgr.Update()


    def openPane(self, pane):
        self.mainApp.mainMgr.RestoreMaximizedPane()
        if pane.dock_direction == 0 and pane.name == 'Dictionaries':
            pane.Bottom().Layer(2).Position(1).Show().NotClosed()
        elif pane.dock_direction == 0 and pane.name == 'Commentaries':
            pane.Bottom().Layer(2).Position(0).Show().NotClosed()
        elif pane.dock_direction == 0 and pane.name == 'Bible':
            pane.CenterPane.Show().NotClosed()
        elif pane.dock_direction == 0 and pane.name == 'Results':
            pane.Right().Layer(1).Show().NotClosed()
        else:
            pane.Show().NotClosed()
        self.mainApp.mainMgr.Update()

        
    def doMax(self, pane, name):
        if pane.IsNotebookPage():
            nb = self.mainApp.mainMgr.GetNotebooks()[0]
            nbInfo = self.mainApp.mainMgr.GetPane(nb)
            if nbInfo.IsMaximized():
                self.mainApp.mainMgr.RestoreMaximizedPane() 
            else:
                for idx in range(nb.GetPageCount()):
                    if nb.GetPageText(idx) == name:
                        nb.SetSelection(idx)
                self.mainApp.mainMgr.RestoreMaximizedPane()
                self.mainApp.mainMgr.MaximizePane(nbInfo)
        elif pane.IsMaximized():
            self.mainApp.mainMgr.RestoreMaximizedPane()
        else:
            self.mainApp.mainMgr.RestoreMaximizedPane()
            self.mainApp.mainMgr.MaximizePane(pane)
        
        self.mainApp.mainMgr.Update()
        

    def getPane(self, name):
        allPanes = self.mainApp.mainMgr.GetAllPanes()
        for pane in allPanes:
            if pane.name == name:
                return pane


    def layoutMenu(self, event):
        if event:
            but = event.GetEventObject()
        
        self.layMenu = FM.FlatMenu()
        delMenu = FM.FlatMenu()
        
        cnt = 2000 
        for item in self.mainApp.layoutTitles:
            self.layMenu.Append(cnt + 1, item, "", wx.ITEM_NORMAL)
            cnt += 1
        
        cnt = 2101
        if len(self.mainApp.layoutTitles) > 2:
            for item in self.mainApp.layoutTitles[2:]:
                delMenu.Append(cnt, item, "", wx.ITEM_NORMAL)
                cnt += 1
            
            self.layMenu.AppendSeparator()
            self.layMenu.AppendMenu(2199, 'Delete a Layout', delMenu, wx.ITEM_NORMAL)
        
        self.mainApp.Bind(wx.EVT_MENU, self.addLayout, id=2001)
        self.mainApp.Bind(wx.EVT_MENU, self.loadDefault, id=2002)
        self.mainApp.Bind(wx.EVT_MENU_RANGE, self.loadLayout, id=2003, id2=2099)
        self.mainApp.Bind(wx.EVT_MENU_RANGE, self.delLayout, id=2101, id2=2198)
        
        if event:
            rect = but.GetButtonRect()
            pt = but.ClientToScreen(rect.GetBottomLeft())

            self.layMenu.Popup(pt, self.mainApp)
            self.popMenu = self.layMenu



    def addLayout(self, event):
        dlg = wx.TextEntryDialog(self, "Enter a name for the new layout:\n(If a layout with this name exists, it will be replaced)", "Bible Analyzer Layout")

        if dlg.ShowModal() != wx.ID_OK:
            return
        
        title = dlg.GetValue()
        
        if title in self.mainApp.layoutTitles:
            idx = self.mainApp.layoutTitles.index(title)
            self.mainApp.layoutList[idx -1] = self.mainApp.mainMgr.SavePerspective()
        else:
            self.mainApp.layoutTitles.append(title)
            self.mainApp.layoutList.append(self.mainApp.mainMgr.SavePerspective())
        
        statText = 'Current Layout: %s' % title
        self.mainApp.statusbar.SetStatusText(statText, 3)
        
        dlg.Destroy()


    def loadDefault(self, event):         
        self.mainApp.mainMgr.RestoreMaximizedPane()
        
        self.mainApp.mainMgr.LoadPerspective(self.mainApp.layoutList[0])
        
        #sx, sy = self.mainApp.GetSize()
        #sx = sx - 100
        #sy = sy - 100
        #print sx/2, sy/2

        #print self.mainApp.mainMgr.GetAttributes(self.mainApp.getBibPane())
        #.BestSize(sx/2, sy/2)
        
        self.mainApp.SetBibFocus(None)
        self.scrollBib()
        
        self.mainApp.mainMgr.Update()
        
        statText = 'Current Layout: %s' % 'Default'
        self.mainApp.statusbar.SetStatusText(statText, 3)


    def loadLayout(self, event):
        self.mainApp.mainMgr.RestoreMaximizedPane()
        
        idx = event.GetId() - 2000
                
        self.mainApp.mainMgr.LoadPerspective(self.mainApp.layoutList[idx -2])
        
        self.scrollBib()
        
        statText = 'Current Layout: %s' % self.mainApp.layoutTitles[idx -1]
        self.mainApp.statusbar.SetStatusText(statText, 3)
        

    def delLayout(self, event):
        idx = event.GetId() - 2101
        del self.mainApp.layoutTitles[idx +2]
        del self.mainApp.layoutList[idx +1]
        
        
    def scrollBib(self):
        tab = self.mainApp.bibNB.GetSelection()
        if self.mainApp.currentRef[-1].isdigit() and tab != len(self.mainApp.bibFileList):
            print self.mainApp.currentRef, 'mm'
            self.mainApp.bibDisplay[tab].ScrollToAnchor(self.mainApp.currentRef)
            self.mainApp.bibDisplay[tab].ScrollLines(-2)

