import wx, os, re
from autoCtrl import AutoTextCtrl
import mainImg
import wx
import genButton as buttons
import dialogs
import parGen
import fixRef
import sqlite3 as sqlite

#import flatMenu as FM

import agw.aui as Aui



class SegmentButton(wx.PyPanel):
    def __init__(self, parent, id, size, segments, labels, events, tips):
        wx.PyPanel.__init__(self, parent, id)
        self.mainApp = wx.GetApp().GetTopWindow()
        
        assert len(labels) == segments
        
        segSize = (size[0] /2, size[1] /2) # 4 segments
        if segments == 3:
            segSize = (size[0], size[1] /3.33)
        if segments == 2:
            segSize = (size[0], size[1] /2)
        
        segList = []
        
        for seg, num in enumerate(range(segments)):
            b = buttons.GenButton(self, -1, labels[num], size = (segSize[0], segSize[1]))
            b.SetUseFocusIndicator(False)
            #b.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
            b.SetForegroundColour(wx.Colour(128, 0, 0))
            b.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
            b.SetBezelWidth(1)
            b.SetToolTipString(tips[num])
            self.Bind(wx.EVT_BUTTON, events[num], b)
            segList.append(b)
        
        gsRows = (2, 2) # 4 segments
        if segments == 3:
            gsRows = (3, 1)
        elif segments == 2:
            gsRows = (2, 1)
        
        gs = wx.GridSizer(gsRows[0], gsRows[1])
        for b in segList:
            gs.Add(b, 0, wx.EXPAND)

        self.SetSizer(gs)





class MainMenu(wx.MenuBar):
    def __init__(self, parent):
        wx.MenuBar.__init__(self)
        
        self.mainApp = parent
        self.curNBInfo = None
        self.oldNBPageInfo = None
        self.lastState = None
        
        
        fileMenu  = wx.Menu()
        self.viewMenu  = wx.Menu()
        smView = wx.Menu()
        #self.layMenu = wx.Menu()
        ctrlMenu = wx.Menu()
        toolsMenu = wx.Menu()
        editorMenu = wx.Menu()
        helpMenu = wx.Menu()
        self.bmMenu = wx.Menu()
        
        paraSM = wx.Menu()
        hiLiteSM = wx.Menu()

            

        # File Menu
        sAll = wx.MenuItem(fileMenu, -1, "&Select All on current page\tCtrl+A", " Select all text in panel")
        fileMenu.AppendItem(sAll)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnSelectAll, sAll)
        
        copy = wx.MenuItem(fileMenu, -1, "&Copy Selected on current page\tCtrl+C", " Copy selected text")
        fileMenu.AppendItem(copy)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnCopy, copy)

        fileMenu.AppendSeparator()

        copy = wx.MenuItem(fileMenu, -1, "Export/Copy verse(s)\tCtrl+E", " Export/Copy verse(s) to File, S-Pad, or Clipboard")
        fileMenu.AppendItem(copy)
        self.mainApp.Bind(wx.EVT_MENU, self.ExportBib, copy)

        fileMenu.AppendSeparator()

        copy = wx.MenuItem(fileMenu, -1, "&Print current page\tCtrl+P", " Print current page")
        fileMenu.AppendItem(copy)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnPrint, copy)

        copy = wx.MenuItem(fileMenu, -1, "Print Preview current page\tAlt+Ctrl+P", " Print Preview current page")
        fileMenu.AppendItem(copy)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnPrintPre, copy)

        fileMenu.AppendSeparator()
                
        exit = wx.MenuItem(fileMenu, -1, "Exit\tAlt+X", "Exit Bible Analyzer")
        fileMenu.AppendItem(exit)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnClose, exit)
        
        
        #View Menu        
        entryTB = wx.MenuItem(smView, -1, "Hide/Show Entry Toolbar", "", wx.ITEM_CHECK)
        smView.AppendItem(entryTB)
        self.entryTbId = entryTB.GetId()
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHideEntryTB, entryTB)
        
        messageTB = wx.MenuItem(smView, -1, "Hide/Show Message Toolbar", "", wx.ITEM_CHECK)
        smView.AppendItem(messageTB)
        self.messageTbId = messageTB.GetId()
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHideMessageTB, messageTB)

        helpTB = wx.MenuItem(smView, -1, "Hide/Show Help Toolbar", "", wx.ITEM_CHECK)
        smView.AppendItem(helpTB)
        self.helpTbId = helpTB.GetId()
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHideHelpTB, helpTB)

        viewerTB = wx.MenuItem(smView, -1, "Hide/Show Viewer Toolbar", "", wx.ITEM_CHECK)
        smView.AppendItem(viewerTB)
        self.viewerTbId = viewerTB.GetId()
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHideViewerTB, viewerTB)

        buttonTB = wx.MenuItem(smView, -1, "Hide/Show Button Toolbar", "", wx.ITEM_CHECK)
        smView.AppendItem(buttonTB)
        self.buttonTbId = buttonTB.GetId()
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHideButtonTB, buttonTB)

        self.viewMenu.AppendMenu(-1, "Toolbars", smView)
        self.viewMenu.AppendSeparator()
        
        
        self.layoutMenu(None)
        
        self.viewMenu.AppendMenu(-1, "Select/Create Layouts", self.layMenu)
        self.viewMenu.AppendSeparator()
        
        bibExp = wx.MenuItem(self.viewMenu, -1, "View/Expand/Contract Bible Window\tAlt+B", " View/Expand/Contract Bible Window")
        self.viewMenu.AppendItem(bibExp)
        self.mainApp.Bind(wx.EVT_MENU, self.bibMax, bibExp)
        
        libExp = wx.MenuItem(self.viewMenu, -1, "View/Expand/Contract Dictionary Window\tAlt+D", " View/Expand/Contract Dictionary Window")
        self.viewMenu.AppendItem(libExp)
        self.mainApp.Bind(wx.EVT_MENU, self.dctMax, libExp)
        
        resExp = wx.MenuItem(self.viewMenu, -1, "View/Expand/Contract Results Window\tAlt+S", " View/Expand/Contract Results Window")
        self.viewMenu.AppendItem(resExp)
        self.mainApp.Bind(wx.EVT_MENU, self.resMax, resExp)
        
        cmtExp = wx.MenuItem(self.viewMenu, -1, "View/Expand/Contract Commentary Window\tAlt+C", " View/Expand/Contract Commentary Window")
        self.viewMenu.AppendItem(cmtExp)
        self.mainApp.Bind(wx.EVT_MENU, self.cmtMax, cmtExp)
        
        self.viewMenu.AppendSeparator()

        hideTree = wx.MenuItem(self.viewMenu, -1, "Hide/Show Bible Tree\tAlt+T", " Hide/Show Tree Panel", wx.ITEM_CHECK)
        self.viewMenu.AppendItem(hideTree)
        self.treeShowId = hideTree.GetId()
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHideTree, hideTree)

        colTree = wx.MenuItem(self.viewMenu, -1, "Collapse Bible Tree Books\tAlt+L", " Collapse Bible Tree Books")
        self.viewMenu.AppendItem(colTree)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.treeCollapse, colTree)
        
        self.viewMenu.AppendSeparator()
                
        nVerse = wx.MenuItem(self.viewMenu, -1, "Next Bible Verse\tF6", " Next Bible Verse")
        self.viewMenu.AppendItem(nVerse)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnNextVerse, nVerse)

        pVerse = wx.MenuItem(self.viewMenu, -1, "Previous Bible Verse\tF5", " Previous Bible Verse")
        self.viewMenu.AppendItem(pVerse)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnPreVerse, pVerse)
        
        nChap = wx.MenuItem(self.viewMenu, -1, "Next Bible Chapter\tF8", " Next Bible Chapter")
        self.viewMenu.AppendItem(nChap)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnNextChap, nChap)

        pChap = wx.MenuItem(self.viewMenu, -1, "Previous Bible Chapter\tF7", " Previous Bible Chapter")
        self.viewMenu.AppendItem(pChap)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnPreChap, pChap)

        self.viewMenu.AppendSeparator()

        nBookChap = wx.MenuItem(self.viewMenu, -1, "Next Book Chapter\tAlt+F6", " Next Book Chapter")
        self.viewMenu.AppendItem(nBookChap)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnNextBookChap, nBookChap)

        pBookChap = wx.MenuItem(self.viewMenu, -1, "Previous Book Chapter\tAlt+F5", " Previous Book Chapter")
        self.viewMenu.AppendItem(pBookChap)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnPreBookChap, pBookChap)
        
        bibRef = wx.MenuItem(self.viewMenu, -1, "Toggle Bible Reference Panel\tCtrl+R", " Previous Book Chapter")
        self.viewMenu.AppendItem(bibRef)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnRefPanel, bibRef)
        
        self.viewMenu.AppendSeparator()

        fontPlus = wx.MenuItem(self.viewMenu, -1, "Increase Windows Font Size\tCtrl++", " Previous Book Chapter")
        self.viewMenu.AppendItem(fontPlus)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnFontPlus, fontPlus)

        fontMinus = wx.MenuItem(self.viewMenu, -1, "Decrease Windows Font Size\tCtrl+-", " Previous Book Chapter")
        self.viewMenu.AppendItem(fontMinus)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnFontMinus, fontMinus)



        #Control Menu
        master = wx.MenuItem(ctrlMenu, 1005, "Master Control Panel\tF2", " Toggle Master Control Panel")
        ctrlMenu.AppendItem(master)
        
        readView = wx.MenuItem(ctrlMenu, 1003, "Book/Article Titles\tF3", " Open Book/Article Titles List")
        ctrlMenu.AppendItem(readView)
        
        imgView = wx.MenuItem(ctrlMenu, 1004, "Image Viewer\tF4", " Open Image Viewer")
        ctrlMenu.AppendItem(imgView)
        
        devotions = wx.MenuItem(ctrlMenu, 1002, "Devotions Viewer\tF9", " Open Daily Devotions Viewer")
        ctrlMenu.AppendItem(devotions)

        multiWin = wx.MenuItem(ctrlMenu, 1010, "MultiWindow\tF10", " Toggle MultiWindow Viewer")
        ctrlMenu.AppendItem(multiWin)

        optDlg = wx.MenuItem(ctrlMenu, 1006, "Preferences Dialog\tF11", " Open Preferences Dialog")
        ctrlMenu.AppendItem(optDlg)

        miniWin = wx.MenuItem(ctrlMenu, 1008, "Audio/Quiz Panel\tF12", " Open Audio/Quiz Panel")
        ctrlMenu.AppendItem(miniWin)

        sessMan = wx.MenuItem(ctrlMenu, 1012, "Session Manager\tCtrl+M", " Harmony/Parallel Generator")
        ctrlMenu.AppendItem(sessMan)

        parGen = wx.MenuItem(ctrlMenu, 1011, "Harmony/Parallel Generator\tAlt+G", " Harmony/Parallel Generator")
        ctrlMenu.AppendItem(parGen)
        
        parGen = wx.MenuItem(ctrlMenu, 1013, "Show/Hide Reference Panel\tAlt+R", " Show/Hide Reference Panel")
        ctrlMenu.AppendItem(parGen)
        
        ctrlMenu.AppendSeparator()

        QSearchFocus = wx.MenuItem(ctrlMenu, -1, "Quick Find Entry (focus)\tAlt+Q", " Go to Quick Search/Ref Entry ")
        ctrlMenu.AppendItem(QSearchFocus)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.SetQSearchFocus, QSearchFocus)

        notesFocus = wx.MenuItem(ctrlMenu, -1, "Show User Notes\tAlt+E", " Show User Notes Tab ")
        ctrlMenu.AppendItem(notesFocus)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnNotesFocus, notesFocus)


        ctrlMenu.AppendSeparator()

        ttsSpeak = wx.MenuItem(ctrlMenu, -1, "Speak/Stop Current Text\tCtrl+F10", " Speak\Stop Current Text (Toggle)")
        ctrlMenu.AppendItem(ttsSpeak)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnTTSSpeak, ttsSpeak)

        pause = wx.MenuItem(ctrlMenu, -1, "Pause/Play Audio\tAlt+/", " Pause Audio Player")
        ctrlMenu.AppendItem(pause)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnPause, pause)


        #Editor Menu
        new = wx.MenuItem(editorMenu, -1, "New\tCtrl+N", " New document in ScripturePad")
        editorMenu.AppendItem(new)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnFileClose, new)
        
        open = wx.MenuItem(editorMenu, -1, "Open\tCtrl+O", " Open file in ScripturePad")
        editorMenu.AppendItem(open)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnFileOpen, open)
        
        save = wx.MenuItem(editorMenu, -1, "Save\tCtrl+S", " Save ScripturePad File")
        editorMenu.AppendItem(save)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnFileSave, save)        
        
        saveas = wx.MenuItem(editorMenu, -1, "Save As", " Save ScripturePad File As")
        editorMenu.AppendItem(saveas)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnFileSaveAs, saveas)

        close = wx.MenuItem(editorMenu, -1, "Close\tCtrl+W", " Close current ScripturePad File")
        editorMenu.AppendItem(close)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnFileClose, close)


        editorMenu.AppendSeparator()
        
        cut = wx.MenuItem(editorMenu, -1, "Cut Selection\tCtrl+X", " Cut Selection")
        editorMenu.AppendItem(cut)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnEditorCut, cut)
        
        copy = wx.MenuItem(editorMenu, -1, "Copy Selection", " Copy Selection")
        editorMenu.AppendItem(copy)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnEditorCopy, copy)
        
        paste = wx.MenuItem(editorMenu, -1, "Paste from Clipboard\tCtrl+V", " Paste from Clipboard")
        editorMenu.AppendItem(paste)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnEditorPaste, paste)
        
        select = wx.MenuItem(editorMenu, -1, "Select All", " Select All")
        editorMenu.AppendItem(select)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnEditorSelectAll, select)

        editorMenu.AppendSeparator()

        undo = wx.MenuItem(editorMenu, -1, "Undo\tCtrl+Z", " Undo")
        editorMenu.AppendItem(undo)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnUndo, undo)

        redo = wx.MenuItem(editorMenu, -1, "Redo", " Redo")
        editorMenu.AppendItem(redo)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnRedo, redo)

        editorMenu.AppendSeparator()

        prnt = wx.MenuItem(editorMenu, -1, "Print\tCtrl+P", " Print Page")
        editorMenu.AppendItem(prnt)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnEditorPrint, prnt)

        printPre = wx.MenuItem(editorMenu, -1, "Print Preview", " Print Preview")
        editorMenu.AppendItem(printPre)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnEditorPrintPre, printPre)

        editorMenu.AppendSeparator()

        bold = wx.MenuItem(editorMenu, -1, "Bold\tCtrl+B", " Make selection bold")
        editorMenu.AppendItem(bold)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnBold, bold)
        
        italics = wx.MenuItem(editorMenu, -1, "Italics\tCtrl+I", " Make selection italics")
        editorMenu.AppendItem(italics)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnItalic, italics)
        
        underline = wx.MenuItem(editorMenu, -1, "Underline\tCtrl+U", " Underline selection")
        editorMenu.AppendItem(underline)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnUnderline, underline)
        
        editorMenu.AppendSeparator() 
        
        makeRefLinks = wx.MenuItem(editorMenu, -1, "Make All Refs. Links\tCtrl+L", " Attempt to make all Refs. links in Page")
        editorMenu.AppendItem(makeRefLinks)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnMakePageLinks, makeRefLinks)

        makeRefLinks = wx.MenuItem(editorMenu, -1, "Remove All Refs. Links\tCtrl+Alt+L", " Remove all Refs. links in Page")
        editorMenu.AppendItem(makeRefLinks)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnRemoveLinks, makeRefLinks)

        editorMenu.AppendSeparator() 

        font = wx.MenuItem(editorMenu, -1, "Font Properties\tCtrl+F", " Font Properties")
        editorMenu.AppendItem(font)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnFont, font)

        fontColor = wx.MenuItem(editorMenu, -1, "Font Color", " Font Color")
        editorMenu.AppendItem(fontColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnColour, fontColor)

        whiteBGColor = wx.MenuItem(editorMenu, -1, "White", " White Background")
        hiLiteSM.AppendItem(whiteBGColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnWhiteBG, whiteBGColor)

        grayBGColor = wx.MenuItem(editorMenu, -1, "Light Gray", " Light Gray Background")
        hiLiteSM.AppendItem(grayBGColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnGrayBG, grayBGColor)

        yellowBGColor = wx.MenuItem(editorMenu, -1, "Light Yellow (Default)", " Light Yellow Background (Default)")
        hiLiteSM.AppendItem(yellowBGColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnYellowBG, yellowBGColor)
        
        blueBGColor = wx.MenuItem(editorMenu, -1, "Light Blue", " Light Blue Background")
        hiLiteSM.AppendItem(blueBGColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnBlueBG, blueBGColor)

        greenBGColor = wx.MenuItem(editorMenu, -1, "Light Green", " Light Green Background")
        hiLiteSM.AppendItem(greenBGColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnGreenBG, greenBGColor)

        redBGColor = wx.MenuItem(editorMenu, -1, "Light Red", " Light Red Background")
        hiLiteSM.AppendItem(redBGColor)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnRedBG, redBGColor)
        
        editorMenu.AppendMenu(-1, "Font Background Color", hiLiteSM)
        
        editorMenu.AppendSeparator()
      
        aleft = wx.MenuItem(editorMenu, -1, "Align Left", " Align left")
        editorMenu.AppendItem(aleft)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnAlignLeft, aleft)
        
        acenter = wx.MenuItem(editorMenu, -1, "Align Center", " Align center")
        editorMenu.AppendItem(acenter)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnAlignCenter, acenter)

        aright = wx.MenuItem(editorMenu, -1, "Align Right", " Align left")
        editorMenu.AppendItem(aright)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnAlignRight, aright)

        bullet = wx.MenuItem(editorMenu, -1, "Bullet List", " Bullet List")
        paraSM.AppendItem(bullet)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnBullet, bullet)

        nobullet = wx.MenuItem(editorMenu, -1, "Remove Bullet List", " Remove Bullet List")
        paraSM.AppendItem(nobullet)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnNoBullet, nobullet)
        
        paraSM.AppendSeparator() 

        indentMore = wx.MenuItem(editorMenu, -1, "Indent More", " Indent More")
        paraSM.AppendItem(indentMore)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnIndentMore, indentMore)

        indentLess = wx.MenuItem(editorMenu, -1, "Indent Less", " Indent Less")
        paraSM.AppendItem(indentLess)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnIndentLess, indentLess)
        
        paraSM.AppendSeparator() 

        spacingMore = wx.MenuItem(editorMenu, -1, "Increase Paragraph Spacing", " Increase Paragraph Spacing")
        paraSM.AppendItem(spacingMore)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnParagraphSpacingMore, spacingMore)

        spacingLess = wx.MenuItem(editorMenu, -1, "Decrease Paragraph Spacing", " Decrease Paragraph Spacing")
        paraSM.AppendItem(spacingLess)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnParagraphSpacingLess, spacingLess)
        
        paraSM.AppendSeparator() 
        
        LineSpacingNorm = wx.MenuItem(editorMenu, -1, "Normal Line Spacing", " Normal Line Spacing")
        paraSM.AppendItem(LineSpacingNorm)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnLineSpacingSingle, LineSpacingNorm)

        LineSpacingHalf = wx.MenuItem(editorMenu, -1, "1.5 Line Spacing", " 1.5 Line Spacing")
        paraSM.AppendItem(LineSpacingHalf)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnLineSpacingHalf, LineSpacingHalf)

        LineSpacingDouble = wx.MenuItem(editorMenu, -1, "Double Line Spacing", " Double Line Spacing")
        paraSM.AppendItem(LineSpacingDouble)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.edit.OnLineSpacingDouble, LineSpacingDouble)

        editorMenu.AppendSeparator() 
        
        editorMenu.AppendMenu(-1, "Paragraph", paraSM)
        
        
        #Tools Menu
        showTools = wx.MenuItem(toolsMenu, -1, "Module Creation Tools", " Show Module Creation/Conversion Tools")
        toolsMenu.AppendItem(showTools)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnTools, showTools)

        exportNotes = wx.MenuItem(toolsMenu, -1, "Export User Notes to File", " Export User Notes to File")
        toolsMenu.AppendItem(exportNotes)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.ExportNotes, exportNotes)

        index = wx.MenuItem(toolsMenu, -1, "Re-index Modules For References", " Re-index Modules For Verse References")
        toolsMenu.AppendItem(index)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnIndex, index)

        showModFolder = wx.MenuItem(toolsMenu, -1, "Display Primary Modules Folder", " Display Primary Modules Folder")
        toolsMenu.AppendItem(showModFolder)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnModFolder, showModFolder)

        showUserFolder = wx.MenuItem(toolsMenu, -1, "Display User Modules Folder", " My Documents/User Modules Folder")
        toolsMenu.AppendItem(showUserFolder)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnUserFolder, showUserFolder)

        showConfigFolder = wx.MenuItem(toolsMenu, -1, "Display User Data Folder", " My Documents/User Modules Folder")
        toolsMenu.AppendItem(showConfigFolder)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnConfigFolder, showConfigFolder)



        #Help Menu
        showHelp = wx.MenuItem(helpMenu, -1, "Show Help\tF1", " Show Bible Analyzer User Manual")
        helpMenu.AppendItem(showHelp)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnHelp, showHelp)
        
        #BAsite = wx.MenuItem(helpMenu, -1, "Check for Update", " Check for Bible Analyzer Update")
        #helpMenu.AppendItem(BAsite)
        #self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnBAupdate, BAsite)

        BAforum = wx.MenuItem(helpMenu, -1, "Bible Analyzer Forum", " Visit the Bible Analyzer Forum")
        helpMenu.AppendItem(BAforum)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnBAforum, BAforum)

        PWsite = wx.MenuItem(helpMenu, -1, "Bible Analyzer Modules", " Bible Analyzer Modules Page")
        helpMenu.AppendItem(PWsite)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnModulesPage, PWsite)

        welcome = wx.MenuItem(helpMenu, -1, "View Welcome Page", " View Welcome Page")
        helpMenu.AppendItem(welcome)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnWelcome, welcome)

        about = wx.MenuItem(helpMenu, wx.ID_ABOUT, "About Bible Analyzer", " About Bible Analyzer")
        helpMenu.AppendItem(about)
        self.mainApp.Bind(wx.EVT_MENU, self.mainApp.OnAbout, about)
        
        ## Bookmarks
        self.addBookmarks()
        self.mainApp.Bind(wx.EVT_MENU_RANGE, self.doBookmark, id=10000, id2=10009)
        

        # Add menu to the menu bar
        self.Append(fileMenu, "&File")
        self.Append(self.viewMenu, "&View")
        self.Append(ctrlMenu, "Co&ntrol")
        self.Append(self.bmMenu, "Boo&kmarks")
        self.Append(editorMenu, "Scripture&Pad")
        self.Append(toolsMenu, "T&ools")        
        self.Append(helpMenu, "&Help")
                
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnMasterControlMini, id=1005)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnImageViewer, id=1004)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnLibViewer, id=1003)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnOptionsDlg, id=1006)
        
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnDevotions, id=1002)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnHelp, id=1007)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnMediaMini, id=1008)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnTTSSpeak, id=1009)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnToggleMini, id=1010)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnParGen, id=1011)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnSessMan, id=1012)
        self.mainApp.Bind(wx.EVT_TOOL, self.mainApp.OnRefPanel, id=1013)
        
        self.parGen = None

    
    ## Bookmarks
    def addBookmarks(self):
        try:
            bookmarkList = self.mainApp.config["settings"]["bookmarklist"]
            self.bookmarkList = ['Go to: %s' % bm for bm in bookmarkList]
        except:
            self.bookmarkList = ['Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty', 'Go to: Empty']

        cnt = 10000
        for item in self.bookmarkList:
            item = '%s\tAlt+%s' % (item, cnt -10000)
            self.bmMenu.Append(cnt, item, "")
            cnt += 1
        
    
    def doBookmark(self, event):
        id = event.GetId()
        #print id
        item = self.bmMenu.FindItemById(id)
        txt = item.GetLabel()
        txt = txt.replace('Go to: ', '')
        #print txt
        if txt == 'Empty' or wx.GetKeyState(wx.WXK_SHIFT):
            self.enterBookMark(id, item)
        else:
            self.mainApp.ShowRefContext(txt)

    
    def enterBookMark(self, id, item):
        dlg = wx.TextEntryDialog(
                self.mainApp, 'Enter Reference for Bookmark %s' % (id -10000),
                'Enter Bookmark', 'Python')

        dlg.SetValue("%s" % self.mainApp.currentRef)

        if dlg.ShowModal() == wx.ID_OK:
            ref = dlg.GetValue()
            if not ref:
                item.SetText('Go to: Empty\tAlt+%s' % (id -10000))
                return

            if ref[-1] == ':': 
                ref = ref + '1'
            ref = fixRef.prelimFix(ref)
            if not ':' in ref and ref[-1].isdigit():
                ref = ref + ':1'
            ref = fixRef.fix(ref)
            item.SetText('Go to: %s\tAlt+%s' % (ref, id -10000))

        dlg.Destroy()


        

    def helpToolbar(self):
        Pref = mainImg.getPrefBitmap()
        Help = mainImg.getHelpBitmap()
        Speech = mainImg.getSpeechBitmap()
        SpeechDis = mainImg.getSpeechDisBitmap()
        
        tb = Aui.AuiToolBar(self.mainApp, -1, wx.DefaultPosition, wx.DefaultSize, agwStyle = Aui.AUI_TB_PLAIN_BACKGROUND)
        tb.SetToolBitmapSize(wx.Size(32, 32))
        tb.AddSimpleTool(1009, "", Speech, "Start\Stop Text to Speech")
        tb.AddSimpleTool(1006, "", Pref, "User Preferences")
        tb.AddSimpleTool(1007, "", Help, "User Manual (Will open in browser)")
        
        #if self.mainApp.loadTts == False:
        #    tb.EnableTool(1009, False)
        
        return tb

    def viewerToolbar(self):
        Calendar = mainImg.getCalendarBitmap()
        Read = mainImg.getBookBitmap()
        View = mainImg.getViewBitmap()
        Hymn = mainImg.getHymnBitmap()
        Multi = mainImg.getMultiBitmap()
        Para = mainImg.getParallelBitmap()
        
        tb = Aui.AuiToolBar(self.mainApp, -1, wx.DefaultPosition, wx.DefaultSize, agwStyle = Aui.AUI_TB_PLAIN_BACKGROUND)
        tb.SetToolBitmapSize(wx.Size(32, 32))
        tb.AddSimpleTool(1002, "", Calendar, "Daily Devotions")
        tb.AddSimpleTool(1008, "", Hymn, "Audio\Quiz Player")        
        tb.AddSimpleTool(1011, "", Para, "Harmony\Parallel Generator")
        tb.AddSimpleTool(1003, "", Read, "Book\Article Viewer")
        tb.AddSimpleTool(1004, "", View, "Image Viewer")
        tb.AddSimpleTool(1010, "", Multi, "MultiWindow\AV-Strong Index")
        
        return tb


    def listCompleter(self, lst, template):
        def completer(query):
            formatted, unformatted = list(), list()
            if query:
                unformatted = [item for item in lst if item.startswith(query)]
                for item in unformatted:
                    s = item.find(query)
                    formatted.append(
                        template % (item[:s], query, item[s + len(query):])
                    )
            return formatted[:18], unformatted[:18]
        return completer


    def entryToolbar(self):
        tb = Aui.AuiToolBar(self.mainApp, -1, wx.DefaultPosition, wx.DefaultSize, agwStyle = Aui.AUI_TB_PLAIN_BACKGROUND)
        
        vsTemplate = '<font color="brown"><b>%s%s%s</b></font>'
        books = [
            'Genesis', 'Exodus', 'Leviticus', 'Numbers', 'Deuteronomy', 'Joshua', 'Judges',
            'Ruth', '1 Samuel', '2 Samuel', '1 Kings', '2 Kings', '1 Chronicles',
            '2 Chronicles', 'Ezra', 'Nehemiah', 'Esther', 'Job', 'Psalms', 'Proverbs',
            'Ecclesiastes', 'Song Solomon', 'Isaiah', 'Jeremiah', 'Lamentations', 'Ezekiel',
            'Daniel', 'Hosea', 'Joel', 'Amos', 'Obadiah', 'Jonah', 'Micah', 'Nahum',
            'Habbakkuk', 'Zephaniah', 'Haggai', 'Zechariah', 'Malachi', 
            'Matthew', 'Mark', 'Luke', 'John', 'Acts', 'Romans', '1 Corinthians', '2 Corinthians', 'Galatians',
            'Ephesians', 'Philippians', 'Colossians', '1 Thessalonians', '2 Thessalonians',
            '1 Timothy', '2 Timothy', 'Titus', 'Philemon', 'Hebrews', 'James', '1 Peter',
            '2 Peter', '1 John', '2 John', '3 John', 'Jude', 'Revelation'
        ]
        
       
        wdTemplate = '%s<font color="brown"><b><u>%s</u></b></font>%s'
        
        self.QSentry = AutoTextCtrl(tb, size = (150, -1), vsCompleter = self.listCompleter(books, vsTemplate), wdCompleter = self.listCompleter(self.mainApp.txtList, wdTemplate))
        #self.QSentry = wx.ComboBox(tb, -1, 'Quick Search or Ref.', choices=[], size=(150, -1), style=wx.CB_DROPDOWN|wx.TE_PROCESS_ENTER)
        self.QSentry.SetToolTipString(' Begin typing search word(s) \n or a Bible reference (Alt+Q) ')
                
        tb.AddControl(self.QSentry, 'Quick Entry')
        #tb.AddSpacer(10)
                
        return tb

    
    def imageToolbar(self):
        tb = Aui.AuiToolBar(self.mainApp, -1, wx.DefaultPosition, wx.DefaultSize, agwStyle = Aui.AUI_TB_PLAIN_BACKGROUND)
        
        #png = wx.Image('ba-scroll32.png', wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        png = mainImg.getScrollBitmap()
        #mask = wx.Mask(png, wx.WHITE)
        #png.SetMask(mask)
        sb = wx.StaticBitmap(tb, -1, png, (0, 0), (png.GetWidth(), png.GetHeight()))
        
        st = wx.StaticText(tb, -1, self.mainApp.toolTxt)
        st.SetForegroundColour(wx.Colour(120, 0, 0))
        st.SetFont(wx.Font(11, wx.DEFAULT, wx.ITALIC, wx.NORMAL, 0))
        
        tb.AddSpacer(5)
        tb.AddControl(st, 'Quick Entry')
        tb.AddSpacer(10)
        tb.AddControl(sb, 'Quick Entry')
                
        return tb


        
    def buttonToolbar(self):
        self.buttonTB = Aui.AuiToolBar(self.mainApp, -1, wx.DefaultPosition, wx.DefaultSize, agwStyle = Aui.AUI_TB_PLAIN_BACKGROUND)
        Control = mainImg.getControlBitmap()
        #Control = wx.Image(("control.png")).ConvertToBitmap()
        
        size = (34, 30)
        segments = 4
        labels = ['B', 'R', 'C', 'D']
        tips = ['Bible Window', 'Results Window', 'Commentary Window', 'Dictionary Window']
        events = [self.bibMax, self.resMax, self.cmtMax, self.dctMax]
        
        segButton = SegmentButton(self.buttonTB, -1, size, segments, labels, events, tips)
        
        self.jumpBut = buttons.GenToggleButton(self.buttonTB, -1, 'J', size = (14, 18))
        self.jumpBut.SetUseFocusIndicator(False)
        self.jumpBut.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
        self.jumpBut.SetForegroundColour(wx.Colour(0, 0, 128))
        self.jumpBut.SetBezelWidth(1)
        self.jumpBut.Bind(wx.EVT_BUTTON, self.OnJumpState)
        self.jumpBut.SetToolTipString('Jump to corresponding window on click when page is maximized.')

        layBut = buttons.GenButton(self.buttonTB, -1, 'L', size = (14, 18))
        layBut.SetUseFocusIndicator(False)
        layBut.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
        layBut.SetForegroundColour(wx.Colour(0, 0, 128))
        layBut.SetBezelWidth(1)
        layBut.Bind(wx.EVT_BUTTON, self.layoutMenu)
        layBut.SetToolTipString('Save and select layouts')

        self.buttonTB.AddControl(self.jumpBut, '')
        self.buttonTB.AddSpacer(2)
        self.buttonTB.AddControl(segButton, '')
        self.buttonTB.AddSpacer(2)
        self.buttonTB.AddControl(layBut, '')
        self.buttonTB.AddSpacer(5)
        self.buttonTB.AddSimpleTool(1005, "", Control, "Master Control Panel")

        return self.buttonTB

    
    def ExportBib(self, event):
        dlg = dialogs.ExportBib(None, -1, "Export/Copy Bible")
        dlg.Show()

    
    def OnJumpState(self, event):
        self.mainApp.jumpState = self.jumpBut.GetValue()

    
    def checkLayout(self):
        if not self.lastState:
            print 'add laststate'
            self.lastState = self.mainApp.lastClosedLayout


    def bibMax(self, event):
        self.checkLayout()
        pane = self.getPane('Bible')
        if pane:
            self.sortPane(pane, 'Bible')
            self.scrollBib()

    def dctMax(self, event):
        self.checkLayout()
        pane = self.getPane('Dictionaries')
        if pane:
            self.sortPane(pane, 'Dictionaries')

    def resMax(self, event):
        self.checkLayout()
        pane = self.getPane('Results')
        if pane:
            self.sortPane(pane, 'Results')

    def cmtMax(self, event):
        self.checkLayout()
        pane = self.getPane('Commentaries')
        if pane:
            self.sortPane(pane, 'Commentaries')
        
        
    def sortPane(self, pane, name):
        #self.mainApp.Freeze()
    
        if pane.IsNotebookPage() and self.mainApp.mainMgr.GetPane('__notebook_0').IsClosed():
            self.mainApp.mainMgr.GetPaneByName('__notebook_0').NotClosed().Restore().Show()
            self.mainApp.mainMgr.Update()
            print '1'
        
        elif pane.IsNotebookPage() and self.mainApp.mainMgr.GetPane('__notebook_0').IsFloating():
            self.reDock(self.mainApp.mainMgr.GetPane('__notebook_0'))
            print '2'
            
        elif pane.IsClosed():
            self.openPane(pane)
            print '3'
            
        elif pane.IsFloating():
            if name == 'Commentaries' or name == 'Dictionaries':
                if pane.dock_direction_get() == 6:
                    pane.dock_direction_set(3)
            elif name == 'Results':
                if pane.dock_direction_get() == 6:
                    pane.dock_direction_set(2)
            
            print '4', 
            self.reDock(pane)
            
        else:
            self.doMax(pane, name)
            if name == 'Bible':
                tab = self.mainApp.bibNB.GetSelection()
                self.mainApp.bibDisplay[tab].SetFocus()
                if '<a name="line"></a>' in self.mainApp.bibCurrent:
                    self.mainApp.bibDisplay[tab].ScrollToAnchor('line')
                    self.mainApp.bibDisplay[tab].ScrollLines(-2)
                    
            if name == 'Dictionaries':
                self.mainApp.dctDisplay[self.mainApp.dctNB.GetSelection()].SetFocus()
            if name == 'Commentaries':
                if self.mainApp.cmtNB.GetSelection() == 0:
                    self.mainApp.notes.SetFocus()
                else:
                    self.mainApp.cmtDisplay[self.mainApp.cmtNB.GetSelection() -1].SetFocus()
            if name == 'Results':
                if self.mainApp.resNB.GetSelection() == 0:
                    self.mainApp.editor.SetFocus()
                else:
                    self.mainApp.resDisplay[self.mainApp.resNB.GetSelection() -1].SetFocus()
            
            self.mainApp.OnSetBorder(None)
            print '5'
        
        #self.mainApp.Thaw()
    
    
    def reDock(self, pane):
        self.mainApp.mainMgr.RestoreMaximizedPane()
        pane.Dock()
        
        self.mainApp.Freeze()
        self.mainApp.mainMgr.Update()
        self.mainApp.Thaw()


    def openPane(self, pane):
        self.mainApp.mainMgr.RestoreMaximizedPane()
        if pane.dock_direction == 0 and pane.name == 'Dictionaries':
            pane.Bottom().Layer(2).Position(1).Show().NotClosed()
        elif pane.dock_direction == 0 and pane.name == 'Commentaries':
            pane.Bottom().Layer(2).Position(0).Show().NotClosed()
        elif pane.dock_direction == 0 and pane.name == 'Bible':
            pane.CenterPane.Show().NotClosed()
        elif pane.dock_direction == 0 and pane.name == 'Results':
            pane.Right().Layer(1).Show().NotClosed()
        else:
            pane.Show().NotClosed()
        self.mainApp.mainMgr.Update()

        
    def doMax(self, pane, name):
        if not self.getMaxPane():
            print 'save laststate'
            self.lastState = self.mainApp.mainMgr.SavePerspective()
            
        if pane.IsNotebookPage():
            nb = self.mainApp.mainMgr.GetNotebooks()[0]
            nbInfo = self.mainApp.mainMgr.GetPane(nb)
            if nbInfo.IsMaximized():
                print 'is nb max', name
                self.mainApp.mainMgr.RestoreMaximizedPane()
                self.mainApp.mainMgr.LoadPerspective(self.lastState, False)
            else:
                print 'nb not max', name
                for idx in range(nb.GetPageCount()):
                    if nb.GetPageText(idx) == name:
                        nb.SetSelection(idx)
                self.mainApp.mainMgr.MaximizePane(nbInfo)
        elif pane.IsMaximized():
            self.mainApp.mainMgr.RestoreMaximizedPane()
            print 'is max', name
            self.mainApp.mainMgr.LoadPerspective(self.lastState, False)
        else:
            print 'not max', name
            self.mainApp.mainMgr.MaximizePane(pane)
        
        self.mainApp.Freeze()
        self.mainApp.mainMgr.Update()
        self.mainApp.Thaw()

    def getMaxPane(self):
        for pane in self.mainApp.mainMgr.GetAllPanes():
            if pane.IsMaximized():
                return True
        return False


    def getPane(self, name):
        for pane in self.mainApp.mainMgr.GetAllPanes():
            if pane.name == name:
                return pane


    def layoutMenu(self, event):
        if event:
            but = event.GetEventObject()
        
        self.layMenu = wx.Menu()
        delMenu = wx.Menu()
        
        cnt = 2000 
        for item in self.mainApp.layoutTitles:
            self.layMenu.Append(cnt + 1, item, "")
            cnt += 1
        
        cnt = 2101
        if len(self.mainApp.layoutTitles) > 6:
            for item in self.mainApp.layoutTitles[6:]:
                delMenu.Append(cnt, item, "")
                cnt += 1
            
            self.layMenu.AppendSeparator()
            self.layMenu.AppendMenu(2199, 'Delete a Layout', delMenu)
        
        self.mainApp.Bind(wx.EVT_MENU, self.addLayout, id=2001)
        self.mainApp.Bind(wx.EVT_MENU_RANGE, self.loadLayout, id=2002, id2=2099)
        self.mainApp.Bind(wx.EVT_MENU_RANGE, self.delLayout, id=2101, id2=2198)
        
        if event:

            self.mainApp.PopupMenu(self.layMenu)
            self.popMenu = self.layMenu


    def addLayout(self, event):
        dlg = wx.TextEntryDialog(self.mainApp, "Enter a name for the new layout:\n(If a layout with this name exists, it will be replaced)", "Bible Analyzer Layout")

        if dlg.ShowModal() != wx.ID_OK:
            return
        
        title = dlg.GetValue()
        
        if title in self.mainApp.layoutTitles:
            idx = self.mainApp.layoutTitles.index(title)
            self.mainApp.layoutList[idx -1] = self.mainApp.mainMgr.SavePerspective()
        else:
            self.mainApp.layoutTitles.append(title)
            self.mainApp.layoutList.append(self.mainApp.mainMgr.SavePerspective())
        
        statText = 'Current Layout: %s' % title
        self.mainApp.statusbar.SetStatusText(statText, 3)
        
        dlg.Destroy()


    def loadLayout(self, event):
        #self.mainApp.Hide()
        self.mainApp.mainMgr.RestoreMaximizedPane()
        
        idx = event.GetId() - 2000
        
        self.mainApp.mainMgr.LoadPerspective(self.mainApp.layoutList[idx -2])
        #self.mainApp.mainMgr.LoadPerspective(self.mainApp.layoutList[idx -2])
        
        self.scrollBib()
        
        title = self.mainApp.layoutTitles[idx -1]
        titleStrip = re.search(r'^(.+?) -', title)
        if titleStrip:
            title = titleStrip.group(1)
        
        statText = 'Current Layout: %s' % title
        self.mainApp.statusbar.SetStatusText(statText, 3)
        #self.mainApp.Thaw()
        

    def delLayout(self, event):
        idx = event.GetId() - 2101
        del self.mainApp.layoutTitles[idx +6]
        del self.mainApp.layoutList[idx +5]
        
        
    def scrollBib(self):
        tab = self.mainApp.bibNB.GetSelection()
        if self.mainApp.currentRef[-1].isdigit() and tab != len(self.mainApp.bibFileList):
            self.mainApp.Freeze()
            print self.mainApp.currentRef, 'mm'
            self.mainApp.bibDisplay[tab].ScrollToAnchor(self.mainApp.currentRef)
            self.mainApp.bibDisplay[tab].ScrollLines(-2)
            self.mainApp.Thaw()

