import wx, os, re, cPickle
import wx.lib.buttons as buttons
import wx.richtext as rt
import fixRef

from artManager import ArtManager
from resources import *
from labelBook import LabelBook



## Make Gradient Title
class FMTitlePanel(wx.Panel):
    def __init__(self, parent, title):
        wx.Panel.__init__(self, parent)
        self._title = title

        # Set the panel size
        dc = wx.MemoryDC()
        dc.SelectObject(wx.EmptyBitmap(1, 1))
        dc.SetFont(wx.SystemSettings_GetFont( wx.SYS_DEFAULT_GUI_FONT ))
        
        ww, hh = dc.GetTextExtent("Tp")
        dc.SelectObject(wx.NullBitmap)

        self.SetSize(wx.Size(ww, hh + 12))

        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)

    def OnEraseBackground(self, event):
        pass


    def OnPaint(self, event):
        dc = wx.BufferedPaintDC(self)

        # Draw the background
        colour1 = wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE)
        colour2 = ArtManager.Get().LightColour(colour1, 90)
        ArtManager.Get().PaintStraightGradientBox(dc, self.GetClientRect(), colour1, colour2, False)

        # Draw the text
        #font = wx.SystemSettings_GetFont(wx.SYS_DEFAULT_GUI_FONT)
        font = wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0)
        #font.SetWeight(wx.BOLD)
        dc.SetFont(font)
        dc.SetTextForeground(wx.BLACK)
        dc.DrawText(self._title, 5, 5)




## Main Dialog
class Options(wx.Dialog):
    def __init__(self, parent=None):
        wx.Dialog.__init__(self, parent, -1, "Preferences", style = wx.SYSTEM_MENU|wx.CAPTION|wx.CLOSE_BOX)
        
        self.parent = parent
        self.mainApp = wx.GetApp().GetTopWindow()
        self._book = None
        
        self.newFace = self.mainApp.SPFace
        self.newSize = self.mainApp.SPSize
    
        self.SetSize((400, 550))
        
        CloseButton = wx.Button(self, -1, "&Save")
        
        #self.HelpButton = wx.Button(self, -1, '?', size=(22, 23)) #, style=wx.BORDER_NONE)
        
        #self.HelpButton.SetToolTip(wx.ToolTip('Help for the Selected Tab'))
        #self.HelpButton.SetFont(wx.Font(11, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, "Verdana"))
        #self.HelpButton.SetForegroundColour(wx.Colour(128, 0, 0))
        
        # Create the main book and add some pages into it
        
        self._book = LabelBook(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize,\
                               style = INB_NO_RESIZE | INB_LEFT | INB_DRAW_SHADOW | INB_BORDER | INB_USE_PIN_BUTTON)

        self._book.SetColour(INB_TAB_AREA_BACKGROUND_COLOR, ArtManager.Get().GetMenuFaceColour())

        colour = wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE)
        self._book.SetColour(INB_ACTIVE_TAB_COLOR, colour)
        
        self.Initialise()
        
        sz = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sz)
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        
        sz.Add(self._book, 1, wx.EXPAND)
        hsizer.Add(CloseButton, 0, wx.EXPAND | wx.ALIGN_RIGHT|wx.BOTTOM|wx.RIGHT, 5)
        #hsizer.Add((110, 0))
        #hsizer.Add(self.HelpButton, 0, wx.TOP, 2)

        sz.Add(wx.StaticLine(self), 0, wx.EXPAND | wx.TOP | wx.BOTTOM, 3)
        sz.Add(hsizer, 0, wx.ALIGN_RIGHT | wx.ALL, 2)
        
        #sz.Layout()
        self.Centre()
        
        self.Bind(wx.EVT_BUTTON, self.OnClose, CloseButton)
        #self.Bind(wx.EVT_CLOSE, self.OnClose)
        #self.Bind(wx.EVT_BUTTON, self.OnHelp, self.HelpButton)
        
    
    def Initialise(self):
        self.Mods = self.mainApp.modules(self)
    
        self._book.DeleteAllPages()
        self._book.AddPage(self.CreateAppearPage(), "Appearance", True)
        self._book.AddPage(self.DefaultSettings(), "Options", False)
        #self._book.AddPage(self.CreateOptionsPage(), "Preferences", False)
        self._book.AddPage(self.FormatCopy(), "Copy Format", False)
        self._book.AddPage(self.Mods, "Session Manager", False)
        self._book.AddPage(self.ScripturePad(), "Font Properties", False)
        self._book.AddPage(self.AudioConfig(), "Audio Settings", False)
        self._book.AddPage(self.Text2Speech(), "Text to Speech", False)
    
    
    def OnHelp(self, event):
        tab = self._book.GetSelection()
        if tab == 0:
            context = 121 
        elif tab == 1:
            context = 131 
        elif tab == 2:
            context = 141 
        elif tab == 3:
            context = 151 
        elif tab == 4:
            context = 271 
        elif tab == 5:
            context = 511 
        elif tab == 6:
            context = 391 
        else:
            return
        
        if context:
            self.mainApp.OnContextHelp(context)
    
    
    def OnClose(self, event):
        '''
        del self.mainApp.NoBibLoad[:]
        for id in self.Mods.bibId:
            box = self.Mods.modPanel.FindWindowById(id)
            if box.IsChecked() == False:
                #print id, box.GetLabel()
                self.mainApp.NoBibLoad.append(self.Mods.bibAllAbbr[id -300])
        del self.mainApp.NoDctLoad[:]
        for id in self.Mods.dctId:
            box = self.Mods.modPanel.FindWindowById(id)
            if box.IsChecked() == False:
                self.mainApp.NoDctLoad.append(self.Mods.dctAllAbbr[id -400])
        del self.mainApp.NoCmtLoad[:]
        for id in self.Mods.cmtId:
            box = self.Mods.modPanel.FindWindowById(id)
            if box.IsChecked() == False:
                self.mainApp.NoCmtLoad.append(self.Mods.cmtAllAbbr[id -500])
        '''
        
        self.mainApp.GetOptions()
        self.mainApp.OnBibNBChanged(None)
        self.Hide()
        #self.Destroy()
       
    
    def OnValidate(self, event):
        # Append Title
        if self.appendTitleCB.IsChecked():
            added = self.appendTitleTC.GetValue()
            if added == '':
                pass
            else:
                added = ' - %s' % added
            title = '%s%s' % (self.mainApp.title, added)
        else:
            title = self.mainApp.title
        self.mainApp.SetTitle(title)
        self.mainApp.appendTitle = self.appendTitleCB
        
        self.OnClose()
        self.Hide()
        
        #self.Destroy()
        #self.mainApp.optionsDlg = None
        
    
    def ScripturePad(self):
        fOpt = wx.Panel(self._book, -1)
        
        fntEn = wx.FontEnumerator()
        fntEn.EnumerateFacenames()
        fntList = fntEn.GetFacenames()
        fntList.sort()        
        
        self.useOldFont = wx.CheckBox(fOpt, -1, "Use Old English Font for AV-1611")
        
        self.fontSizeRB = wx.RadioBox(fOpt, -1, "Window Font Size", choices=["100%", "120%", "150%", "200%"], majorDimension=1, style=wx.RA_SPECIFY_ROWS)
        self.popFontSize = wx.RadioBox(fOpt, -1, "Popup Font Size", choices=["Normal      ", "Large", "Larger"], majorDimension=1, style=wx.RA_SPECIFY_ROWS)
        
        #self.fontFaceRB = wx.RadioBox(fOpt, -1, "Window Font Face", choices=["Georgia, (Serif)     ", "Verdana, (Sans)"], majorDimension=1, style=wx.RA_SPECIFY_ROWS)
        ffLabel = wx.StaticText(fOpt, -1, "Default Window Font")
        
        self.fontFace = wx.Choice(fOpt, -1, choices=fntList)
        self.fontFace.SetStringSelection(self.mainApp.font)
        
        self.versePopupRB = wx.RadioBox(fOpt, -1, "Popup Background Color", choices=["Yellow ", "Red", "Blue", "Green"], majorDimension=1, style=wx.RA_SPECIFY_ROWS)
        
        fontText = wx.StaticText(fOpt, -1, "S-Pad/Notes/Prayer List Default Font")
        fontText2 = wx.StaticText(fOpt, -1, "Sets the default properties for a new page.\nTo change properties of a current page,\nuse the toolbar or menu.")
        font = wx.Button(fOpt, -1, "S-Pad Font", size=(-1, 25))
        
        ffLabel.SetForegroundColour(wx.Colour(128, 0, 0))
        font.SetForegroundColour(wx.Colour(142, 35, 35))
        fontText.SetForegroundColour(wx.Colour(142, 35, 35))
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        fOpt.SetSizer(sizer)
        
        title = FMTitlePanel(fOpt, "Font Properties")
        sizer.Add(title, 0, wx.EXPAND | wx.ALL, 2)
        
        sizer.Add((-1, 10))
        sizer.Add(ffLabel, 0, wx.ALIGN_CENTER_HORIZONTAL, 5)
        sizer.Add(self.fontFace, 0, wx.TOP|wx.ALIGN_CENTER_HORIZONTAL, 2)
        sizer.Add((-1, 10))        
        sizer.Add(self.useOldFont, 0, wx.TOP|wx.ALIGN_CENTER_HORIZONTAL, 5)
        sizer.Add((-1, 10))
        sizer.Add(self.fontSizeRB, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5, 0)
        sizer.Add((-1, 5))
        sizer.Add(self.popFontSize, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5, 0)
        sizer.Add((-1, 5))
        sizer.Add(self.versePopupRB, 0, wx.EXPAND | wx.LEFT|wx.RIGHT, 5)
        sizer.Add((-1, 15))
        sizer.Add(fontText, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        sizer.Add((-1, 7))
        sizer.Add(fontText2, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        
        sizer.Add((-1, 10))
        sizer.Add(font, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        
        self.Bind(wx.EVT_BUTTON, self.OnSPFont, font)
        #self.Bind(wx.EVT_CHOICE, self.OnFont, self.fontFace)
        
        return fOpt

    def OnFont(self, event):
        self.mainApp.font = self.fontFace.GetStringSelection()
        
        
    def OnSPFont(self, evt):
        font = wx.Font(10, wx.ROMAN, wx.NORMAL, wx.NORMAL, 0, "Georgia")
        font.SetFaceName(self.mainApp.SPFace)
        font.SetPointSize(self.mainApp.SPSize)
        
        fontData = wx.FontData()
        fontData.EnableEffects(False)
        fontData.SetInitialFont(font)

        dlg = wx.FontDialog(self, fontData)
        if dlg.ShowModal() == wx.ID_OK:
            data = dlg.GetFontData()
            newfontData = data.GetChosenFont()
            if newfontData:
                self.newFace = newfontData.GetFaceName()
                self.newSize = newfontData.GetPointSize()
                #basicStyle = self.editor.GetBasicStyle()
                #basicStyle.SetFont(fontData)
                newData = wx.Font(10, wx.ROMAN, wx.NORMAL, wx.NORMAL, 0, "Georgia")
                newData.SetFaceName(self.newFace)
                newData.SetPointSize(self.newSize)
                self.mainApp.editor.BeginFont(newData)
                self.mainApp.notes.BeginFont(newData)
        dlg.Destroy()

    
    
    def FormatCopy(self):
        format = wx.Panel(self._book, -1)
        self.formatStyle = wx.RadioBox(format, -1, "Copy/Export Format Style", choices=["1   ", "2   ", "3   ", "4   "], majorDimension=1, style=wx.RA_SPECIFY_ROWS)
        #self.refPos = wx.RadioBox(format, -1, "Reference Position", choices=["Before Text", "After Text"], majorDimension=1, style=wx.RA_SPECIFY_ROWS)
        
        self.refPos = wx.CheckBox(format, -1, ' Reference Before Text (After is Default)')
        self.formatDisp = wx.html.HtmlWindow(format, -1, style=wx.SUNKEN_BORDER)
        self.formatFullBk = wx.CheckBox(format, -1, ' Full Book Name') 
        self.showBibleVer = wx.CheckBox(format, -1, ' Show Bible Version') 
        self.refSep = wx.CheckBox(format, -1, ' Reference on Separate Line')
        self.refParen = wx.CheckBox(format, -1, ' Parenthesis Around References')
        self.refColor = wx.CheckBox(format, -1, ' References in Color (S-Pad only)')
        
        fsizer = wx.BoxSizer(wx.VERTICAL)
        format.SetSizer(fsizer)
        
        title = FMTitlePanel(format, "Copied Text Format:")
        fsizer.Add(title, 0, wx.EXPAND | wx.ALL, 2)
        
        fsizer.Add(self.formatStyle, 0, wx.EXPAND | wx.ALL, 5)
        
        fsizer.Add(self.refPos, 0, wx.LEFT, 5)
        fsizer.Add((-1, 3))
        fsizer.Add(self.refSep, 0, wx.LEFT, 5)
        fsizer.Add((-1, 3))
        fsizer.Add(self.refParen, 0, wx.LEFT, 5)
        fsizer.Add((-1, 3))
        fsizer.Add(self.refColor, 0, wx.LEFT, 5)
        fsizer.Add((-1, 3))
        fsizer.Add(self.formatFullBk, 0, wx.LEFT, 5)
        fsizer.Add((-1, 3))
        fsizer.Add(self.showBibleVer, 0, wx.LEFT, 5)
        
        fsizer.Add((-1, 3))
        fsizer.Add(self.formatDisp, 1, wx.EXPAND | wx.ALL, 5)
        
        self.formatStyle.Bind(wx.EVT_RADIOBOX, self.OnStyle)
        self.formatFullBk.Bind(wx.EVT_CHECKBOX, self.OnStyle)
        self.showBibleVer.Bind(wx.EVT_CHECKBOX, self.OnStyle)
        self.refPos.Bind(wx.EVT_CHECKBOX, self.OnStyle)
        self.refSep.Bind(wx.EVT_CHECKBOX, self.OnStyle)
        self.refParen.Bind(wx.EVT_CHECKBOX, self.OnStyle)
        self.refColor.Bind(wx.EVT_CHECKBOX, self.OnStyle)

        self.formatStyle.SetSelection(self.parent.copyFormat)
        self.OnStyle(None)
        
        return format
    
    
    def OnStyle(self, event):
        txtBib = 'AV'
        self.parent.copyFormat = self.formatStyle.GetSelection()
            
        txtBody = '''{1} In the beginning God created the heaven and the earth.<br>{2} And the earth was without form, and void;
        and darkness <i>was</i> upon the face of the deep. And the Spirit of God moved upon the face of the waters.<br>{3} And God said,
        Let there be light: and there was light.<br>'''

        if self.formatFullBk.GetValue() == 1:
            txtRef = 'Genesis 1:1-3'
        else:
            txtRef = 'Gen 1:1-3'

        if self.showBibleVer.GetValue() == 1:
            txtRef += ' AV'
        
        bRef = r'\1'
        if self.refParen.GetValue() == 1:
            txtRef = '(%s)' % txtRef
            bRef = r'(\1)'

        if self.refSep.GetValue() == 1:
            txtRef = '<br>%s<br>' % txtRef
        
        if self.refColor.GetValue() == 1:
            txtBody = txtBody.replace('{', '<font color="brown">{')
            txtBody = txtBody.replace('}', '}</font>')
            txtRef = '<font color="brown">%s</font>' % txtRef
 
        ext = (txtBody[:-4], txtRef)
        if self.refPos.GetValue() == 1:
           ext = (txtRef, txtBody[:-4])

        if self.formatStyle.GetSelection() == 0:
            txt = '<font face="Verdana">%s %s</font>' % ext
            txt = re.sub('{.}', '', txt)
            self.formatDisp.SetPage(txt)
        elif self.formatStyle.GetSelection() == 1:
            txt = '<font face="Verdana">%s %s</font>' % ext
            txt = txt.replace('<br>', ' ')
            txt = re.sub('{.}', '', txt)
            self.formatDisp.SetPage(txt)
        elif self.formatStyle.GetSelection() == 2:
            txt = '<font face="Verdana">%s %s</font>' % ext
            txt = txt.replace('<br>', ' ')
            txt = re.sub(r'{(.)}', bRef, txt)
            self.formatDisp.SetPage(txt)
        elif self.formatStyle.GetSelection() == 3:
            txt = '<font face="Verdana">%s %s</font>' % ext
            txt = re.sub(r'{(.)}', bRef, txt)
            self.formatDisp.SetPage(txt)
            
        
    
    def DefaultSettings(self):
        panel = wx.Panel(self._book, -1)
        
        self.defaultDctLabel = wx.StaticText(panel, -1, "Default Dictionary")
        self.defaultDct = wx.Choice(panel, -1, choices = self.mainApp.dDescList)
        self.defaultCmtLabel = wx.StaticText(panel, -1, "Default Commentary")
        self.defaultCmt = wx.Choice(panel, -1, choices = self.mainApp.cDescList)
        self.defaultStrongsLabel = wx.StaticText(panel, -1, "Default Strong's Resource")
        self.defaultStrongs = wx.Choice(panel, -1, choices = self.mainApp.dctStrongList)
        self.defaultPopupLabel = wx.StaticText(panel, -1, "Default Popup Text")
        self.defaultPopupTxt = wx.Choice(panel, -1, choices = self.mainApp.bDescList)
        self.defaultStrongsPopLabel = wx.StaticText(panel, -1, "Default Strong's Popup Resource")
        self.defaultStrongsPop = wx.Choice(panel, -1, choices = self.mainApp.dctStrongList)

        self.useMini = wx.CheckBox(panel, -1, "Show Dct/Cmt in MultiWindow")
        self.showRedLetterCk = wx.CheckBox(panel, -1, "Show Red Letters")
        self.showBibSum = wx.CheckBox(panel, -1, "Show Bible Chapter Summary")
        
        self.bibExpandMW = wx.CheckBox(panel, -1, "Use MultiWindow When Bible is Expanded")
        
        self.popRange = wx.Choice(panel, -1, choices = ['1', '2', '3', '4', '5', '6', '7'])
        self.popRangeLabel = wx.StaticText(panel, -1, "Maximum Popup Verse Range")
        
        #self.showDctWL = wx.CheckBox(panel, -1, "Show Dictionary Topics List on Startup")
        #self.showCmtWL = wx.CheckBox(panel, -1, "Show Commentary Verse List on Startup")
        
        self.showBkWin = wx.CheckBox(panel, -1, "Show Books in Separate Window")
        
        self.showQuote = wx.CheckBox(panel, -1, "Show Date and Quote at Startup")
        self.loadFile = wx.CheckBox(panel, -1, "Load Last Saved S-Pad File at Startup (XML)")
        
        self.fullBibRef = wx.CheckBox(panel, -1, "Show Full References in Bible Window")
        self.saveAllMW = wx.CheckBox(panel, -1, "Save all MultiWin Pages in History")
        
        self.popRange.SetSelection(2)


        self.defaultDctLabel.SetForegroundColour(wx.Colour(128, 0, 0))
        self.defaultCmtLabel.SetForegroundColour(wx.Colour(128, 0, 0))
        self.defaultStrongsLabel.SetForegroundColour(wx.Colour(128, 0, 0))
        self.defaultStrongsPopLabel.SetForegroundColour(wx.Colour(128, 0, 0))
        self.defaultPopupLabel.SetForegroundColour(wx.Colour(128, 0, 0))
        
        optSizer = wx.BoxSizer(wx.VERTICAL)
        
        title = FMTitlePanel(panel, "User Options:")
        optSizer.Add(title, 0, wx.EXPAND | wx.ALL, 2)
        
        optSizer.Add((-1, 5))
        optSizer.Add(self.defaultDctLabel, 0, wx.ALIGN_CENTER | wx.LEFT|wx.RIGHT, 5)
        optSizer.Add((-1, 2))
        optSizer.Add(self.defaultDct, 0, wx.EXPAND | wx.LEFT|wx.RIGHT, 5)
        
        optSizer.Add((-1, 5))
        optSizer.Add(self.defaultCmtLabel, 0, wx.ALIGN_CENTER | wx.LEFT|wx.RIGHT, 5)
        optSizer.Add((-1, 2))
        optSizer.Add(self.defaultCmt, 0, wx.EXPAND | wx.LEFT|wx.RIGHT, 5)

        optSizer.Add((-1, 5))
        optSizer.Add(self.defaultStrongsLabel, 0, wx.ALIGN_CENTER | wx.LEFT|wx.RIGHT, 5)
        optSizer.Add((-1, 2))
        optSizer.Add(self.defaultStrongs, 0, wx.EXPAND | wx.LEFT|wx.RIGHT, 5)

        optSizer.Add((-1, 5))
        optSizer.Add(self.defaultPopupLabel, 0, wx.ALIGN_CENTER | wx.LEFT|wx.RIGHT, 5)
        optSizer.Add((-1, 2))
        optSizer.Add(self.defaultPopupTxt, 0, wx.EXPAND | wx.LEFT|wx.RIGHT, 5)

        optSizer.Add((-1, 5))
        optSizer.Add(self.defaultStrongsPopLabel, 0, wx.ALIGN_CENTER | wx.LEFT|wx.RIGHT, 5)
        optSizer.Add((-1, 2))
        optSizer.Add(self.defaultStrongsPop, 0, wx.EXPAND | wx.LEFT|wx.RIGHT, 5)

        optSizer.Add((-1, 10))
        optSizer.Add(self.useMini, 0, wx.LEFT|wx.RIGHT, 5)
        optSizer.Add((-1, 3))
        optSizer.Add(self.bibExpandMW, 0, wx.LEFT|wx.RIGHT, 5)
        optSizer.Add((-1, 10))
        
        optSizer.Add(self.showRedLetterCk, 0, wx.LEFT|wx.RIGHT, 5)
        optSizer.Add((-1, 3))
        #optSizer.Add(self.showDctWL, 0, wx.LEFT|wx.RIGHT, 5)
        #optSizer.Add((-1, 3))
        optSizer.Add(self.showBibSum, 0, wx.LEFT|wx.RIGHT, 5)
        optSizer.Add((-1, 3))
        #optSizer.Add(self.showCmtWL, 0, wx.LEFT|wx.RIGHT, 5)
        #optSizer.Add((-1, 3))
        optSizer.Add(self.showBkWin, 0, wx.LEFT|wx.RIGHT, 5)
        optSizer.Add((-1, 10)) 
        
        optSizer.Add(self.showQuote, 0, wx.LEFT|wx.RIGHT, 5)
        optSizer.Add((-1, 3))
        optSizer.Add(self.loadFile, 0, wx.LEFT|wx.RIGHT, 5)
        optSizer.Add((-1, 3))
        optSizer.Add(self.fullBibRef, 0, wx.LEFT|wx.RIGHT, 5)        
        optSizer.Add((-1, 3))
        optSizer.Add(self.saveAllMW, 0, wx.LEFT|wx.RIGHT, 5)        
        optSizer.Add((-1, 10))

        rangeSizer = wx.BoxSizer(wx.HORIZONTAL)
        rangeSizer.Add(self.popRange, 0, wx.LEFT|wx.RIGHT, 5)
        rangeSizer.Add(self.popRangeLabel, 0, wx.ALIGN_CENTER, 0)
        optSizer.Add(rangeSizer, 0, 0, 0)
        
        panel.SetSizer(optSizer)
        
        self.defaultDct.SetSelection(self.mainApp.defaultDctIdx)
        panel.SetSizer(optSizer)
        
        return panel
        

    def CreateAppearPage(self):
        panel = wx.Panel(self._book, -1)
        
        self.backColorIdx = wx.RadioBox(panel, -1, "Verse Background Color", choices=["Gray", "Red", "Blue", "Green"], majorDimension=1, style=wx.RA_SPECIFY_ROWS)
        colors = ['None', 'Blue', 'Green', 'Yellow', 'Teal', 'Pink', 'Purple', 'Gray']
        
        colorHeader = wx.StaticText(panel, -1, "Background Colors")
        dctClrTxt = wx.StaticText(panel, -1, "Dictionary Windows")
        self.dctColorIdx = wx.Choice(panel, -1, choices = colors)
        cmtClrTxt = wx.StaticText(panel, -1, "Commentary Windows")
        self.cmtColorIdx = wx.Choice(panel, -1, choices = colors)
        resClrTxt = wx.StaticText(panel, -1, "Results Windows")
        self.resColorIdx = wx.Choice(panel, -1, choices = colors)
        refClrTxt = wx.StaticText(panel, -1, "Reference Panel")
        self.refColorIdx = wx.Choice(panel, -1, choices = colors)
        
        colorHeader.SetForegroundColour(wx.Color(128, 0, 0))
        
        self.appendTitleCB = wx.CheckBox(panel, -1, "Append Text to Titlebar (50 char. max.)")
        self.appendTitleTC = wx.TextCtrl(panel, -1,)
        
        self.appendTitleTC.SetMaxLength(50)

        self.appendToolCB = wx.CheckBox(panel, -1, "Append Text to Message Toolbar")
        self.appendToolTC = wx.TextCtrl(panel, -1,)
        
        txtHeader = wx.StaticText(panel, -1, "Texts will update on next start.")
        txtHeader.SetForegroundColour(wx.Color(128, 0, 0))
        
        #self.appendToolTC.SetMaxLength(50)



        self.appendTitleTC.SetMaxLength(50)
        fsizer = wx.BoxSizer(wx.VERTICAL)
        csizer1 = wx.BoxSizer(wx.HORIZONTAL)
        csizer2 = wx.BoxSizer(wx.HORIZONTAL)
        csizer3 = wx.BoxSizer(wx.HORIZONTAL)
        csizer4 = wx.BoxSizer(wx.HORIZONTAL)
        
        panel.SetSizer(fsizer)

        title = FMTitlePanel(panel, "Application Appearance:")
        
        fsizer.Add((-1, 5))
        fsizer.Add(title, 0, wx.EXPAND | wx.LEFT|wx.RIGHT, 5)

        csizer1.Add(dctClrTxt, 0, wx.LEFT, 10)
        csizer1.Add((-1, -1), 1, 0, 0)
        csizer1.Add(self.dctColorIdx, 0, wx.ALIGN_RIGHT|wx.RIGHT, 10)
        csizer2.Add(cmtClrTxt, 0, wx.LEFT, 10)
        csizer2.Add((-1, -1), 1, 0, 0)
        csizer2.Add(self.cmtColorIdx, 0, wx.ALIGN_RIGHT|wx.RIGHT, 10)
        csizer3.Add(resClrTxt, 0, wx.LEFT, 10)
        csizer3.Add((-1, -1), 1, 0, 0)
        csizer3.Add(self.resColorIdx, 0, wx.ALIGN_RIGHT|wx.RIGHT, 10)
        csizer4.Add(refClrTxt, 0, wx.LEFT, 10)
        csizer4.Add((-1, -1), 1, 0, 0)
        csizer4.Add(self.refColorIdx, 0, wx.ALIGN_RIGHT|wx.RIGHT, 10)

        fsizer.Add(colorHeader, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL | wx.TOP, 12)
        fsizer.Add(csizer1, 0, wx.EXPAND|wx.TOP, 10)
        fsizer.Add(csizer2, 0, wx.EXPAND|wx.TOP, 5)
        fsizer.Add(csizer3, 0, wx.EXPAND|wx.TOP, 5)
        fsizer.Add(csizer4, 0, wx.EXPAND|wx.TOP, 5)

        fsizer.Add((-1, 5))
        fsizer.Add(self.backColorIdx, 0, wx.EXPAND | wx.LEFT|wx.RIGHT, 5)
        fsizer.Add((-1, 10))
        fsizer.Add(self.appendTitleCB, 0, wx.EXPAND | wx.TOP|wx.LEFT|wx.RIGHT, 5)
        fsizer.Add(self.appendTitleTC, 0, wx.EXPAND | wx.TOP|wx.LEFT|wx.RIGHT, 5)       
        fsizer.Add((-1, 10))
        fsizer.Add(self.appendToolCB, 0, wx.EXPAND | wx.TOP|wx.LEFT|wx.RIGHT, 5)
        fsizer.Add(self.appendToolTC, 0, wx.EXPAND | wx.TOP|wx.LEFT|wx.RIGHT, 5) 
        fsizer.Add((-1, 15))
        fsizer.Add(txtHeader, 0, wx.wx.ALIGN_CENTER, 0) 

        return panel
        

    def Text2Speech(self):
        speech = wx.Panel(self._book, -1)
        
        self.startSelect = wx.RadioBox(speech, -1, 'Startup Options', size=(200, -1), choices=['Custom Greeting (Enter Below)', 'No Speech at Start'], style=wx.RA_SPECIFY_ROWS)
        
        txtLabel = wx.StaticText(speech, -1, "Custom Greeting Read at Startup")
        self.ttsStart = wx.TextCtrl(speech, -1, '')
        
        formatList = ['16kHz, 16bit, mono', '22kHz, 16bit, mono']
        voiceLabel = wx.StaticText(speech, -1, "Select Default Voice")
        self.voices = wx.Choice(speech, -1, choices = self.mainApp.voices)
        
        volumeLabel = wx.StaticText(speech, -1, "Speech Volume (0% - 100%)")
        self.volSlider = wx.Slider(speech, -1, 80, 0, 100)#, style=wx.SL_AUTOTICKS)
        rateLabel = wx.StaticText(speech, -1, "Speech Rate (-10 - +10)")
        self.rateSlider = wx.Slider(speech, -1, 0, -10, 10, style=wx.SL_AUTOTICKS)
        self.skipRef = wx.CheckBox(speech, -1, 'Skip References While Reading.')

        self.voices.SetSelection(0)

        
        self.Bind(wx.EVT_COMMAND_SCROLL, self.setVolume, self.volSlider)
        self.Bind(wx.EVT_COMMAND_SCROLL, self.setRate, self.rateSlider)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        speech.SetSizer(sizer)
        
        title = FMTitlePanel(speech, "Text to Speech Options")
        sizer.Add(title, 0, wx.EXPAND | wx.ALL, 2)
        
        sizer.Add((-1, 5))
        sizer.Add(self.startSelect, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5)
        sizer.Add(txtLabel, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.TOP, 10)
        sizer.Add((-1, 2))
        sizer.Add(self.ttsStart, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5)
        
        sizer.Add(voiceLabel, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.TOP, 10)
        sizer.Add((-1, 2))
        sizer.Add(self.voices, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5)
        
        sizer.Add(volumeLabel, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.TOP, 15)
        sizer.Add((-1, 2))
        sizer.Add(self.volSlider, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5)
        sizer.Add(rateLabel, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.TOP, 10)
        sizer.Add((-1, 2))
        sizer.Add(self.rateSlider, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5)
        sizer.Add((-1, 10))
        sizer.Add(self.skipRef, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5)
                
        return speech


    def setVolume(self, event):
        level = self.volSlider.GetValue()
        self.mainApp.ttsVolume = level
        self.mainApp.tts.setVolume(level)

    def setRate(self, event):
        rate = self.rateSlider.GetValue()
        self.mainApp.ttsRate = rate
        self.mainApp.tts.setRate(rate)
        
        

    def AudioConfig(self):
        
        panel = wx.Panel(self._book, -1)
        
        txt1 = '''\
  Bible Analyzer can play audio of the Bible text in two ways. It can "read" the text with any \
installed Text-to-Speech voice, or it can play an audio recording (.mp3, wav) of the text.
  To hear the text by TTS, ensure the Bible Panel has the focus (click in it), and then click \
the Speech Button in the toolbar. The current verse will be highlighted and will advance/scroll \
with the reading of the text.
  To use pre-recorded audio files, use the "Link Audio Files" Button to select the parent \
folder, and ensure the "Use Audio Files" button is selected.
  To hear the text read click in the Bible Panel to set focus then click the Speech Button in the \
toolbar. The verses will be highlighted and the page will scroll.'''


        txtLabel1 = wx.StaticText(panel, -1, txt1)
        txtLabel1.Wrap(275)
        #txtLabel2 = wx.StaticText(panel, -1, txt2)
        #txtLabel3 = wx.StaticText(panel, -1, txt3)
        txtLabel1.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        #txtLabel2.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        #txtLabel3.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        
        self.readOpt = wx.RadioBox(panel, -1, "Speech Type", size=(90, -1), choices=["Text-To-Speech", "Use Audio Files" ], majorDimension=1, style=wx.RA_SPECIFY_ROWS)
        
        self.readChapCon = wx.CheckBox(panel, -1, "Continue Reading Next Chapter")
        self.showHighlight = wx.CheckBox(panel, -1, "Show Verse Highlight")
        linkButton = wx.Button(panel, -1, "Link Audio Files") #, size=(100, -1))
        
        self.Bind(wx.EVT_BUTTON, self.OnAudioBrowse, linkButton)
        
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        panel.SetSizer(sizer)
        
        title = FMTitlePanel(panel, "Audio Settings")
        sizer.Add(title, 0, wx.EXPAND | wx.ALL, 2)
        
        sizer.Add((-1, 5))
        sizer.Add(txtLabel1, 0, wx.ALIGN_LEFT|wx.LEFT|wx.RIGHT, 5)
        #sizer.Add((-1, 5))
        #sizer.Add(txtLabel2, 0, wx.ALIGN_LEFT|wx.LEFT|wx.RIGHT, 5)
        #sizer.Add((-1, 5))
        #sizer.Add(txtLabel3, 0, wx.ALIGN_LEFT|wx.LEFT|wx.RIGHT, 5)
        sizer.Add((-1, 10))
        sizer.Add(self.readOpt, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5)
        sizer.Add((-1, 10))
        sizer.Add(linkButton, 0, wx.ALIGN_CENTER|wx.LEFT|wx.RIGHT, 5)
        sizer.Add((-1, 10))
        sizer.Add(self.readChapCon, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5)
        sizer.Add((-1, 7))
        sizer.Add(self.showHighlight, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5)
       
        if not os.access(self.mainApp.userDir + r'/audioPaths.dat', 1):
            self.readOpt.EnableItem(1, False)

        return panel


    def OnAudioBrowse(self, event):
        
        #path = self.mainApp.audioPath
        #if self.folderPath.GetValue():
        #    print self.folderPath.GetValue(), 'folderPath'
        path = os.getcwd()
        
        dialog = wx.DirDialog(self,
                                  message = 'Select the folder that contains all the speech audio files.',
                                  defaultPath = path,
                                  style = wx.DD_DIR_MUST_EXIST)

        if dialog.ShowModal() == wx.ID_OK:
            wx.BeginBusyCursor()
            print dialog.GetPath(), 'path'

            pathsTmp = []
            audioPaths = {}
                
            for root, dirs, files in os.walk(dialog.GetPath()):
                for fn in files:
                    if fn.endswith('.mp3') or fn.endswith('.wav'):
                        pathsTmp.append('%s/%s' % (root, fn))
                                
            for pth in pathsTmp:    # This uses the path
                path = pth.replace('III ', '3 ')
                path = path.replace('II ', '2 ')
                path = path.replace('I ', '1 ')
                fn = os.path.basename(path)
                for book in fixRef.refOrder:
                    hit = re.search(r'%s\D* ?(\d+)' % book, fn)
                    if hit:
                        digits = hit.group(1)
                        
                        while digits[0] == '0':
                            digits = digits[1:]
                        
                        ref = '%s %s' % (fixRef.refmatches[book], digits)
                        
                        if not pth in audioPaths.values():
                            audioPaths[ref] = pth
                    
                    elif fn.endswith('Obadiah.mp3'):
                        audioPaths['Oba 1'] = pth
                        
            pFile = file(self.mainApp.userDir + r'/audioPaths.dat', 'wb')
            cPickle.dump(audioPaths, pFile)
            pFile.close()
                       
            self.readOpt.EnableItem(1, True)
            wx.EndBusyCursor()

        dialog.Destroy()



