import wx, wx.html, re
import codecs
import sqlite3 as sqlite
import utility, fixRef
import baseClass
import toolImg
import genButton as buttons
#import flatMenu as FM


intro = '''
<center><font color="#660000" face="Verdana" size="+2">Harmony
Display/Parallel
Generator</font></center>
<hr style="" font-family:="" verdana;=""><font face="Verdana">The
Harmony <b>Display/Parallel
Generator</b> features a new approach of viewing and studying the
Scriptures. In addition to displaying easily created harmony/parallel
modules (.pll), it can instantly generate parallel columns of text
according to user input. For instance the Generator can:<br>
<br>
</font>
<ul>
<li style="font-family: Verdana;"><font face="Verdana">Display easily
created <b>Harmony/Parallel</b> (.pll) modules when a module and title
is selected.</font></li>
<li style="font-family: Verdana;"><font face="Verdana">Instantly
display a different Bible
text when selected in the Bible Texts List. </font></li>
<li style="font-family: Verdana;"><font face="Verdana">Display <b>consecutive
and
non-consecutive ranges of text</b> in a single column. For example, <i>Gen
3.15, 18-20, Psa 23, Isa 40.1-2, Jn 3.18-20, Rom 4.5</i> can be
displayed
by simply typing, or pasting, it in the <b>Entry Box</b> below.</font></li>
<li style="font-family: Verdana;"><font face="Verdana"><b>Adding the
pipe symbol</b> ( | )
between the references will cause them to be <b>displayed in separate
columns</b>. Paste the following into the Entry to see the difference, <i>Gen
3.15,
18-20 | Psa 23, Isa 40.1-2 | Jn 3.18-20, Rom 4.5</i></font></li>
<li style="font-family: Verdana;"><font face="Verdana">If a title is
entered in
the Title Entry Box the page will be <b>saved in History</b> and can
be
redisplayed by using the navigation buttons or "Hist" Drop-Down List.</font></li>
<li><font face="Verdana"><span style="font-family: Verdana;"><span
style="font-family: Verdana;">If there are pages in the History when
Bible Analyzer closes, the <b>option will be given to save the pages
as a
new .pll Module</b> or append an existing one. This way the user can
easily
create their own Harmony / Parallel modules!.</span><br>
</span></font></li>
</ul>
<font face="Verdana"><br>
<span style="font-family: Verdana;">The Harmony Display / Parallel
Generator is a combination <b>Text Viewer, Harmony Viewer, Topical
Viewer,
Table
Generator, Module Creator, and more.</b> We hope you find it of value
in
your studies.</span></font>
<br>
<br>
'''


class ParGen(wx.MiniFrame):
    def __init__(self, parent):
        wx.MiniFrame.__init__(self, parent, style = wx.DEFAULT_FRAME_STYLE)
        self.mainApp = parent
        
        backImg = toolImg.getBackArrowBitmap()
        forwImg = toolImg.getForwArrowBitmap()
        
        panel = wx.Panel(self, -1)
        
        self.back = buttons.GenBitmapButton(panel, -1, backImg, size=(25, 22), style=wx.BORDER_NONE)
        self.forw = buttons.GenBitmapButton(panel, -1, forwImg, size=(25, 22), style=wx.BORDER_NONE)
        self.list = buttons.GenButton(panel, -1, 'Hist', size=(40, 18))
        self.list.SetUseFocusIndicator(False)
        
        modLabel = wx.StaticText(panel, -1, "Select Module")
        modLabel.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))

        titleLabel = wx.StaticText(panel, -1, "Select Harmony / Parallel Title")
        titleLabel.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        
        bibLabel = wx.StaticText(panel, -1, "Bible Text")
        bibLabel.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        
        self.modChoice = wx.Choice(panel, -1, choices=self.mainApp.harmonyTitleList, size=(170, -1))
        self.titleChoice = wx.Choice(panel, -1)
        self.bibChoice = wx.Choice(panel, -1, choices=self.mainApp.bAbbrList, size=(75, -1))
        
        self.display = baseClass.HtmlWindow(panel, -1, style = wx.BORDER_THEME)
        self.display.Bind(wx.EVT_CHAR, self.OnKeyDown)
        self.display.SetName('parGen')
        
        enterLabel = wx.StaticText(panel, -1, "Parallel Passage Generator")
        enterLabel2 = wx.StaticText(panel, -1, "Enter References: Use Pipe [ | ] for Separate Column(s)")
        enterLabel3 = wx.StaticText(panel, -1, "Optional: Enter Title to Save in History")

        self.refEnter = wx.TextCtrl(panel, -1, size=(450, -1), style=wx.TE_PROCESS_ENTER)
        self.titleEnter = wx.TextCtrl(panel, -1, size=(350, -1), style=wx.TE_PROCESS_ENTER)

        #helpButton = wx.Button(panel, -1, '?', size=(22, 23)) #, style=wx.BORDER_NONE)
        
        #helpButton.SetToolTip(wx.ToolTip('Open User Manual'))
        #helpButton.SetFont(wx.Font(11, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, "Verdana"))
        #helpButton.SetForegroundColour(wx.Colour(128, 0, 0))

        goButton = wx.Button(panel, -1, 'Display')#, size=(55, 22))
        closeButton = wx.Button(panel, -1, 'Hide')#, size=(45, 22))

        enterLabel.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        #enterLabel.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
        enterLabel2.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        enterLabel3.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        goButton.SetForegroundColour(wx.Colour(128, 0, 0))
        
        self.back.SetToolTipString('Cycle History Backward')
        self.forw.SetToolTipString('Cycle History Forward')
        self.list.SetToolTipString('List of Generated Pages')
        self.refEnter.SetToolTipString('Enter References to display. Those separated with a | will be in a separate column')
        self.titleEnter.SetToolTipString('When a title is entered, the references will be saved in history')

        self.SetTitle('Harmony/Parallel Generator')

        sizer = wx.BoxSizer(wx.VERTICAL)
        buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
        goSizer = wx.BoxSizer(wx.HORIZONTAL)

        sizer.Add((-1, 5))
        
        selSizer = wx.FlexGridSizer(2, 3, 3, 3)  # rows, cols, vgap, hgap
        selSizer.AddMany([(modLabel, 0, wx.ALIGN_CENTER_HORIZONTAL),
                            (titleLabel,   0, wx.ALIGN_CENTER_HORIZONTAL),
                            (bibLabel, 0, wx.ALIGN_CENTER_HORIZONTAL),
                            (self.modChoice, 0, wx.ALIGN_CENTER_HORIZONTAL),
                            (self.titleChoice, 0,  wx.EXPAND|wx.ALIGN_CENTER_HORIZONTAL),
                            (self.bibChoice, 0, wx.ALIGN_CENTER_HORIZONTAL),
                            ])
        selSizer.AddGrowableCol(1)
        sizer.Add(selSizer, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5)
        
        sizer.Add(self.display, 1, wx.EXPAND|wx.ALL, 5)
        sizer.Add((-1, 5))
        
        historySizer = wx.FlexGridSizer(1, 4, 1, 2)  # rows, cols, vgap, hgap
        historySizer.AddMany([(self.back, 0),
                            (self.list, 0),
                            (self.forw, 0),
                            (enterLabel, 0, wx.ALIGN_CENTER|wx.RIGHT, 60),
                            ])
        historySizer.AddGrowableCol(3)
        sizer.Add(historySizer, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5)
        
        sizer.Add(wx.StaticLine(panel), 0, wx.EXPAND | wx.LEFT|wx.RIGHT, 100)
        sizer.Add((-1, 10))
            
        sizer.Add(enterLabel2, 0, wx.ALIGN_CENTER, 50)
        sizer.Add((-1, 2))
        sizer.Add(self.refEnter, 0, wx.ALIGN_CENTER | wx.LEFT|wx.RIGHT, 5)
        sizer.Add((-1, 3))
        sizer.Add(enterLabel3, 0, wx.ALIGN_CENTER, 50)
        sizer.Add((-1, 1))

        goSizer.Add(self.titleEnter, 0, wx.TOP, 1)
        #sizer.Add((-1, 10))
        goSizer.Add(goButton, 0, wx.LEFT, 10)        
        sizer.Add(goSizer, 0, wx.ALIGN_CENTER, 5)
        
        sizer.Add((-1, 7))
        
        buttonSizer = wx.FlexGridSizer(1, 4, 1, 2)  # rows, cols, vgap, hgap
        buttonSizer.AddMany([((-1, -1)), (closeButton, 0, wx.LEFT, 25),
                            ((-1, -1)),
                            #(helpButton, 0, wx.ALIGN_RIGHT),
                            ])
        buttonSizer.AddGrowableCol(0)
        buttonSizer.AddGrowableCol(2)
        sizer.Add(buttonSizer, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, 5)

        
        sizer.Add((-1, 7))
                
        panel.SetSizer(sizer)
        
        #self.Layout()
        
        self.SetSize(self.mainApp.parWinSize)
        self.SetPosition(self.mainApp.parWinPos)
        #self.Center()

        self.back.Enable(False)
        self.forw.Enable(False)
        self.list.Enable(False)

        self.Bind(wx.EVT_BUTTON, self.OnBack, self.back)
        self.Bind(wx.EVT_BUTTON, self.OnForw, self.forw)
        self.Bind(wx.EVT_BUTTON, self.OnList, self.list)      
        #self.Bind(wx.EVT_BUTTON, self.OnHelp, helpButton)
        
        self.Bind(wx.EVT_BUTTON, self.OnGo, goButton)
        self.Bind(wx.EVT_BUTTON, self.OnClose, closeButton)
        self.Bind(wx.EVT_CHOICE, self.OnGo, self.titleChoice)
        self.Bind(wx.EVT_CHOICE, self.OnGo, self.bibChoice)
        self.Bind(wx.EVT_CHOICE, self.LoadHarmony, self.modChoice)
        self.Bind(wx.EVT_TEXT_ENTER, self.OnGo, self.refEnter)
        self.Bind(wx.EVT_TEXT_ENTER, self.OnGo, self.titleEnter)
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        
        self.display.Bind(wx.EVT_LEFT_UP, self.OnSetFocus)
        self.display.Bind(wx.html.EVT_HTML_CELL_CLICKED, self.mainApp.OnDictClicked)
        
        #self.MiniWindow.Bind(wx.html.EVT_HTML_CELL_CLICKED, self.OnClick)
        self.display.Bind(wx.html.EVT_HTML_CELL_HOVER, self.OnOver)
        
        self.bibChoice.SetSelection(self.mainApp.bibNB.GetSelection())
        
        self.display.SetPage(intro)
        #self.display.SetFocus()
        
        self.mainApp.currentParGen = intro
        
        self.parHistory = []
        self.parHistoryTitle = []
        self.parHistoryIdx = -1
        self.fromChoice = 0


    def OnSetFocus(self, event):
        self.display.SetFocus()
        event.Skip()
        
        
        

    def OnKeyDown(self, event):
        keycode = event.GetKeyCode()
        if keycode == 1:
            self.mainApp.OnSelectAll(None)
        elif keycode == 3:
            self.mainApp.OnCopy(None)
        event.Skip()



    
    def OnListSel(self, event):
        idx = event.GetId() -300
        self.refEnter.SetValue(self.parHistory[idx])
        title = self.parHistoryTitle[idx]
        self.OnGo(event, title)
        self.SetTitle('%s - %s' % ('Harmony/Parallel Generator', self.parHistoryTitle[idx]))

    def OnList(self, event):
        but = event.GetEventObject()
        self.histMenu = wx.Menu()
        
        idx = 300
        for item in self.parHistoryTitle:
            self.histMenu.Append(idx, item, "")
            idx += 1
        
        self.mainApp.Bind(wx.EVT_MENU_RANGE, self.OnListSel, id=300, id2=320)
        
        #rect = but.GetButtonRect()
        #pt = but.ClientToScreen(rect.GetBottomLeft())

        #self.histMenu.Popup(pt, self)
        
        self.PopupMenu(self.histMenu)
        self.histMenu.Destroy()    
        
    
    def OnBack(self, event):
        entries = len(self.parHistory)
        posIdx = self.parHistoryIdx - self.parHistoryIdx - self.parHistoryIdx
        if entries < 2:
            return
        if posIdx == entries:
            self.parHistoryIdx = 0
        self.parHistoryIdx -= 1
        self.SetTitle('%s - %s' % ('Harmony/Parallel Generator', self.parHistoryTitle[self.parHistoryIdx]))
        self.refEnter.SetValue(self.parHistory[self.parHistoryIdx])
        title = self.parHistoryTitle[self.parHistoryIdx]
        self.OnGo(event, title)
        self.histCurrent = self.parHistory[self.parHistoryIdx]

    def OnForw(self, event):
        entries = len(self.parHistory)
        if entries < 2:
            return
        if self.parHistoryIdx +1 == entries:
            self.parHistoryIdx = -1
        self.parHistoryIdx += 1
        self.SetTitle('%s - %s' % ('Harmony/Parallel Generator', self.parHistoryTitle[self.parHistoryIdx]))
        self.refEnter.SetValue(self.parHistory[self.parHistoryIdx])
        title = self.parHistoryTitle[self.parHistoryIdx]
        self.OnGo(event, title)
        self.histCurrent = self.parHistory[self.parHistoryIdx]


    def formatError(self, fn):
        dlg = wx.MessageDialog(self.mainApp, '%s is not in the proper format.\n\n\
The first line of the file must be the module title and not include a Tab.\n\
The following Topic lines must be have a Tab between the Topic Title and Reference(s):\n\n\
Module Title\n\
Topic Title[Tab]Reference(s)\n\
Topic Title[Tab]Reference(s)\n\
etc.' % fn,
'PLL Formatting Error', wx.OK | wx.ICON_INFORMATION    )
        dlg.ShowModal()
        dlg.Destroy()



    def LoadHarmony(self, event):
        self.titleChoice.Clear()
        
        fn = self.mainApp.harmonyFileList[self.modChoice.GetSelection()]
        
        #hList = file(fn).readlines()
        
        try:
            hList = codecs.open(fn, encoding='utf-8').readlines()
        except UnicodeError:
                dlg = wx.MessageDialog(self.mainApp, '%s\ndoes not have the proper encoding.\n\
The file must be plain ASCII or encoded in UTF-8' % fn,
'Module Encoding Error', wx.OK | wx.ICON_INFORMATION	)
                dlg.ShowModal()
                dlg.Destroy()
        
                return


        if '\t' in hList[0] or len(hList) == 1:
            self.formatError(fn)
            return

        
        self.titleList = []
        self.refList = []
                
        for ln in hList[1:]:
            idx = ln.find('\t')
            if not idx:
                self.formatError(fn)
                return
            title = ln[:idx].replace('<br>', '')
            self.titleList.append(title)
            self.titleChoice.Append(title)
            self.refList.append(ln[idx +1:])
                
        self.titleChoice.SetSelection(0)
        self.OnGo(event)
        
        
    
    def OnGo(self, event, title = ''):
        #print title
        wx.BeginBusyCursor()
        
        if self.modChoice.GetSelection() == -1 and not self.refEnter.GetValue():
            print 'return'
            wx.EndBusyCursor()
            return
        
        txts = []
        
        #print type(self.bibChoice)
        #print event.GetEventObject()
        
        rList = []

        if isinstance(event.GetEventObject(), wx.Menu) or isinstance(event.GetEventObject(), buttons.GenBitmapButton):
            print 'from history'
            rList = self.refEnter.GetValue().split('|')
            heading = 'Untitled Custom Module'
            
            self.modChoice.SetSelection(-1)
            self.titleChoice.SetSelection(-1)
            

        elif event.GetEventObject() == self.bibChoice:
            print 'bible changed'

            if self.fromChoice == 1:
                sel = self.titleChoice.GetSelection()
                rList = self.refList[sel].split('|')                   
                title = self.titleList[sel].replace('~', '<br>')
                heading = self.modChoice.GetStringSelection()
            elif self.refEnter.GetValue():
                rList = self.refEnter.GetValue().split('|')
                title = 'Custom Entry'
                heading = 'Untitled Custom Module'
                if self.titleEnter.GetValue():
                    title = self.titleEnter.GetValue()            
            

        elif self.refEnter.GetValue() and not isinstance(event.GetEventObject(), wx.Choice):
            print 'is entry, from button/enter'
            rList = self.refEnter.GetValue().split('|')
            title = 'Custom Entry'
            heading = 'Untitled Custom Module'
            if self.titleEnter.GetValue():
                title = self.titleEnter.GetValue()
                
            self.modChoice.SetSelection(-1)
            self.titleChoice.SetSelection(-1)
            self.fromChoice = 0
        
        
        elif event.GetEventObject() == self.modChoice or event.GetEventObject() == self.titleChoice:
            print 'from title choice'
            sel = self.titleChoice.GetSelection()
            rList = self.refList[sel].split('|')
                
            title = self.titleList[sel].replace('~', '<br>')
            heading = self.modChoice.GetStringSelection()
            self.fromChoice = 1
            
        else: 
            print 'in else'
            wx.EndBusyCursor()
            return
        
        if not rList:
			wx.EndBusyCursor()
			return
            
        fName = self.mainApp.bibFileList[self.bibChoice.GetSelection()]
        abbr = self.mainApp.bAbbrList[self.bibChoice.GetSelection()]
        
        con = sqlite.connect(fName)
        cur = con.cursor()
        
        origFixRefList = []
        
        for r in rList:
            tmp = []
            r = r.strip()
            #print r, 'rlist item'
            r = re.sub(r' *[,;.]$', '', r)
            #print r, 'rlist item'
            
            try:
                idxList, ref = fixRef.parseRng(r, fName)
                if not idxList:
                    wx.EndBusyCursor()
                    return
            except:
                print r, 'OnGo Exception'
                wx.EndBusyCursor()
                return
            
            
            origFixRefList.append(ref)
            #print r, idxList, ref
            
            for cyc, idx in enumerate(idxList):
                print idxList[cyc][0], idxList[cyc][1]
                if not type(idxList[cyc][0]) == int or not type(idxList[cyc][1]) == int:
                    print 'No index'
                    wx.MessageBox("A reference is not found in the selected Bible or is not properly formatted.\n\
Try another Bible or a reference within the range of the selected Bible.\n\
Also, references must be separated by a comma, semi-colon, or pipe symbol.", "Reference Not Found or Properly Formatted")
                    return
                
                selStr = "SELECT * FROM bible WHERE id BETWEEN %d AND %d" % (idxList[cyc][0], idxList[cyc][1])
                cur.execute(selStr)
                for row in cur:
                    row = utility.CheckEncryptByHeader(row)
                    line = '%s\t%s' % (row[1], row[2])
                    line = re.sub(r'\n', '<br>', line)
                    #line = re.sub(r'.*:(\d+)\t', r'<font color="#336666">&nbsp;&nbsp;&nbsp;&nbsp;\1, </font>', line)
                    line = re.sub(r'(\w\w\w) (\d+):(\d+)\t', r'<font color="#336666">&nbsp;&nbsp;&nbsp;&nbsp;\1&nbsp;\2:\3, </font>', line)

                    if fName in self.mainApp.strongsList:
                        if self.mainApp.showStrongs != True:
                            line = re.sub(r'\[\(?[GH](.+?)\)?\]', r'', line)
                        else:
                            line = re.sub(r'\[([GH].+?)\]', r'<sup><font color="#000066">\1</font></sup>', line)
                            line = re.sub(r'\[\(([GH].+?)\)\]', r'<sup><font color="#660000">\1</font></sup>', line)

                    if fName in self.mainApp.interList:
                        if self.mainApp.showText != True:
                            line = re.sub(r'{(.+?)}', r'', line)
                        else:    
                            line = re.sub(r'{(.+?)}', r'<sup><font color="#777777" size="+1"> \1</font></sup>', line)

                    line = re.sub(r'<fn>(\d+?)</fn>', r'<a href="bibleFN/%s/%s://\1"><font color="brown"><sup>\1</sup></font></a> ' % (abbr, row[1]), line)
                    
                    if abbr == 'NET':
                        if not 'NET' in self.mainApp.cAbbrList:
                            abbr = 'NET-'
                    
                    if 'embNote' in line:
                        line = re.sub(r'<s href="bible://[^>]+>(.+?)</s>', r'\1', line)
                        line = re.sub(r'(<a href="embNote.+?">)(.+?)</a>', r'\1<nu>\2</nu></a>', line)

                    line = re.sub(r'<fnc>(\d+?)</fnc>', r'<a href="commFN/%s/%s://\1"><font color="brown"><sup><nu>\1</nu></sup></font></a> ' % (abbr, row[1]), line)


                    tmp.append(line)
            txts.append(tmp)
        
        cols = []
        width = '%d' % (100 / len(txts))
        
        remainder = 0
        if int(width) * len(txts) != 100:
            #remainder = 100 - int(width) * len(txts)
            remainder = 1
        
        for n, set in enumerate(txts):
            if n %2 == 0:
                col = '<td valign="top" width="%s">%s</td>' % (str(width)+'%', ''.join(set))
            else:
                width = int(width) + remainder
                col = '<td valign="top" bgcolor="#EFEFEF" width="%s">%s</td>' % (str(width)+'%', ''.join(set))
            
            cols.append(col)
        
        fixRefList = ['<td valign="center"><center><b>%s</b></center></td>' % (r) for r in origFixRefList]
        
        t = '<center><table border="0" cellpadding="7" cellspacing="0"><tbody><tr>%s</tr><tr>%s</tr></tbody></table></center></font>' % (''.join(fixRefList), ''.join(cols))
        t = '<font face="%s"><center><font size="+1" color="navy"><b>%s</b></font><font size="+1" color="brown"><center>%s</center></font></font></font><hr><font face="%s">%s</font>' % (self.mainApp.font, heading, title, self.mainApp.font, t)
        self.display.SetPage(t)
        
        self.mainApp.currentParGen = t

        if event and self.titleEnter.GetValue(): 
            if isinstance(event.GetEventObject(), wx.Button) or isinstance(event.GetEventObject(), wx.TextCtrl):
                print event.GetEventObject()
                title = self.titleEnter.GetValue()
                self.parHistory.append('|'.join(origFixRefList))
                self.parHistoryTitle.append(title)
                self.parHistoryIdx = -1
            
                if len(self.parHistory) > 1:
                    self.back.Enable(True)
                    self.forw.Enable(True)
                    self.list.Enable(True)

        wx.EndBusyCursor()
    
    
    def OnOver(self, event):
        try:
            self.mainApp.OnPopup(event)
        except:
            pass

    
    def OnHelp(self, event):
        context = 411 
        self.mainApp.OnContextHelp(context)



    def OnClose(self, event):
        self.Hide()
        self.mainApp.Raise()
