import re, wx, utility, controlMini, advSearch, fixRef, zlib, dialogs
import flatNotebook as fnb
import sqlite3 as sqlite
import stemming.porter


class RelatedSearch(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        
        self.CheckWinState = utility.CheckWinState()
        
        self.parent = parent
        self.mainApp = wx.GetApp().GetTopWindow()
        self.advSearch = advSearch.primSearch(self)
        
        self.verseBoxLabel1 = wx.StaticText(self, -1, "Verse Reference")
        self.verseBoxLabel2 = wx.StaticText(self, -1, "Words")
        self.verseBox = wx.TextCtrl(self, -1, "", style=wx.TE_PROCESS_ENTER)
        self.goButton = wx.Button(self, -1, "Go", size=(35, 23))
        self.span = wx.SpinCtrl(self, -1, "", size=(45, 23))
        
        self.wordListLabel = wx.StaticText(self, -1, "Search Phrases")
        self.wordList = wx.CheckListBox(self, -1, wx.DefaultPosition, wx.DefaultSize, [])
        self.doSingle = wx.CheckBox(self, -1, "Show Verses (Single Word Only)")
        self.doRoot = wx.CheckBox(self, -1, "Use Root Words (Stemming)")
        self.searchButton = wx.Button(self, -1, "Search")
        
        self.span.SetRange(1,5)
        self.span.SetValue(3)
        self.searchButton.SetForegroundColour(wx.Colour(142, 35, 35))
        #self.searchButton.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
        self.verseBoxLabel1.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        self.verseBoxLabel2.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        self.wordListLabel.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))

        
        self.libSizer = wx.BoxSizer(wx.VERTICAL)
        refSizer = wx.BoxSizer(wx.HORIZONTAL)
        labelSizer = wx.BoxSizer(wx.HORIZONTAL)
        
        title = controlMini.FMTitlePanel(self, "Related Verse Search")
        self.libSizer.Add(title, 0, wx.EXPAND|wx.ALL, 5)
        
        self.libSizer.Add((-1, 3))
        labelSizer.Add(self.verseBoxLabel1, 1, wx.LEFT|wx.EXPAND, 3)
        labelSizer.Add(self.verseBoxLabel2, 0, wx.LEFT, 0)
        self.libSizer.Add(labelSizer, 0, wx.LEFT|wx.RIGHT|wx.EXPAND, 8)
        
        refSizer.Add(self.verseBox, 1, wx.TOP, 0)
        refSizer.Add(self.goButton, 0, wx.LEFT, 5)
        refSizer.Add(self.span, 0, wx.LEFT, 29)
        
        self.libSizer.Add(refSizer, 0, wx.LEFT|wx.RIGHT|wx.EXPAND, 8)
        self.libSizer.Add(self.wordListLabel, 0, wx.ALIGN_CENTER|wx.TOP, 7)
        self.libSizer.Add(self.wordList, 1, wx.EXPAND|wx.RIGHT|wx.LEFT, 8)
        self.libSizer.Add((-1, 5))
        self.libSizer.Add(self.doRoot, 0, wx.EXPAND|wx.RIGHT|wx.LEFT, 8)
        self.libSizer.Add((-1, 5))
        self.libSizer.Add(self.doSingle, 0, wx.EXPAND|wx.RIGHT|wx.LEFT, 8)        
        self.libSizer.Add(self.searchButton, 0, wx.ALIGN_CENTER|wx.TOP, 8)
        self.SetSizer(self.libSizer)
        self.Layout()

        self.Bind(wx.EVT_TEXT_ENTER, self.OnRef, self.verseBox)
        self.Bind(wx.EVT_BUTTON, self.OnSearch, self.searchButton)
        self.Bind(wx.EVT_BUTTON, self.OnRef, self.goButton)
        self.Bind(wx.EVT_SPINCTRL, self.OnRef, self.span)
        
        #self.OnRef(self.mainApp.currentRef)
        self.doSingle.Enable(False)


    def OnRef(self, event):
        
        tab = self.mainApp.bibNB.GetSelection()
        if tab == len(self.mainApp.bibFileList):
            dialogs.noRelatedParallel(self.mainApp)
            return
        
        self.parent.Freeze()
        
        fName = self.mainApp.bibFileList[tab]
        
        if self.span.GetValue() == 1:
            self.doSingle.Enable(True)
        else:
            self.doSingle.Enable(False)
        
        origRef = self.verseBox.GetValue()
        try:
            ref = fixRef.parseRef(origRef, fName, 1)
            
            if not ':' in ref and ref[-1].isdigit():
                ref = ref + ':1'
            if not ' ' in ref:
                ref = ref + ' 1:'
            if '-' in ref:
                idx = ref.find('-')
                ref = ref[:idx]
                
            #self.verseBox.Append(ref)
            self.verseBox.SetValue(ref)
            self.GetWords(ref)
            self.parent.Thaw()
        
        except:
            dlg = wx.MessageDialog(self, '"%s" is not a valid reference.\n\n\
You must enter a single reference (Gen 3:15, Joh 3:16, etc.).\n\
Partial references (Gen, re, etc.) will go to the first chapter.\n\
Most abbreviations are accepted (Ge, mat, mt, ro 4:5, Rev., etc.)\n\
A period or space may be used in place of a colon (Ro 4.5, ro 4 5).\n\n\
See User Manual or Quick Help for more tips.' % origRef,
                                     'Reference Error', wx.OK | wx.ICON_INFORMATION	)
            dlg.ShowModal()
            dlg.Destroy()
            self.parent.Thaw()
            return


    
    def ImportWordList(self, wList):
        self.verseBox.SetValue('')
        self.wordList.Clear()
        self.span.SetValue(0)
        
        for wd in wList:
            if not wd.isdigit():
                self.wordList.Append(wd.lower())
        



    def GetWords(self, ref):
        if ref[-1] == ':':
            ref = ref + '1'
        elif len(ref) == 3:
            ref = ref + ' 1:1'
        
        self.verseBox.SetValue(ref)
        self.wordList.Clear()
        span = self.span.GetValue()
        
        tab = self.mainApp.bibNB.GetSelection()
        if tab == len(self.mainApp.bibFileList):
            dialogs.noRelatedParallel(self.mainApp)
            return
        
        fn = self.mainApp.bibFileList[tab]
        con = sqlite.connect(fn)
        cur = con.cursor() 
        
        encrypt = 0
        cur.execute("SELECT name FROM sqlite_master WHERE name='option'")
        if cur.fetchone():
            cur.execute('SELECT * FROM option')
            if len(cur.fetchone()) > 2:
                encrypt = 1

        try:
            cur.execute('SELECT * FROM bible WHERE ref LIKE ?', (ref,))
        except:
            print 'Wrong encoding', fn
            return

        row = cur.fetchone()
        row = utility.CheckEncryptByHeader(row)
        
        #if row and encrypt == 1:
        #    print 'encrypt'
        #    vsTxt = zlib.decompress('x\x01' + str(row[2]))
        #    row = (row[0], row[1], unicode(vsTxt, 'utf-8'))
            
        if row:
            txt = re.sub(r'<a href="bible[^>]+>.+?</a>', r'', row[2])
            txt = re.sub(r'<a href="embNote[^>]+>.+?</a>', r'', txt)
            
            words = re.sub(r'[,.?;:!-()]|<.+?>', '', txt).split()
            #print words
            idx = 0
            for n in range(len(words) - span +1):
                tmp = words[idx:span]
                self.wordList.Append(' '.join(tmp).lower())
                span += 1
                idx += 1
    
    
    def OnSearch(self, event):
        if self.doSingle.IsChecked() and self.doSingle.IsEnabled():    
            self.OnSingleSearch()
        else:
            self.OnNormSearch()
    
    
    
    def OnSingleSearch(self):
        tmp = []
        full = 1
        cnt = self.wordList.GetCount()
        for num in range(cnt):
            if self.wordList.IsChecked(num):
                tmp.append(self.wordList.GetString(num))
        
        if self.doRoot.IsChecked():
            full = 0
            wdLst = [stemming.porter.stem(w) for w in tmp]
            tmp = wdLst[:]
        
        words = ' '.join(tmp)
        self.DisplaySingleSearch(words, full)

    

    def OnRoot(self, phrase):

        phTmp = phrase.split()

        phLst = [stemming.porter.stem(w) for w in phTmp]

        return ' '.join(phLst)

        

    
    def OnNormSearch(self):

        self.tmp = []
        self.total = 0
        cnt = self.wordList.GetCount()
        
        wx.BeginBusyCursor()

        if self.span.GetValue() == 1:

            tmp = []

            for num in range(cnt):

                if self.wordList.IsChecked(num):

                    wd = self.wordList.GetString(num)

                    tmp.append(wd)

                    

            self.processPhrase(' '.join(tmp), 1)

        

        else:
            for num in range(cnt):
                if self.wordList.IsChecked(num):
                    phrase = self.wordList.GetString(num)
                    

                    self.processPhrase(phrase, 0)
        

        self.formatDisplay(self.tmp, self.total)



    

    def processPhrase(self, phrase, opt):

        origPhrase = phrase

        full = 1

        
        if self.doRoot.IsChecked():

            phrase = self.OnRoot(phrase)

            full = 0
        
        try:
            grp = self.DisplaySearch(phrase, full)
        except:
            print 'Related search error', phrase
            return
        
        self.total += len(grp)
        
        ref = [r.replace(' ','_') for r in grp]
        ul = ['<u>%s</u>' % r for r in ref]
        
        tab = self.mainApp.bibNB.GetSelection()
        fName = self.mainApp.bibFileList[tab]
        abbr = self.mainApp.bAbbrList[tab]
        fface = self.mainApp.font
        
        link = '<a href="search://[%s; %s; %s; %s; %s; %s; %s; %s; %s; %s; %s; %s; %s; %s; %s; %s]">Show Verses</a>'\
        % (phrase, fName, abbr, 1, opt, 0, fface, 1, 0, [[1, 31102]], None, 'Gen-Rev', 0, 0, 0, 'Gen-Rev')
        
        self.tmp.append('<br><br><font color="#660000"><b>%s</b></font><font size="-1"> - %s</font><br>%s' % (origPhrase, link, ', '.join(ul)))
        





    def formatDisplay(self, tmp, total):
        self.mainApp.searchQList.append('')
        tabLabel = 'Related'
        title = 'Related Word/Phrase Search'  
        
        rootTxt = ''
        if self.doRoot.IsChecked():
            rootTxt = ', (Use Roots)'
        
        result = '''<body bgcolor="%s" link="#336666"><font face="%s"><b>%s</b>%s<br><font color="#660000"><b>%s</b></font> Hits among <font color="#660000"><b>%s</b></font> Phrases<hr>%s</font>'''\
        % (self.mainApp.resColor, self.mainApp.font, title, rootTxt, total, len(tmp), ''.join(tmp))
        
        result = re.sub(r'<u>(\w\w\w)_([^<]+)</u>', r'<a href="bible://\1 \2">\1&nbsp;\2</a>', result)

        

        if self.mainApp.fontSize  > 0:

            result = '<font size="+%s">%s</font>' % (self.mainApp.fontSize, result)        

        
        self.mainApp.Freeze()

        self.mainApp.AddResult(tabLabel, title, result)
        self.mainApp.Thaw()
        
        self.CheckWinState.CheckState()

        wx.EndBusyCursor()



    def DisplaySearch(self, entry, full, case =0, opt=0, redSearch=0): 

        if self.span.GetValue() == 1:

            opt = 1

        tab = self.mainApp.bibNB.GetSelection()
        abbr = self.mainApp.bAbbrList[tab]
        
        self.fixedRef = 'Gen - Rev'
        exBox = None
        exclude = None
        self.entry = entry
        self.sWord = self.entry
               
        result = self.advSearch.StdSearch\
        (entry, self.mainApp.bibFileList[tab], abbr, full, opt, case, self.mainApp.font, self.mainApp.showRed, redSearch, [[1, 31102]], exBox, self.fixedRef, 1, 0, 0, self.fixedRef, 1)
        
        return result
        

    def DisplaySingleSearch(self, entry, full, case =0, opt=1, redSearch=0):   

        tab = self.mainApp.bibNB.GetSelection()
        abbr = self.mainApp.bAbbrList[tab]
        
        self.fixedRef = 'Gen - Rev'
        exBox = None
        exclude = None
        self.entry = entry
        self.sWord = self.entry
        
        wx.BeginBusyCursor()
        try:
            result, verseList, title, count, findStr, totalHits, exclude, rngRefList = self.advSearch.StdSearch\
            (entry, self.mainApp.bibFileList[tab], abbr, full, opt, case, self.mainApp.font, self.mainApp.showRed, redSearch, [[1, 31102]], exBox, self.fixedRef, 0, 0, 0, self.fixedRef, 0)
        except:
            print 'Related Search Exception'
            wx.EndBusyCursor()
            return
            
        self.mainApp.resNB.resPanel.hitList.Clear()
        for vs in verseList:
            self.mainApp.resNB.resPanel.hitList.Append(vs)
        
        # Add data to search query list
        temp = [entry, self.mainApp.bibFileList[tab], self.mainApp.bAbbrList[tab], full, opt, case, self.mainApp.showRed, redSearch, count, findStr, totalHits, exclude, self.fixedRef, rngRefList, [[1, 31102]]]
        self.mainApp.searchQList.append(temp)
        
        self.mainApp.Freeze()
        
        tabLabel = entry
        
        if self.mainApp.fontSize  > 0:
            result = '<font size="+%s">%s</font>' % (self.mainApp.fontSize, result)        
        
        self.mainApp.AddResult(tabLabel, title, result)
        self.mainApp.Showing = result
        
        self.CheckWinState.CheckState()

        self.mainApp.Thaw()
        wx.EndBusyCursor()


