from __future__ import with_statement

import sys, wx, os, re, wx.html, cPickle, random, time
import utility, labelBook, optionsDlg
from artManager import ArtManager, DCSaver
import wx.richtext as rt
from ObjectListView import GroupListView, ColumnDefn, Filter
import genButton as buttons
import sqlite3 as sqlite
from configobj import ConfigObj

from resources import *


def ErrorApp():
	class Error(wx.App):
		def OnInit(self):        
			f = wx.Frame(None, -1)
			dlg = wx.MessageDialog(f, 
						'''Bible Analyzer cannot find the \Modules folder or the "av1769.bib" file.\n\nIt expected to find it here:\n%s%s\n\n\
Try reinstalling Bible Analyzer, or if using the Portable edition, ensure the \\Modules folder is in the same folder as the bibleanalyzer4.exe file.'''\
% (allUsersDir, '\Modules'), 'Modules Folder Not Found', wx.OK | wx.ICON_INFORMATION    )
			dlg.ShowModal()
			dlg.Destroy()
			
			f.Destroy()
			return True

	app = Error(None)
	app.MainLoop()
	sys.exit()


portable = False
modDirList = []

appDir = os.getcwd()

if "__WXGTK__" in wx.PlatformInfo:
	if portable:
		allUsersDir = appDir
		homeUserDir = appDir
		myDocsDir = appDir + r'/modules'
		userDir = appDir
		modDirList.append(allUsersDir + r'/modules/')
	
	else:
		home = os.environ['HOME']
		homeUserDir = home + ur"/Documents/BibleAnalyzer"
		userDir = home + ur"/.config/bibleanalyzer"
		
		if not os.path.isdir(homeUserDir):
			os.mkdir(ur'%s' % homeUserDir)
			print 'no home user dir'

		if not os.path.isdir(userDir):
			os.mkdir(ur'%s' % userDir)
			print 'no user dir'

		allUsersDir = ur'/opt/bibleanalyzer'

		myDocsDir = homeUserDir # + r'/Modules'
		
		modDirList.append(allUsersDir + r'/modules/')
		modDirList.append(myDocsDir)

		qpath = allUsersDir
		path = allUsersDir + r'/modules/'

    
hit = 0
for root, dirs, files in os.walk(allUsersDir + r'/modules'):
    for fn in files:
        if fn.endswith('av1769.bib'):
            hit = 1
            break
if hit:
    #modDirList.append(allUsersDir)
    #modDirList.append(myDocsDir) # Disable for local folder
    qpath = allUsersDir
else:
    print 'no av1769.bib'
    #ErrorApp()

print appDir, 'App dir'
print homeUserDir, 'home user folder'
print allUsersDir, 'all users'
print myDocsDir, 'myDocs'
print userDir, 'user'
#print modDirList


config = ConfigObj(ur'%s/config4.ini' % userDir, encoding='UTF8', unrepr=True)


## Get random qoute
#print allUsersDir +r'/quotes.txt'

quotes = file(ur'%s/quotes.txt' % allUsersDir).readlines()

num = random.randint(1, len(quotes))
author, quote = quotes[num -1].split('\t')   


## S-Pad Quote and Image
def editorQuote():
	mainApp = wx.GetApp().GetTopWindow()   
		
	editor = mainApp.editor
	
	date = time.strftime("%A, %B %d, %Y")
	curTime = time.strftime("%I:%M %p")
	
	editor.BeginAlignment(rt.TEXT_ALIGNMENT_CENTRE)
	editor.BeginBold() 
	editor.BeginFontSize(11)
	editor.BeginTextColour((150, 0, 0))

	editor.WriteText("%s" % date)
	#editor.Newline()
	#editor.WriteText("%s" % curTime)
	editor.EndFontSize()
	editor.EndTextColour()
	editor.EndBold()
	editor.Newline()
	
	editor.BeginTextColour((0, 0, 100))  
	editor.BeginItalic()
	editor.WriteText(quote)
	editor.EndItalic()   
	editor.EndTextColour()
	
	editor.WriteText('(%s)' % author)
	editor.Newline()
	
	editor.EndAlignment()



class InitModules():
	def __init__(self):
		self.mainApp = wx.GetApp().GetTopWindow()

		#Holders for module info
		self.bibList, self.dctList, self.cmtList = [], [], []
		self.bibData, self.dctData, self.cmtData = [], [], []
		
		self.dctStrongList = []

		self.bkList = []
		self.bkData = []

		self.imdList = []
		self.userImgList = []

		self.devList = []
		self.devData = []

		self.audioList = []
		self.audioTitleList = []

		self.harmonyList = []
		self.harmonyTitleList = []
		self.harmonyFileList = []
		
		self.dctIdxPending = []
		self.cmtIdxPending = []
		
		self.needRefIdx = []


	## Get Available Modules and Remove Unchecked
	def GetResources(self):
		#print userDir, type(userDir), 'USERDIR'
		if not os.path.isfile(ur'%s/notes.usr' % userDir):
			con = sqlite.connect(ur'%s/notes.usr' % userDir)
			cur = con.cursor()
			cur.execute("create table commentary(ref, comment)")
			con.commit()

		for dir in modDirList:
			cnt = 0
			for root, dirs, files in os.walk(dir):
				for fn in files:
					if fn.endswith('.bib'):
						bibPath = ur'%s/%s' % (root, fn)
						self.bibList.append(bibPath.replace('\\', '/'))
						cnt += 1
					elif fn.endswith('.dct'):
						dctPath = ur'%s/%s' % (root, fn)
						self.dctList.append(dctPath.replace('\\', '/'))
					elif fn.endswith('.cmt'):
						cmtPath = ur'%s/%s' % (root, fn)
						self.cmtList.append(cmtPath.replace('\\', '/'))
					elif fn.endswith('.bk'):
						bkPath = ur'%s/%s' % (root, fn)
						self.bkList.append(bkPath.replace('\\', '/'))
					elif fn.endswith('.imd'):
						imdPath = ur'%s/%s' % (root, fn)
						self.imdList.append(imdPath.replace('\\', '/'))
					elif fn.endswith('.dev'):
						devPath = ur'%s/%s' % (root, fn)
						self.devList.append(devPath.replace('\\', '/'))
					elif fn.endswith('mid') or fn.endswith('mp3') or fn.endswith('wav'):
						audPath = ur'%s/%s' % (root, fn)
						self.audioList.append(audPath.replace('\\', '/'))
					elif fn.endswith('.pll'):
						pllPath = ur'%s/%s' % (root, fn)
						self.harmonyList.append(pllPath.replace('\\', '/'))
					elif root.endswith('Modules') or root.endswith('Image'):
						if fn.endswith('.gif') or fn.endswith('.jpg') or fn.endswith('.png'):
							usrPath = ur'%s/%s' % (root, fn)
							self.userImgList.append(usrPath.replace('\\', '/'))



		for bibPath in self.bibList:
			temp = []
			try:
				con = sqlite.connect(bibPath)
				con.text_factory = str
				cur = con.cursor()
				cur.execute('SELECT * FROM title')
				row = cur.fetchone()
				temp.append(unicode(row[0].strip(' \n'), 'utf-8'))
				temp.append(unicode(row[1].strip(' \n'), 'utf-8'))
				temp.append(bibPath)
			
				try:
					cur.execute('SELECT * FROM option')
					row = cur.fetchone()
					temp.append(row[0]) #Strongs
					temp.append(row[1]) #Interlinear
				except:
					temp.append(0)
					temp.append(0)
			
			except:
				print bibPath, 'remove'
				self.bibList.remove(bibPath)
				#continue

				
			self.bibData.append(temp)

		#print bibData
		
		#Get Dict info from first line of text
		for dct in self.dctList:
			#print dct
			temp = []
			try:
				con = sqlite.connect(dct)
				con.text_factory = str
				cur = con.cursor()                 
										
				cur.execute('SELECT * FROM title')
				row = cur.fetchone()
				
				temp.append(unicode(row[0].strip(' \n'), 'utf-8'))
				temp.append(unicode(row[1].strip(' \n'), 'utf-8'))
				temp.append(dct)
				
				if row[0] == 'StrongEx'or row[0] == "Strong's" or row[0] == 'BDB/Thayer' or row[0] == 'KJC':
					self.dctStrongList.append(row[1])
				
				elif len(row) > 5:
					if row[5]:
						self.dctStrongList.append(row[1])
				
				self.dctData.append(temp)
				
				cur.execute("SELECT count(*) FROM sqlite_master WHERE type='table' AND name='refindex'")
				row = cur.fetchone()
				if not row[0]:
					self.needRefIdx.append(dct)
						
			except:
				print dct, 'remove'
				self.dctList.remove(dct)
				#continue


		#Get Cmt info from first line of text
		for cmt in self.cmtList:
			#print cmt
			temp = []
			try:
				con = sqlite.connect(cmt)
				con.text_factory = str
				cur = con.cursor()
				
				cur.execute('SELECT * FROM title')
				row = cur.fetchone()
				
				temp.append(unicode(row[0].strip(' \n'), 'utf-8'))
				temp.append(unicode(row[1].strip(' \n'), 'utf-8'))
				temp.append(cmt)
				self.cmtData.append(temp)
				
				cur.execute("SELECT count(*) FROM sqlite_master WHERE type='table' AND name='refindex'")
				row = cur.fetchone()
				if not row[0] and not cmt.endswith('tske.cmt'):
					self.needRefIdx.append(cmt)
				
			except:
				print cmt, 'remove'
				self.cmtList.remove(cmt)
				#continue


	## Get Books
	def GetBooks(self):   
		for bk in self.bkList[:]:
			#print bk
			temp = []
			chaps = []
			mapData = []
			
			try:
				con = sqlite.connect(bk)
				con.text_factory = str
				cur = con.cursor()                 
										
				cur.execute('SELECT * FROM info')
				titleRow = cur.fetchone()
				
				cur.execute('SELECT chapter FROM book')
				for c in cur:
					#print c
					c = unicode(c[0], 'utf-8')
					c = c.replace('"', '[q]')
					chaps.append(c)

				temp.append(unicode(titleRow[0].strip(' \n'), 'utf-8').replace('"', '[q]'))
				temp.append(unicode(titleRow[1].strip(' \n'), 'utf-8').replace('"', '[q]'))
							
				temp.append(bk.replace('"', '[q]'))
				temp.append(chaps)
				
				# For book group
				if len(titleRow) > 2:
					temp.append(titleRow[2].replace('"', '[q]'))
				else:
					temp.append(None)
				
				cur.execute('SELECT * FROM book')
				contentsRow = cur.fetchone()
				
				# Image map data
				if len(contentsRow) > 3:
					cur.execute('SELECT mapData FROM book')
					for map in cur:
						mapData.append(eval(map[0]))
				
				temp.append(mapData)

				self.bkData.append(temp)
				
				cur.execute("SELECT count(*) FROM sqlite_master WHERE type='table' AND name='refindex'")
				row = cur.fetchone()
				if not row[0]:
					self.needRefIdx.append(bk)
				
			except:
				self.bkList.remove(bk)
				print bk, 'Removed'
				#continue


	#Get Devotion info from first line of text
	def GetDevotions(self):
		for dev in self.devList:
			temp = []
			try:
				con = sqlite.connect(dev)
				cur = con.cursor()
				cur.execute('SELECT * FROM title')
				row = cur.fetchone()
				
				temp.append(row[0])
				temp.append(row[1])
				temp.append(dev)
				self.devData.append(temp)
			except:
				self.devList.remove(dev)
				#continue

	def GetHarmonies(self):
		for har in self.harmonyList:
			try:
				hList = file(har).readlines()
				hList = [h[:-2] for h in hList]
				self.harmonyTitleList.append(hList[0])
				self.harmonyFileList.append(har)
			except:
				self.harmonyList.remove(har)
				#continue


	def GetAudio(self):
		for path in self.audioList:
			fn = os.path.basename(path)
			self.audioTitleList.append(fn[:-4])



class SMData(object):
	def __init__(self, tab, desc, type):
		self.tab = tab
		self.desc = desc
		self.type = type


class SessionManager(wx.Frame):
	def __init__(self, parent):
		wx.Frame.__init__(self, parent, -1, " Session/Module Manager", style = wx.CAPTION|wx.CLOSE_BOX|wx.SYSTEM_MENU|wx.FRAME_FLOAT_ON_PARENT|wx.CLIP_CHILDREN)
		
		self.mainApp = parent
		self.LBook = None
		
		self.tabTitles = self.mainApp.sessTabTitles
				
		self.saveButton = wx.Button(self, -1, "Load / Save")
		self.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE))
		self.saveButton.SetForegroundColour(wx.Colour(128, 0, 0))
		
		#self.HelpButton = wx.Button(self, -1, '?', size=(22, 22)) #, style=wx.BORDER_NONE)
		#self.HelpButton.SetToolTip(wx.ToolTip('Help for the Selected Tab'))
		#self.HelpButton.SetFont(wx.Font(11, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, "Verdana"))
		#self.HelpButton.SetForegroundColour(wx.Colour(128, 0, 0))
		
		self.sessList = [[[], [], [], []], [[], [], [], []], [[], [], [], []], [[], [], [], []], [[], [], [], []], [[], [], [], []], [[], [], [], []]]
		
		if os.path.isfile(self.mainApp.userDir + r'/sessions.dat'):
			pklFile = open(self.mainApp.userDir + r'/sessions.dat', 'rb')
			self.sessList = cPickle.load(pklFile)
			if len(self.sessList) < 7:
				tmp = [[], [], [], []]
				self.sessList.append(tmp)
			pklFile.close()

		self.CreateDialog()
		self.SetSize((580, 600))
		self.Center()
		self.Layout()
		
		
		self.Bind(wx.EVT_CLOSE, self.OnClose)
		self.Bind(wx.EVT_BUTTON, self.OnLoad, self.saveButton)
		#self.Bind(wx.EVT_BUTTON, self.OnHelp, self.HelpButton)
		
		#acclTable = self.mainApp.GetAcceleratorTable()
		#self.SetAcceleratorTable(acclTable)


	def CreateDialog(self):
		mainSizer = wx.BoxSizer(wx.VERTICAL)
		bodySizer = wx.BoxSizer(wx.HORIZONTAL)
		buttonSizer = wx.BoxSizer(wx.VERTICAL)
		topSizer = wx.BoxSizer(wx.HORIZONTAL)
		olvSizer = wx.BoxSizer(wx.VERTICAL)
		self.SetSizer(mainSizer)
		
		self.olv = GroupListView(self, -1, style=wx.LC_REPORT|wx.SUNKEN_BORDER)
		self.olv.SetFilter(Filter.TextSearch(self.olv, self.olv.columns[0:2]))
		self.olv.groupBackgroundColour = wx.Colour(200, 225, 255)
		self.olv.evenRowsBackColor = wx.Colour(255, 255, 255) 
		self.olv.oddRowsBackColor = wx.Colour(255, 235, 235) 
		
		info = '''Up to six sessions or groups can be defined, named, and saved. \
The first button always includes all modules.\n\n\
To use, select a button above, check the resources you want from the list, and then click the Load/Save button below.'''
		infoLabel = wx.StaticText(self, -1, info)
		infoLabel.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
		infoLabel.Wrap(135)
		
		headLabel = wx.StaticText(self, -1, ' Custom Module Groups')
		headLabel.SetForegroundColour(wx.Colour(128, 0, 0))
		headLabel.SetFont(wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0))
		
		self.searchCtrl = wx.SearchCtrl(self, -1, size = (160, -1))
		self.searchCtrl.Bind(wx.EVT_TEXT, self.OnTextSearchCtrl)
		self.searchCtrl.Bind(wx.EVT_SEARCHCTRL_CANCEL_BTN, self.OnCancelSearchCtrl)
		
		buttCheck = wx.Button(self, -1, " Check All  ")
		buttCheck.useFocusInd = False
		self.Bind(wx.EVT_BUTTON, self.OnCheck, buttCheck)

		buttUnCheck = wx.Button(self, -1, "Check None")
		buttUnCheck.useFocusInd = False
		self.Bind(wx.EVT_BUTTON, self.OnUnCheck, buttUnCheck)

		buttRename = wx.Button(self, -1, "Rename")
		buttRename.useFocusInd = False
		self.Bind(wx.EVT_BUTTON, self.OnRename, buttRename)

		b1 = buttons.GenToggleButton(self, -1, self.mainApp.sessTabTitles[0])
		b2 = buttons.GenToggleButton(self, -1, self.mainApp.sessTabTitles[1])
		b3 = buttons.GenToggleButton(self, -1, self.mainApp.sessTabTitles[2])
		b4 = buttons.GenToggleButton(self, -1, self.mainApp.sessTabTitles[3])
		b5 = buttons.GenToggleButton(self, -1, self.mainApp.sessTabTitles[4])
		b6 = buttons.GenToggleButton(self, -1, self.mainApp.sessTabTitles[5])
		b7 = buttons.GenToggleButton(self, -1, self.mainApp.sessTabTitles[6])
		
		b1.SetToggle(True)
		
		self.buttonList = [b1, b2, b3, b4, b5, b6, b7]
		for btn in self.buttonList:
			btn.useFocusInd = False
			btn.SetBezelWidth(1)
			#btn.SetFont(wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.NORMAL, 0))
			btn.SetBackgroundColour(wx.Colour(200, 225, 255))
			btn.SetForegroundColour(wx.Colour(80, 0, 0))
			self.Bind(wx.EVT_BUTTON, self.OnToggleButtons, btn)

		buttonSizer.Add(b1, 0, wx.EXPAND|wx.TOP, 0)
		buttonSizer.Add(b2, 0, wx.EXPAND|wx.TOP, 3)
		buttonSizer.Add(b3, 0, wx.EXPAND|wx.TOP, 3)
		buttonSizer.Add(b4, 0, wx.EXPAND|wx.TOP, 3)
		buttonSizer.Add(b5, 0, wx.EXPAND|wx.TOP, 3)
		buttonSizer.Add(b6, 0, wx.EXPAND|wx.TOP, 3)
		buttonSizer.Add(b7, 0, wx.EXPAND|wx.TOP, 3)
		buttonSizer.Add(buttRename, 0, wx.ALIGN_CENTER|wx.TOP, 10)
		
		buttonSizer.Add(wx.StaticLine(self), 0, wx.EXPAND | wx.TOP, 10)
		buttonSizer.Add((-1, 6), 0, 0, 0)
		buttonSizer.Add(infoLabel, 0, wx.ALIGN_CENTER|wx.ALL, 4)
		#buttonSizer.Add(wx.StaticLine(self), 0, wx.EXPAND | wx.TOP, 5)
		#buttonSizer.Add((130, -1), 1, 0, 0)
		#buttonSizer.Add(searchLabel, 0, wx.ALIGN_CENTER|wx.TOP, 10)
		
		topSizer.Add(headLabel, 0, wx.ALIGN_CENTER|wx.TOP, 1)
		topSizer.Add((-1, -1), 1, 0, 0)
		topSizer.Add(self.searchCtrl, 0, wx.ALIGN_CENTER|wx.TOP, 1)
		topSizer.Add(buttCheck, 0, wx.ALIGN_CENTER|wx.LEFT, 20)
		topSizer.Add(buttUnCheck, 0, wx.ALIGN_CENTER|wx.LEFT, 5)
		mainSizer.Add(topSizer, 0, wx.EXPAND|wx.ALL, 5)
		olvSizer.Add(self.olv, 1, wx.EXPAND|wx.TOP, 0)
		
		bodySizer.Add(buttonSizer, 0, wx.EXPAND|wx.ALL, 3)
		bodySizer.Add(olvSizer, 1, wx.EXPAND|wx.ALL, 2)
		mainSizer.Add(bodySizer, 1, wx.EXPAND)

		self.data = self.GetData()
		self.InitOLV()
		
		#self.SetChecked(0)
		
		footSizer = wx.BoxSizer(wx.HORIZONTAL)

		footSizer.Add((-1, -1), 1, 0, 0)
		footSizer.Add(self.saveButton, 0, wx.TOP | wx.BOTTOM, 2)
		footSizer.Add((-1, -1), 1, 0, 0)
		#footSizer.Add(self.HelpButton, 0, wx.ALIGN_RIGHT|wx.RIGHT|wx.TOP, 3)

		mainSizer.Add(wx.StaticLine(self), 0, wx.EXPAND | wx.TOP, 3)
		mainSizer.Add(footSizer, 0, wx.ALL|wx.EXPAND, 2)

	
	
	def OnCheck(self, event):
		if self.mainApp.sessTab == 0:
			return
		objects = self.olv.GetObjects()
		for obj in objects:
			self.olv.SetCheckState(obj, True)
		self.olv.RefreshObjects(objects)

	def OnUnCheck(self, event):
		if self.mainApp.sessTab == 0:
			return
		objects = self.olv.GetObjects()
		for obj in objects:
			if obj.desc != 'Authorized King James Version':
				self.olv.SetCheckState(obj, False)
		self.olv.RefreshObjects(objects)

	def GetData(self):
		tmp = []
		for grp in self.mainApp.initMods.bibData:
			tmp.append(SMData(grp[0], grp[1], 'Bible'))
		for grp in self.mainApp.initMods.dctData:
			tmp.append(SMData(grp[0], grp[1], 'Dictionary'))
		for grp in self.mainApp.initMods.cmtData:
			tmp.append(SMData(grp[0], grp[1], 'Commentary'))
		for grp in self.mainApp.initMods.bkData:
			tmp.append(SMData('Books', grp[1], 'Book'))

		return tmp
		
	
	def InitOLV(self):
		#self.olv.putBlankLineBetweenGroups = False
		self.olv.SetColumns([
			ColumnDefn("Tab Name", "left", 90, "tab", useInitialLetterForGroupKey=True),
			ColumnDefn("Description", "left", 190, "desc", useInitialLetterForGroupKey=True),
			ColumnDefn("Type", "left", 90, "type"),
		])
		self.olv.CreateCheckStateColumn()
		self.olv.SetSortColumn(self.olv.columns[4])
		self.olv.SetObjects(self.data)
	
	
	def OnToggleButtons(self, event):
		if not event:
			currButton = self.buttonList[self.mainApp.sessTab]
			currButton.SetToggle(True)
			objects = self.olv.GetObjects()
			for obj in objects:
				self.olv.SetCheckState(obj, True)
			self.olv.RefreshObjects(objects)
		else:
			currButton = event.GetEventObject()
			
		remainList = self.buttonList[:]
		remainList.remove(currButton)
		currIdx = self.buttonList.index(currButton)
		
		self.olv.Enable(True)
		if currIdx == 0:
			self.olv.Enable(False)
			#print self.olv.GetScreenPosition()
			#print self.olv.GetSize()
		
		for b in remainList:
			if b.GetToggle() == True:
				downIdx = self.buttonList.index(b)
				self.GetUnChecked(downIdx)
				b.SetToggle(False)
				self.SetChecked(currIdx)
		self.currButtonIdx = currIdx
		


	def SetChecked(self, currIdx):
		print currIdx, 'currentDown'
		self.currButton = currIdx
		objects = self.olv.GetObjects()
		if currIdx == 0:
			for obj in objects:
				self.olv.SetCheckState(obj, True)
		else:
			for obj in objects:
				self.olv.SetCheckState(obj, True)
				currSessList = []
				for grp in self.sessList[currIdx -1]:
					for desc in grp:
						currSessList.append(desc)
				#print currSessList
				if obj.desc in currSessList:
					self.olv.SetCheckState(obj, False)                       
                if obj.desc == 'Authorized King James Version':
                    self.olv.SetCheckState(obj, True)                       


					
		self.olv.RefreshObjects(objects)



	def GetUnChecked(self, downIdx):
		#print downIdx, 'lastDown'
		if downIdx == 0:
			return
        
		objects = self.olv.GetObjects()
		self.idDct = False
		self.idCmt = False
		self.idBk = False
		bibTmp = []
		dctTmp = []
		cmtTmp = []
		bkTmp = []
        
		for obj in objects:
			if obj.desc == 'Authorized King James Version':
				self.olv.SetCheckState(obj, True)
			if not self.olv.IsChecked(obj):

				if obj.type == 'Bible':
					bibTmp.append(obj.desc)
				elif obj.type == 'Dictionary':
					dctTmp.append(obj.desc)
				elif obj.type == 'Commentary':
					cmtTmp.append(obj.desc)
				elif obj.type == 'Book':
					bkTmp.append(obj.desc)
			else:
				if obj.type == 'Dictionary':
					self.idDct = True
				if obj.type == 'Commentary':
					self.idCmt = True
				if obj.type == 'Book':
					self.idBk = True
		
		self.sessList[downIdx -1] = [bibTmp, dctTmp, cmtTmp, bkTmp]
		#print self.sessList[downIdx -1]
		

	def OnTextSearchCtrl(self, event):
		self.searchCtrl.ShowCancelButton(len(self.searchCtrl.GetValue()))
		self.olv.GetFilter().SetText(self.searchCtrl.GetValue())
		self.olv.RebuildGroups()

	def OnCancelSearchCtrl(self, event):
		self.searchCtrl.SetValue("")
		self.OnTextSearchCtrl(None)
	

	def OnRename(self, event):
		btn = self.buttonList[self.currButtonIdx]
		
		dlg = wx.TextEntryDialog(self, 'Enter a new name for the session', 'Rename Session Group', '')
		dlg.SetValue(btn.GetLabel())
		if dlg.ShowModal() == wx.ID_OK:
			btn.SetLabel(dlg.GetValue())
			self.mainApp.sessTabTitles[self.currButtonIdx] = dlg.GetValue()
		dlg.Destroy()
		
		btn.Refresh()


	def OnHelp(self, event):
		self.mainApp.OnContextHelp(412)

	
	def OnCloseWindow(self, event):
		self.Hide()
		self.Destroy()


	def OnLoad(self, event):
		
		self.GetUnChecked(self.currButtonIdx)
		
		if not self.idDct or not self.idCmt or not self.idBk:
			dlg = wx.MessageDialog(self, 'At least one title in each catagory must be checked.\n\n\
Ensure at least one Dictionary, one Commentary, and one Book title is checked under the current seaaion tab and then click "Load".\n\n \
The Authorized Version Bible is always checked. It must be available for Bible Analyzer to start.',
			'One Title in Each Catagory Must Be Checked ', wx.OK | wx.ICON_INFORMATION    )
			dlg.ShowModal()
			dlg.Destroy()
			return
        
		pFile = file(self.mainApp.userDir + r'/sessions.dat', 'wb')
		cPickle.dump(self.sessList, pFile)
		pFile.close()
		
		self.mainApp.sessTab = self.currButton 
		self.mainApp.Freeze()
		self.mainApp.OnReload()
		self.mainApp.Thaw()
		wx.CallAfter(self.Hide)
		
	
	def OnClose(self, event):
		self.Hide()


