# -*- coding: utf-8 -*-

import wx, os, re
import stats, utility
import modTools, rtf2htm 
import sqlite3 as sqlite

book = stats.books

class ESConvert(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        
        self.mainApp = wx.GetApp().GetTopWindow()
        self.path = self.mainApp.myDocsDir
        
        self.esModType = wx.RadioBox(self, -1, "E-Sword Module Type", choices=["Bible", "Dictionary", "Commentary         ", "Topic"], majorDimension=2, style=wx.RA_SPECIFY_COLS)
        fileLabel = wx.StaticText(self, -1, 'Select E-Sword file')     
        fileButton = wx.Button(self, -1, "Browse", size=(50, -1))        
        self.esFileName = wx.TextCtrl(self, -1, "", size=(170, -1))
        self.makeTxt = wx.CheckBox(self, -1, 'Make formatted text file (with RTF tags).')
        createButton = wx.Button(self, -1, "Create", size=(50, -1))
        #finishLabel = wx.StaticText(self, -1, 'If .')
        
        
        #finishLabel = wx.StaticText(self, -1, '         Restart Bible Analyzer\nto view new module.')
        finishLabel2 = wx.StaticText(self, -1, 'If problems, see Module Tips at left.')
        
        fileLabel.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        
        #finishLabel.SetForegroundColour(wx.Colour(0, 0, 100))
        finishLabel2.SetForegroundColour(wx.Colour(0, 0, 100))
        
        self.esProgLabel = wx.StaticText(self, -1, '')
        self.esProgLabel.SetForegroundColour(wx.Colour(100, 0, 0))
        
        self.esSizer = wx.BoxSizer(wx.VERTICAL)
        fileSizer = wx.BoxSizer(wx.HORIZONTAL)
        
        self.SetSizer(self.esSizer)
        
        title = modTools.optionsDlg.FMTitlePanel(self, "Formatted Text From Esword")
        self.esSizer.Add(title, 0, wx.EXPAND | wx.ALL, 2)
        
        self.esSizer.Add(self.esModType, 0, wx.EXPAND | wx.ALL, 5)
        
        self.esSizer.Add(fileLabel, 0, wx.ALIGN_CENTER)
        self.esSizer.Add((-1, 2))
        
        fileSizer.Add(self.esFileName, 0, wx.LEFT, 0)    
        
        fileSizer.Add(fileButton, 0, wx.LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
        self.esSizer.Add(fileSizer, 0, wx.LEFT, 5)
        self.esSizer.Add((-1, 15))
        self.esSizer.Add(self.makeTxt, 0, wx.LEFT, 5)

        self.esSizer.Add(createButton, 0, wx.wx.ALIGN_CENTER|wx.TOP, 10)        
        self.esSizer.Add((-1, 15))        
        self.esSizer.Add(self.esProgLabel, 0, wx.ALIGN_CENTER, 0)
        #self.esSizer.Add(finishLabel, 0, wx.TOP|wx.ALIGN_CENTER, 25)
        self.esSizer.Add(finishLabel2, 0, wx.TOP|wx.ALIGN_CENTER, 5)
        
        fileButton.Bind(wx.EVT_BUTTON, self.OnEsFile)
        createButton.Bind(wx.EVT_BUTTON, self.OnEsCreate)
        
        
        #self.esModType.EnableItem(0, False)
        #self.esModType.EnableItem(1, False)
        #self.esModType.EnableItem(2, False)
        #self.esModType.EnableItem(3, False)
        #self.makeTxt.Enable(False)
        #createButton.Enable(False)
        #fileButton.Enable(False)


        return

    
    def OnEsFile(self, event):
        self.esProgLabel.SetLabel('')
        self.esSizer.Layout()        
        self.Update()
        
        type = self.esModType.GetSelection()
        
        if type == 0:
            wildcard = "E-Sword Bible File (*.bblx)|*.bblx|" "All files (*.*)|*.*"
        elif type == 1:
            wildcard = "E-Sword Dictionary File (*.dctx)|*.dctx|" "All files (*.*)|*.*"
        elif type == 2:
            wildcard = "E-Sword Commentary File (*.cmtx)|*.cmtx|" "All files (*.*)|*.*"
        elif type == 3:
            wildcard = "E-Sword Topics File (*.topx)|*.topx|" "All files (*.*)|*.*"

        dlg = wx.FileDialog(self, message="Choose a file", defaultDir=os.getcwd(), 
            defaultFile="", wildcard=wildcard, style=wx.OPEN | wx.CHANGE_DIR)

        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPaths()
            self.esFileName.SetValue(path[0])
            self.txtPath = path[0]

        dlg.Destroy()        
        
    
    def OnEsCreate(self, event):
        if self.esFileName.IsEmpty():
            dlg = wx.MessageDialog(self, 'You must select or enter a file to process.',
                                         'File Name Needed', wx.OK | wx.ICON_INFORMATION	)
            dlg.ShowModal()
            dlg.Destroy()
            return
        
        type = self.esModType.GetSelection()
        
        try:
            pth, fn = os.path.split(self.txtPath)
        except IOError:    
            dlg = wx.MessageDialog(self, 'This file was not found.',
                                         'File Not Found', wx.OK | wx.ICON_INFORMATION    )
            dlg.ShowModal()
            dlg.Destroy()
            return

        
        
        
        self.baseName, ext = os.path.splitext(fn)
        
        wx.BeginBusyCursor()
     
        try:
            if type == 0:
                self.esBible2Text(self.txtPath)
            elif type == 1:
                self.esDct2Text(self.txtPath)
            elif type == 2:
                self.esCmt2Text(self.txtPath)
            elif type == 3:
                self.esTop2Text(self.txtPath)
        except:
            dlg = wx.MessageDialog(self, 'The converter encountered an error and has been aborted. Due to inconsistant formatting \
or other issues, some e-sword modules cannot be reliably converted.',
                                         'Compiling Error', wx.OK | wx.ICON_INFORMATION	)
            dlg.ShowModal()
            dlg.Destroy()
        
        wx.EndBusyCursor()
    

    def esBible2Text(self, dbPath):
        self.esProgLabel.SetLabel('Extracting E-Sword Data...')
        self.esSizer.Layout()
        self.Update()

        verses = []
        raw = []
        
        folder, fn = os.path.split(dbPath)
        base, ext = os.path.splitext(fn)
        
        con = sqlite.connect(dbPath)
        con.text_factory = str
        cur = con.cursor()
        
        cur.execute('SELECT * FROM details')
        detailRow = cur.fetchone()
               
        cur.execute('SELECT * FROM bible')
        rows = cur.fetchall()
        
        
        desc = detailRow[0]
        abbr = detailRow[1]
        info = detailRow[2]
        strongs = detailRow[9]
        
        info = rtf2htm.r2h(info)
        
            
        for i in rows:
            try:
                bk = book[i[0] -1]
            except:
                continue
                
            ch = i[1]
            vs = i[2]
            ln = i[3]
            
            try:
                ln = unicode(ln, "utf-8")
            except:
                print 'Latin1'
                ln = unicode(ln, "Latin1")


            line = u'%s %s:%s\t%s' % (bk, ch, vs, ln)
            raw.append(line)

            ln = rtf2htm.r2h(ln, 1)
                                   
            line = u'%s %s:%s\t%s' % (bk, ch, vs, ln)
            verses.append(line)
    
        os.chdir(self.path) # + 'Bible')
        if os.access(abbr + ".bib", 1):
            os.remove(abbr + ".bib")
        
        
        self.esProgLabel.SetLabel('Making Word List...')
        self.esSizer.Layout()
        self.Update()
        
        # Make word list
        tmpList = []
        wCount = {}
                
        for line in verses:
            if strongs == 1:
                line = re.sub(r'\[.+?\]', '', line)
            line = re.sub(r'\w\w\w \d+:\d+\t|[().,:;?!"-\'\[\]*&%$#\^]|<.*?>|\d', '', line)
            wds = line.split()
            tmpList.extend(wds)

        for word in tmpList:
            wCount[word] = wCount.get(word, 0) 

        wdList = wCount.keys()
        wdList.sort()
                
        self.esProgLabel.SetLabel('Making Database...')
        self.esSizer.Layout()
        self.Update()
        
        print os.getcwd()
        
        # Make database
        con = sqlite.connect(abbr.lower() + '.bib')
        #con.text_factory = str
        cur = con.cursor()

        cur.execute("create table bible(id INTEGER PRIMARY KEY, ref, verse)")
        cur.execute("create table wordlist(word)")
        cur.execute("create table title(abbr, desc, info)")
        cur.execute("create table strongs(idx)")

        addm = '<center>This <b>Bible Analyzer</b> Bible module<br>was made from an e-sword module by a third party.<hr></center><br><br>'
        
        modInfo = addm + info

        cur.execute('insert into title values (?, ?, ?)', (abbr, desc, modInfo))
        cur.execute('insert into strongs values (?)', (strongs,))

        num = 1
        for word in wdList:   
            num += 1
            cur.execute('insert into wordlist values (?)', (word.encode("utf-8"),))

        for line in verses:
            idx = line.find('\t')
            line = line.encode("utf-8") + '\n'
            #line = line + '\n'
            
            cur.execute('insert into bible values (null, ?, ?)', (line[:idx], line[idx+1:],))

        con.commit()        
        
        if self.makeTxt.IsChecked():
            txt = '\n'.join(raw)
            #print type(txt)
            txt = txt.encode("utf-8")
            fOut = file(base + ".txt", 'w')
            fOut.write(txt)
            fOut.close()
        
        self.esProgLabel.SetLabel('Conversion Complete!')
        self.esSizer.Layout()
        self.Update()
        
        folder, fn = os.path.split(dbPath)
        os.chdir(folder)


    def esDct2Text(self, dbPath):
        self.esProgLabel.SetLabel('Extracting E-Sword Dictionary Data...')
        self.esSizer.Layout()
        self.Update()

        entries = []
        raw = []
        
        folder, fn = os.path.split(dbPath)
        base, ext = os.path.splitext(fn)
        
        con = sqlite.connect(dbPath)
        con.text_factory = str
        cur = con.cursor()
        
        cur.execute('SELECT * FROM details')
        detailRow = cur.fetchone()
        
        desc = detailRow[0]
        abbr = detailRow[1]
        info = detailRow[2]
        
        info = rtf2htm.r2h(info)
        
        cur.execute('SELECT * FROM dictionary')
        rows = cur.fetchall()
        
        for i in rows:
            topic = i[0]
            defin = i[1]
            print topic
            
            #topic = unicode(topic, 'utf-8', 'replace')
            topic = utility.CheckEncoding(topic)
            defin = utility.CheckEncoding(defin)

            line = u'%s\t%s' % (topic, defin)
            raw.append(line)

            defin = rtf2htm.r2h(defin)
                                   
            line = u'%s\t%s' % (topic, defin)
            entries.append(line)

        self.esProgLabel.SetLabel('Making Database...')
        self.esSizer.Layout()
        self.Update()
        
        # Make database
        os.chdir(self.path)
        if os.access(abbr + ".dct", 1):
            os.remove(abbr + ".dct")
        
        con = sqlite.connect(abbr.lower() + '.dct')
        cur = con.cursor()

        cur.execute("create table dictionary(id INTEGER PRIMARY KEY, topic, definition)")
        cur.execute("create table topics(topic)")
        cur.execute("create table title(abbr, desc, info)")

        addm = '<center>This <b>Bible Analyzer</b> Dictionary module<br>was made from an e-sword module by a third party.<hr></center><br><br>'
        modInfo = addm + info

        cur.execute('insert into title values (?, ?, ?)', (abbr, desc, modInfo))

        tmpCon = sqlite.connect(abbr.lower() + '.idx')
        tmpCur = tmpCon.cursor()
        tmpCur.execute("create table topics(id INTEGER PRIMARY KEY, topic)")

        for line in entries:
            idx = line.find('\t')
            line = line.encode("utf-8")
            cur.execute('insert into topics values (?)', (line[:idx].title(),))
            tmpCur.execute('insert into topics values (null, ?)', (line[:idx].title(),))
            cur.execute('insert into dictionary values (null, ?, ?)', (line[:idx].title(), line[idx+1:],))

        con.commit() 
        tmpCon.commit() 

        if self.makeTxt.IsChecked():
            txt = '\n'.join(raw)
            txt = txt.encode("utf-8")
            fOut = file(base + ".txt", 'w')
            fOut.write(txt)
            fOut.close()
        
        self.esProgLabel.SetLabel('Dictionary Conversion Complete!')
        self.esSizer.Layout()
        self.Update()
        
        folder, fn = os.path.split(dbPath)
        os.chdir(folder)


    def esCmt2Text(self, dbPath):
        self.esProgLabel.SetLabel('Extracting E-Sword Commentary Data...')
        self.esSizer.Layout()
        self.Update()

        raw = []
        entries = []
        
        folder, fn = os.path.split(dbPath)
        base, ext = os.path.splitext(fn)
        
        con = sqlite.connect(dbPath)
        con.text_factory = str
        cur = con.cursor()
        
        cur.execute('SELECT * FROM details')
        detailRow = cur.fetchone()
        
        desc = detailRow[0]
        abbr = detailRow[1]
        info = detailRow[2]
        
        info = rtf2htm.r2h(info)

        ## Book comments
        cur.execute('SELECT * FROM books')
        rows = cur.fetchall()
        for i in rows:
            try:
                bk = book[i[0] -1]
                ln = i[1]
            except:
                pass
            
            line = '%s\t%s' % (bk, ln)
            line = re.sub(r'\n', ' ', line)
            raw.append(line)            
            
            ln = rtf2htm.r2h(ln)
            
            line = '%s\t%s' % (bk, ln)
            entries.append(line)


        ## Chapter comments
        cur.execute('SELECT * FROM chapters')
        rows = cur.fetchall()
        for i in rows:
            try:
                bk = book[i[0] -1]
                ch = i[1]
                ln = i[2]
            except:
                pass

            line = '%s %s:\t%s' % (bk, ch, ln)
            line = re.sub(r'\n', ' ', line)
            raw.append(line)

            ln = rtf2htm.r2h(ln)
            
            line = '%s %s:\t%s' % (bk, ch, ln)
            entries.append(line)

        
        ## Verse comments
        cur.execute('SELECT * FROM verses')
        rows = cur.fetchall()
        for i in rows:
            try: 
                bk = book[i[0] -1]
            except:
                pass
            ch = i[1]
            vs1 = i[3]
            vs2 = i[4]
            ln = i[5]
            
            if vs1 == vs2:
                vs = vs1
            else:
                vs = '%s-%s' % (vs1, vs2)

            line = '%s %s:%s\t%s' % (bk, ch, vs, ln)
            line = re.sub(r'\n', ' ', line)
            raw.append(line)

            ln = rtf2htm.r2h(ln)
                                  
            line = '%s %s:%s\t%s' % (bk, ch, vs, ln)
            entries.append(line)


        self.esProgLabel.SetLabel('Making Database...')
        self.esSizer.Layout()
        self.Update()
        
        # Make database
        os.chdir(self.path)
        if os.access(abbr + ".cmt", 1):
            os.remove(abbr + ".cmt")
        
        con = sqlite.connect(abbr.lower() + '.cmt')
        cur = con.cursor()

        cur.execute("create table commentary(id INTEGER PRIMARY KEY, ref, comment)")
        cur.execute("create table reference(ref)")
        cur.execute("create table title(abbr, desc, info)")

        addm = '<center>This <b>Bible Analyzer</b> Commentary module<br>was made from an e-sword module by a third party.<hr></center><br><br>'
        modInfo = addm + info

        cur.execute('insert into title values (?, ?, ?)', (abbr, desc, modInfo))
        
        tmpCon = sqlite.connect(abbr.lower() + '.idx')
        tmpCur = tmpCon.cursor()
        tmpCur.execute("create table reference(id INTEGER PRIMARY KEY, ref)")

        for line in entries:
            idx = line.find('\t')
            line = line.encode("utf-8")
            line = re.sub(r'\n', ' ', line)
            cur.execute('insert into reference values (?)', (line[:idx],))
            tmpCur.execute('insert into reference values (null, ?)', (line[:idx],))    
            cur.execute('insert into commentary values (null, ?, ?)', (line[:idx], line[idx+1:],))

        con.commit()
        tmpCon.commit()


        if self.makeTxt.IsChecked():
            #txt = '\n'.join(entries)
            txt = '\n'.join(raw)
            txt = txt.encode("utf-8")
            fOut = file(base + ".txt", 'w')
            fOut.write(txt)
            fOut.close()
        
        self.esProgLabel.SetLabel('Commentary Conversion Complete!')
        self.esSizer.Layout()
        self.Update()
        
        folder, fn = os.path.split(dbPath)
        os.chdir(folder)


    def esTop2Text(self, dbPath):
        self.esProgLabel.SetLabel('Extracting E-Sword Topics Data...')
        self.esSizer.Layout()
        self.Update()

        entries = []
        raw = []
        
        con = sqlite.connect(dbPath)
        con.text_factory = str
        cur = con.cursor()
                
        cur.execute('SELECT * FROM topics')
        rows = cur.fetchall()
        
        for i in rows:
            title = i[0]
            notes = i[1]
            
            title = utility.CheckEncoding(title)
            notes = utility.CheckEncoding(notes)

            line = '%s\t%s' % (title, notes)
            raw.append(re.sub(r'\n', ' ', line))

            notes = rtf2htm.r2h(notes)
            
            print title.encode('utf-8')
                                   
            line = '%s\t%s' % (title, notes)
            entries.append(line)

        self.esProgLabel.SetLabel('Making Database...')
        self.esSizer.Layout()
        self.Update()

        folder, fn = os.path.split(dbPath)
        base, ext = os.path.splitext(fn)
        
        print base
        abbr = base[:12] + '...'
        desc = base

        # Make database
        os.chdir(self.path)
        if os.access(base + ".bk", 1):
            os.remove(base + ".bk")
        
        con = sqlite.connect(base + '.bk')
        cur = con.cursor()

        cur.execute("create table book(id INTEGER PRIMARY KEY, chapter, text)")
        cur.execute("create table info(abbr, title)")

        cur.execute('insert into info values (?, ?)', (abbr, desc))

        for line in entries:
            idx = line.find('\t')
            line = line.encode("utf-8")
            cur.execute('insert into book values (null, ?, ?)', (line[:idx], line[idx+1:],))

        con.commit() 

        if self.makeTxt.IsChecked():
            txt = '\n'.join(raw)
            #txt = '\n'.join(entries) 
            
            #path = self.mainApp.userDir + r'\Modules'
            
            #txt = re.sub(r'\n', ' ', txt.encode("utf-8"))

            txt = txt.encode("utf-8")
            fOut = file(base + ".txt", 'w')
            fOut.write(txt)
            fOut.close()
        
        #print raw[0]
        self.esProgLabel.SetLabel('Topic Conversion Complete!')
        self.esSizer.Layout()
        self.Update()
        
        os.chdir(folder)
