import wx, os, re
import utility, optionsDlg 
import wx.lib.dialogs

import esConvert, xmlConvert

#from pysqlite2 import dbapi2 as sqlite
import sqlite3 as sqlite

from artManager import ArtManager
from resources import *
from labelBook import LabelBook




class ModTools(wx.Dialog):
    def __init__(self, parent=None):
        self.mainApp = wx.GetApp().GetTopWindow()
        self.path = self.mainApp.myDocsDir
        
        wx.Dialog.__init__(self, parent, -1, "Module Creation Tools (Beta)")

        self.CreateDialog()
        self.Initialize()
        
        self.SetSize((350, 450))

    def CreateDialog(self):
        CloseButton = wx.Button(self, -1, "Close", size=(-1, 25))
        
        sz = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sz)
        
        self.tBook = LabelBook(self, -1, style = INB_NO_RESIZE | INB_LEFT | INB_DRAW_SHADOW | INB_BORDER)
        sz.Add(self.tBook, 1, wx.EXPAND)

        self.tBook.SetColour(INB_TAB_AREA_BACKGROUND_COLOR, ArtManager.Get().GetMenuFaceColour())

        colour = wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE)
        self.tBook.SetColour(INB_ACTIVE_TAB_COLOR, colour)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        hsizer.Add(CloseButton, 0, wx.EXPAND | wx.ALIGN_RIGHT)
        sz.Add(wx.StaticLine(self), 0, wx.EXPAND | wx.TOP | wx.BOTTOM, 3)
        sz.Add(hsizer, 0, wx.ALIGN_RIGHT | wx.ALL, 2)
        
        self.Bind(wx.EVT_BUTTON, self.OnClose, CloseButton)
        self.Bind(wx.EVT_CLOSE, self.OnClose)

    
    def Initialize(self):
        esPage = esConvert.ESConvert(self)
        xmlPage = xmlConvert.XMLConvert(self)
        
        self.tBook.DeleteAllPages()
        self.tBook.AddPage(self.createMod(), "Module Creator", True)
        self.tBook.AddPage(esPage, "Convert E-Sword", False)
        self.tBook.AddPage(xmlPage, "Convert XML", False)
        self.tBook.AddPage(self.QuickHelpPage(), "Module Tips", False)

    def OnClose(self, event):
        self.Hide()
        self.Destroy()


    def QuickHelpPage(self):
        qHelp = wx.Panel(self, -1)
        display = wx.html.HtmlWindow(qHelp, -1, style=wx.SUNKEN_BORDER )
        
        qSizer = wx.BoxSizer(wx.VERTICAL)
        qHelp.SetSizer(qSizer)
        
        title = optionsDlg.FMTitlePanel(qHelp, "Module Conversion/Creation Tips")
        qSizer.Add(title, 0, wx.EXPAND|wx.ALL, 5)
        qSizer.Add(display, 1, wx.EXPAND | wx.LEFT | wx.RIGHT, 5)
        
        text = '''<font face="Verdana" size="-1"><i><b>Module Creator</b><br>
            <br>
            Modules can only be created from properly formatted text files. Each module type 
            (Bible, Dictionary Commentary, Devotion) has its own format. See the <i>Users Manual</i> for formatting
            information.
            <br><br>
            If <i>Module Creator</i> says a module was successfully made but it does not show up in Bible Analyzer after a restart or is
            malformed, there is likely a formatting error. 
            <hr>
            <b>E-Sword Module Conversion</b><br><br>           
            <i>If the text in the new module is garbled, it is because the e-sword module is encrypted. All modules from e-sword.net are encrypted. 
            Only user made files can be converted. Either delete the file or uncheck it in Preferences.</i>
            <br><br>
            Bible Analyzer can attempt to convert non-encrypted e-sword modules that are in the version 9 SQLite format. 
            It cannot convert the older JET database format (version 8 and earlier). 
            <br><br>
            E-sword modules use RTF for text markup
            while Bible Analyzer uses HTML. The converter will convert basic RTF tags to HTML such as <i>bold, italic, font, etc.,</i>
            but it will not convert more complex tags such as <i>tables, indentions, etc.</i> If the user wishes to convert these 
            tags manually, check the <i>Make Text File</i> box and edit the text. Then run the edited text through <i>Module Creator.</i>
            <br><br>
            In most cases if the E-Sword module has properly formatted references that produced popups in the E-Sword application, they will
            be converted to the Bible Analyzer format.  
            <br><br>
            Non Latin based texts (Hebrew, Greek, etc.) are not supported at this time. However, some Bible texts with Strongs Numbers are 
            supported.
            <br><br>
            <i>Note:</i>
            <br>
            E-Sword modules are not consistant in their RTF tagging or layout. Thus it is very difficult to convert them with the same general look.
            For a more consistant look and more control within Bible Analyzer, it is recommended that a text file be made and edited and then run 
            through <i>Module Creator</i>.
            <br><br>
                            
            <br>
            </font>
            '''
                
        display.SetPage(text)
        
        return qHelp




    ## Create module from raw file
    def createMod(self):
        panel = wx.Panel(self.tBook, -1)
        self.modType = wx.RadioBox(panel, -1, "Module Type", choices=["Bible", "Dictionary", "Commentary         ", "Devotion", "Book/Article"], majorDimension=2, style=wx.RA_SPECIFY_COLS)
        fileLabel = wx.StaticText(panel, -1, 'Select formatted text file')
        
        fileButton = wx.Button(panel, -1, "Browse", size=(50, -1))        
        self.fileName = wx.TextCtrl(panel, -1, "", size=(170, -1))
        
        tabLabel = wx.StaticText(panel, -1, 'Text on tab')
        self.tabTxt = wx.TextCtrl(panel, -1, "", size=(100, -1))

        titleLabel = wx.StaticText(panel, -1, 'Text for title')
        self.titleTxt = wx.TextCtrl(panel, -1, "", size=(160, -1))

        infoLabel = wx.StaticText(panel, -1, 'Text for module info (Can use basic HTML)')
        self.infoTxt = wx.TextCtrl(panel, -1, "", size=(-1, 80), style=wx.TE_MULTILINE)

        createButton = wx.Button(panel, -1, "Create", size=(50, -1))
        
        self.progLabel = wx.StaticText(panel, -1, '')

        
        tabLabel.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        titleLabel.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        infoLabel.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        fileLabel.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        self.progLabel.SetForegroundColour(wx.Colour(100, 0, 0))



        self.fSizer = wx.BoxSizer(wx.VERTICAL)
        fileSizer = wx.BoxSizer(wx.HORIZONTAL)
        tabSizer = wx.BoxSizer(wx.HORIZONTAL)
        titleSizer = wx.BoxSizer(wx.HORIZONTAL)
        
        panel.SetSizer(self.fSizer)
        
        title = optionsDlg.FMTitlePanel(panel, "Create Module From Formatted Text")
        self.fSizer.Add(title, 0, wx.EXPAND | wx.ALL, 2)
        
        self.fSizer.Add(self.modType, 0, wx.EXPAND | wx.ALL, 5)
        
        self.fSizer.Add(fileLabel, 0, wx.ALIGN_CENTER)
        self.fSizer.Add((-1, 2))
        
        fileSizer.Add(self.fileName, 0, wx.LEFT, 0)        
        fileSizer.Add(fileButton, 0, wx.LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
        self.fSizer.Add(fileSizer, 0, wx.LEFT, 5)

        tabSizer.Add(self.tabTxt, 0, wx.LEFT, 0)
        tabSizer.Add(tabLabel, 0, wx.LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
        self.fSizer.Add(tabSizer, 0, wx.LEFT|wx.TOP, 5)

        titleSizer.Add(self.titleTxt, 0, wx.LEFT, 0)        
        titleSizer.Add(titleLabel, 0, wx.LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
        self.fSizer.Add(titleSizer, 0, wx.LEFT|wx.TOP, 5)

        self.fSizer.Add(infoLabel, 0, wx.TOP|wx.ALIGN_CENTER, 5)
        self.fSizer.Add((-1, 2))        
        self.fSizer.Add(self.infoTxt, 0, wx.LEFT|wx.RIGHT|wx.EXPAND, 5)        

        self.fSizer.Add(createButton, 0, wx.wx.ALIGN_CENTER|wx.TOP, 10) 
        self.fSizer.Add((-1, 10))        
        self.fSizer.Add(self.progLabel, 0, wx.ALIGN_CENTER, 0)
        
        fileButton.Bind(wx.EVT_BUTTON, self.OnFile)
        createButton.Bind(wx.EVT_BUTTON, self.OnCreate)
        self.Bind(wx.EVT_RADIOBOX, self.SelChanged, self.modType)
        self.Layout()
                
        return panel

    
    def SelChanged(self, event):
        if self.modType.GetSelection() == 4:
            self.tabTxt.Enable(False)
            self.titleTxt.Enable(False)
            self.infoTxt.Enable(False)
        else:
            self.tabTxt.Enable(True)
            self.titleTxt.Enable(True)
            self.infoTxt.Enable(True)
            
    
    def OnFile(self, event):
        self.progLabel.SetLabel('')
        self.fSizer.Layout()        
        self.Update()

        wildcard = "Text File (*.txt)|*.txt|" "All files (*.*)|*.*"
        dlg = wx.FileDialog(self, message="Choose a file", defaultDir=os.getcwd(), 
            defaultFile="", wildcard=wildcard, style=wx.OPEN | wx.CHANGE_DIR)

        if dlg.ShowModal() == wx.ID_OK:
            txtPath = dlg.GetPaths()
            self.fileName.SetValue(txtPath[0])

        dlg.Destroy()        

    
    def OnError(self):
        dlg = wx.MessageDialog(self, '''
        The compiler encountered an error and has been aborted.
        Ensure that the text is formatted correctly
        and that the correct module type is selected.''',
                                     'Compiling Error', wx.OK | wx.ICON_INFORMATION	)
        dlg.ShowModal()
        dlg.Destroy()
        
    

    def OnCreate(self, event):
        if self.fileName.IsEmpty():
            dlg = wx.MessageDialog(self, 'You must select or enter a file to process.',
                                         'File Name Needed', wx.OK | wx.ICON_INFORMATION	)
            dlg.ShowModal()
            dlg.Destroy()
            return
        
        if self.tabTxt.IsEmpty() and self.modType.GetSelection() != 4:
            dlg = wx.MessageDialog(self, 'You must enter a name in the Tab Text box.',
                                         'Tab Name Needed', wx.OK | wx.ICON_INFORMATION	)
            dlg.ShowModal()
            dlg.Destroy()
            return

        type = self.modType.GetSelection()
        
        if self.modType.GetSelection() != 4:
            self.tab = self.tabTxt.GetValue()
            self.title = self.titleTxt.GetValue()
            self.info = self.infoTxt.GetValue()
        
        self.fName = self.fileName.GetValue()
        
        try:
            lines = file(self.fName).readlines()
        except IOError:    
            dlg = wx.MessageDialog(self, 'This file was not found.',
                                         'File Not Found', wx.OK | wx.ICON_INFORMATION    )
            dlg.ShowModal()
            dlg.Destroy()
            return
            
            

        try:
            print 'Latin1'
            self.lines = [ln.decode("latin1") for ln in lines]
        except:
            print 'ASCII or UTF-8'
            self.lines = [ln.decode("utf-8") for ln in lines]

        
        wx.BeginBusyCursor()
     
        try:
            if type == 0:
                self.MakeBib()
            elif type == 1:
                self.MakeDict()
            elif type == 2:
                self.MakeCom()
            elif type == 3:
                self.MakeDev()
            elif type == 4:
                self.MakeBook()

        except:
            self.OnError()
                
        wx.EndBusyCursor()


    def MakeBib(self):
        error = []
        
        self.progLabel.SetLabel('Making Word List...')
        self.fSizer.Layout()        
        self.Update()
        
        # Make word list
        tmpList = []
        wCount = {}
        strongs = 0
        
        if re.search(r'\[[GH]\d+\]', self.lines[0]):
            strongs = 1
        
        for line in self.lines:
            if strongs == 1:
                line = re.sub(r'\[.+?\]', '', line)
            line = re.sub(r'\w\w\w \d+:\d+\t|[().,:;?!"-\'\[\]*&%$#\^]|<.*?>|\d', '', line)
            wds = line.split()
            tmpList.extend(wds)

        for word in tmpList:
            wCount[word] = wCount.get(word, 0) 

        wdList = wCount.keys()
        wdList.sort()
        
        os.chdir(self.path)
        
        if os.access(self.tab + ".bib", 1):
            os.remove(self.tab + ".bib")

        self.progLabel.SetLabel('Making Bible Module...')
        self.fSizer.Layout()
        self.Update()
        
        # Make database
        con = sqlite.connect(self.tab + '.bib')
        #con.text_factory = str
        cur = con.cursor()

        cur.execute("create table bible(id INTEGER PRIMARY KEY, ref, verse)")
        cur.execute("create table wordlist(word)")
        cur.execute("create table title(abbr, desc, info)")
        cur.execute("create table strongs(idx)")

        addm = '<center>This <b>Bible Analyzer</b> Bible module was made by a third party.<hr></center><br><br>'
        #addm = ''
        
        desc = addm + self.info

        cur.execute('insert into title values (?, ?, ?)', (self.tab, self.title, desc))

        cur.execute('insert into strongs values (?)', (strongs,))

        num = 1
        for word in wdList:   
            num += 1
            cur.execute('insert into wordlist values (?)', (word,))
        
        cyc = 1
        for line in self.lines:
            line = line.encode('utf-8')
            if '\t' in line:
                idx = line.find('\t')
                line[idx+1:]
                cur.execute('insert into bible values (null, ?, ?)', (line[:idx], line[idx+1:],))
            else:
                error.append(str(cyc))
                continue
            cyc += 1

        con.commit()        
        
        self.progLabel.SetLabel('%s Bible Module Complete!' % self.tab)
        self.fSizer.Layout()
        
        if error:
            self.errorDlg(error)
    
    
    def MakeDict(self):
        self.progLabel.SetLabel('Making Dictionary Module...')
        self.fSizer.Layout()
        self.Update()
        
        error = []
        
        os.chdir(self.path)
        if os.access(self.tab + ".dct", 1):
            os.remove(self.tab + ".dct")
        
        con = sqlite.connect(self.tab + '.dct')
        cur = con.cursor()

        cur.execute("create table dictionary(id INTEGER PRIMARY KEY, topic, definition)")
        cur.execute("create table topics(topic)")
        cur.execute("create table title(abbr, desc, info)")

        addm = '<center>This <b>Bible Analyzer</b> Dictionary module was made by a third party.<hr></center><br><br>'
        
        desc = addm + self.info

        cur.execute('insert into title values (?, ?, ?)', (self.tab, self.title, desc))

        cyc = 1
        for line in self.lines:
            line = line.encode('utf-8')
            if '\t' in line:
                idx = line.find('\t')
                cur.execute('insert into topics values (?)', (line[:idx],))
                cur.execute('insert into dictionary values (null, ?, ?)', (line[:idx], line[idx+1:],))
            else:
                error.append(str(cyc))
                continue
            
            cyc += 1
                
        con.commit()
        
        self.progLabel.SetLabel('%s Dictionary Module Complete!' % self.tab)
        self.fSizer.Layout()
        
        if error:
            self.errorDlg(error)

    
    def MakeCom(self):
        self.progLabel.SetLabel('Making Commentary Module...')
        self.fSizer.Layout()
        self.Update()

        error = []
        
        os.chdir(self.path)
        if os.access(self.tab + ".cmt", 1):
            os.remove(self.tab + ".cmt")
                
        con = sqlite.connect(self.tab + '.cmt')
        cur = con.cursor()

        cur.execute("create table commentary(id INTEGER PRIMARY KEY, ref, comment)")
        cur.execute("create table reference(ref)")
        cur.execute("create table title(abbr, desc, info)")

        addm = '<center>This <b>Bible Analyzer</b> Commentary module was made by a third party.<hr></center><br><br>'
        
        desc = addm + self.info

        cur.execute('insert into title values (?, ?, ?)', (self.tab, self.title, desc))
        
        cyc = 1
        for line in self.lines:
            line = line.encode('utf-8')
            if '\t' in line:
                idx = line.find('\t')
                cur.execute('insert into reference values (?)', (line[:idx],))
                cur.execute('insert into commentary values (null, ?, ?)', (line[:idx], line[idx+1:],))
            else:
                error.append(str(cyc))
                continue
            
            cyc += 1

        con.commit()
        
        self.progLabel.SetLabel('%s Commentary Module Complete!' % self.tab) 
        self.fSizer.Layout()
        
        if error:
            self.errorDlg(error)

    
    def MakeDev(self):
        self.progLabel.SetLabel('Making Devotion Module...')
        self.fSizer.Layout()
        self.Update()
        
        os.chdir(self.path)
        if os.access(self.tab + ".dev", 1):
            os.remove(self.tab + ".dev")
        
        con = sqlite.connect(self.tab + '.dev')
        cur = con.cursor()

        cur.execute("create table devotion(id INTEGER PRIMARY KEY, month, day, devotion)")
        cur.execute("create table title(abbr, desc, info)")

        addm = '<center>This <b>Bible Analyzer</b> Devotion module was made by a third party.<hr></center><br><br>'
        desc = addm + self.info

        cur.execute('insert into title values (?, ?, ?)', (self.tab, self.title, desc))

        for line in self.lines:
            line = line.encode('utf-8')
            idx = line.find('\t')
            idx2 = line.rfind('\t')
            cur.execute('insert into devotion values (null, ?, ?, ?)', (line[:idx], line[idx+1:idx2], line[idx2+1:],))

        con.commit()

        self.progLabel.SetLabel('%s Devotions Module Complete!' % self.tab)
        self.fSizer.Layout()
        
    
    def MakeBook(self):
        self.progLabel.SetLabel('Making Book/Article Module...')
        self.fSizer.Layout()
        self.Update()
        
        error = []
        
        os.chdir(self.path)

        fn = os.path.basename(self.fName)
        base, ext = os.path.splitext(fn)

        if os.access(base + '.bk', 1):
            os.remove(base + '.bk')

        con = sqlite.connect(base + '.bk')
        cur = con.cursor()

        cur.execute("create table book(id INTEGER PRIMARY KEY, chapter, text)")
        cur.execute("create table info(abbr, title)")

        title = re.sub(r'<title>|</title>|<TITLE>|</TITLE>', '', self.lines[0], re.I)
        abbr = title[:16] + '...'

        cur.execute('insert into info values (?, ?)', (abbr, title,))

        cyc = 1
        for line in self.lines[1:]:
            line = line.encode('utf-8')
            if '\t' in line:
                idx = line.find('\t')
                cur.execute('insert into book values (null, ?, ?)', (line[:idx], line[idx+1:],))
            else:
                error.append(str(cyc))
                continue
            cyc += 1
        
        con.commit()
        
        self.progLabel.SetLabel('%s Book Module Complete!' % base)
        self.fSizer.Layout()
        
        if error:
            self.errorDlg(error)
    
    
    def errorDlg(self, errorList):
        msg = 'The following lines do not have a tab (Tab key) between the Reference/Topic and the Data:\n\n%s' % '\n'.join(errorList)
        
        dlg = wx.lib.dialogs.ScrolledMessageDialog(self, msg, "Formatting Error")
        dlg.ShowModal()
