import wx, os, re 
import modTools 


class XMLConvert(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        
        self.xmlModType = wx.RadioBox(self, -1, "XML Type", choices=["Zefania     ", "ThML     ", "OSIS"], majorDimension=1, style=wx.RA_SPECIFY_ROWS)
        fileLabel = wx.StaticText(self, -1, 'Select XML Bible file')     
        fileButton = wx.Button(self, -1, "Browse", size=(50, -1))        
        self.xmlFileName = wx.TextCtrl(self, -1, "", size=(170, -1))
        createButton = wx.Button(self, -1, "Create", size=(50, -1))       
        #finishLabel = wx.StaticText(self, -1, 'File will be found in Bible Analyzer folder.')
        finishLabel2 = wx.StaticText(self, -1, '         Inspect formatted text\nthen run through Module Creator.')
        
        fileLabel.SetForegroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DDKSHADOW))
        
        #finishLabel.SetForegroundColour(wx.Colour(0, 0, 100))
        finishLabel2.SetForegroundColour(wx.Colour(0, 0, 100))
        
        self.xmlProgLabel = wx.StaticText(self, -1, '')
        self.xmlProgLabel.SetForegroundColour(wx.Colour(100, 0, 0))
        
        self.xmlSizer = wx.BoxSizer(wx.VERTICAL)
        fileSizer = wx.BoxSizer(wx.HORIZONTAL)
        
        self.SetSizer(self.xmlSizer)
        
        title = modTools.optionsDlg.FMTitlePanel(self, "Create Formatted Text From XML")
        self.xmlSizer.Add(title, 0, wx.EXPAND | wx.ALL, 2)
        
        self.xmlSizer.Add(self.xmlModType, 0, wx.EXPAND | wx.ALL, 5)
        
        self.xmlSizer.Add(fileLabel, 0, wx.ALIGN_CENTER)
        self.xmlSizer.Add((-1, 2))
        
        fileSizer.Add(self.xmlFileName, 0, wx.LEFT, 0)        
        fileSizer.Add(fileButton, 0, wx.LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
        self.xmlSizer.Add(fileSizer, 0, wx.LEFT, 5)

        self.xmlSizer.Add(createButton, 0, wx.wx.ALIGN_CENTER|wx.TOP, 10)        
        self.xmlSizer.Add((-1, 15))        
        self.xmlSizer.Add(self.xmlProgLabel, 0, wx.ALIGN_CENTER, 0)
        #self.xmlSizer.Add(finishLabel, 0, wx.TOP|wx.ALIGN_CENTER, 25)
        self.xmlSizer.Add(finishLabel2, 0, wx.TOP|wx.ALIGN_CENTER, 5)
        
        fileButton.Bind(wx.EVT_BUTTON, self.OnXmlFile)
        createButton.Bind(wx.EVT_BUTTON, self.OnXmlCreate)
        
        #self.xmlModType.EnableItem(0, False)
        self.xmlModType.EnableItem(1, False)
        self.xmlModType.EnableItem(2, False)    
        #createButton.Enable(False)
        #fileButton.Enable(False)
        
        return 
    
    
    def OnXmlFile(self, event):
        wildcard = "Xml File (*.xml)|*.xml|" "All files (*.*)|*.*"
        dlg = wx.FileDialog(self, message="Choose a file", defaultDir=os.getcwd(), 
            defaultFile="", wildcard=wildcard, style=wx.OPEN | wx.CHANGE_DIR)

        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPaths()
            self.xmlFileName.SetValue(path[0])

        dlg.Destroy()        
        
    
    def OnXmlCreate(self, event):
        if self.xmlFileName.IsEmpty():
            return
        
        type = self.xmlModType.GetSelection()
                
        wx.BeginBusyCursor()
     
        if type == 0:            
            try:
                self.convertZef()
            except:
                wx.EndBusyCursor()
                dlg = wx.MessageDialog(self, 'The converter encountered an error and has been aborted. ',
                                             'Formatting Error', wx.OK | wx.ICON_INFORMATION	)
                dlg.ShowModal()
                dlg.Destroy()
                return
                        
        elif type == 1:
            pass
        elif type == 2:
            pass
        
        wx.EndBusyCursor()
        
        
    def convertZef(self):
        self.xmlProgLabel.SetLabel('Converting Zefania XML file...')
        self.xmlSizer.Layout()
        self.Update()
        
        fName = self.xmlFileName.GetValue()
        
        pth, fn = os.path.split(fName)
        fn, ext = os.path.splitext(fn)
        
        print pth

        txtRead = file(fName).read()
        txtLines = file(fName).readlines()           

        verses = []

        book = ['Gen', 'Exo', 'Lev', 'Num', 'Deu', 'Jos', 'Jdg', 'Rth', '1Sa', '2Sa', '1Ki', '2Ki', '1Ch', '2Ch',
            'Ezr', 'Neh', 'Est', 'Job', 'Psa', 'Pro', 'Ecc', 'Son', 'Isa', 'Jer', 'Lam', 'Eze', 'Dan', 'Hos',
            'Joe', 'Amo', 'Oba', 'Jon', 'Mic', 'Nah', 'Hab', 'Zep', 'Hag', 'Zec', 'Mal', 'Mat', 'Mar', 'Luk',
            'Joh', 'Act', 'Rom', '1Co', '2Co', 'Gal', 'Eph', 'Phi', 'Col', '1Th', '2Th', '1Ti', '2Ti', 'Tit',
            'Phm', 'Heb', 'Jam', '1Pe', '2Pe', '1Jo', '2Jo', '3Jo', 'Jud', 'Rev']

        if '<gr str=' in txtRead:
            strongs = 1
        else:
            strongs = 0

        # Extract verse info and make it in proper format
        for line in txtLines:
            
            # Find current book
            cb = re.search('<BIBLEBOOK bnumber="(\d+)"', line)
            if cb:
                curBook = cb
                
            # Find current chapter
            cc = re.search('<CHAPTER cnumber="(\d+)"', line)
            if cc:
                curChap = cc

            # Find verse ref and verse body
            rawVerse = re.search(r'<VERS vnumber="(\d+)">(.*?)</VERS>', line)
            if rawVerse:
                # Replace italics tags
                vs = re.sub(r'<STYLE css="color:#808080;font-style:italic"> (.*?) </STYLE>', r'<i>\1</i>', rawVerse.group(2)) 
                vs = re.sub(r'<STYLE css="font-style:italic"> ?(.*?) ?</STYLE>', r'<i>\1</i>', vs) 

                # Format Strong's numbers if available
                if strongs == 1:
                    vs = re.sub(r'\*(\d)', r'\1', vs)
                    vs = re.sub(r' <gr str="(\d+)"> </gr>', r'[\1] ', vs)
                    vs = re.sub(r'<gr str="(\d+)">(.+?)( ?)</gr>', r'\2[\1]\3', vs)
                    vs = re.sub(r'<gr str="(\d+) (\d+)">(.+?)( ?)</gr>', r'\3[\1] [\2]\4', vs)
                    vs = re.sub(r'<gr str="(\d+) (\d+) (\d+)">(.+?)( ?)</gr>', r'\4[\1] [\2] [\3]\5', vs)
                    vs = re.sub(r'<gr str="(\d+) (\d+) (\d+) (\d+)">(.+?)( ?)</gr>', r'\5[\1] [\2] [\3] [\4]\6', vs)
                    vs = re.sub(r'<gr str="(\d+) (\d+) (\d+) (\d+) (\d+)">(.+?)( ?)</gr>', r'\6[\1] [\2] [\3] [\4] [\5]\7', vs)
                    vs = re.sub(r'<gr str="(\d+) (\d+) (\d+) (\d+) (\d+) (\d+)">(.+?)( ?)</gr>', r'\7[\1] [\2] [\3] [\4] [\5] [\6]\8', vs)
                    
                    vs = re.sub(r'(\[\d+\])([.,;:?!-])', r'\2\1', vs)
                    vs = re.sub(r'] ([.,;:?!-])', r']\1 ', vs)
                
                # Convert red letters
                redComp = re.compile(r'<STYLE css="color:#ff0000"> ?', re.I) 
                vs = redComp.sub(r'<r>', vs)
                vs = re.sub(r'</STYLE>', r'</r>', vs) 
                vs = re.sub(r'</r> *<r>', r'', vs) 
                
                # Remove DIV and NOTE tags and all between
                vs = re.sub(r'<DIV>.*?</DIV>', r'', vs) 
                
                # Remove refs in parenth.
                vs = re.sub(r'\(\w+ \d+[^\)]+\)', '', vs)

                # Put it all together
                vs = '%s %s:%s\t%s' % (book[int(curBook.group(1))-1], curChap.group(1), rawVerse.group(1), vs)
                
                # Append line to list
                verses.append(vs)

        fileOut = file(fn + ".txt", 'w')
        fileOut.write('\n'.join(verses))
        fileOut.close()        
        
        self.xmlProgLabel.SetLabel('Conversion Complete!')
        self.xmlSizer.Layout()
        self.Update()
        
